@@ -16,7 +16,9 @@
 
 import os
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import importutils
 import webob.dec
 import webob.exc
 
@@ -24,9 +26,7 @@ import cinder.api.openstack
 from cinder.api.openstack import wsgi
 from cinder.api import xmlutil
 from cinder import exception
-from cinder.i18n import _
-from cinder.openstack.common import importutils
-from cinder.openstack.common import log as logging
+from cinder.i18n import _LE, _LI, _LW
 import cinder.policy
 
 
@@ -181,7 +181,7 @@ class ExtensionManager(object):
     """
 
     def __init__(self):
-        LOG.info(_('Initializing extension manager.'))
+        LOG.info(_LI('Initializing extension manager.'))
 
         self.cls_list = CONF.osapi_volume_extension
         self.extensions = {}
@@ -196,7 +196,7 @@ class ExtensionManager(object):
             return
 
         alias = ext.alias
-        LOG.info(_('Loaded extension: %s'), alias)
+        LOG.info(_LI('Loaded extension: %s'), alias)
 
         if alias in self.extensions:
             raise exception.Error("Found duplicate extension: %s" % alias)
@@ -240,8 +240,8 @@ class ExtensionManager(object):
                       ' '.join(extension.__doc__.strip().split()))
             LOG.debug('Ext namespace: %s', extension.namespace)
             LOG.debug('Ext updated: %s', extension.updated)
-        except AttributeError as ex:
-            LOG.exception(_("Exception loading extension: %s"), unicode(ex))
+        except AttributeError:
+            LOG.exception(_LE("Exception loading extension."))
             return False
 
         return True
@@ -269,26 +269,13 @@ class ExtensionManager(object):
 
         extensions = list(self.cls_list)
 
-        # NOTE(thingee): Backwards compat for the old extension loader path.
-        # We can drop this post-grizzly in the H release.
-        old_contrib_path = ('cinder.api.openstack.volume.contrib.'
-                            'standard_extensions')
-        new_contrib_path = 'cinder.api.contrib.standard_extensions'
-        if old_contrib_path in extensions:
-            LOG.warn(_('osapi_volume_extension is set to deprecated path: %s'),
-                     old_contrib_path)
-            LOG.warn(_('Please set your flag or cinder.conf settings for '
-                       'osapi_volume_extension to: %s'), new_contrib_path)
-            extensions = [e.replace(old_contrib_path, new_contrib_path)
-                          for e in extensions]
-
         for ext_factory in extensions:
             try:
                 self.load_extension(ext_factory)
             except Exception as exc:
-                LOG.warn(_('Failed to load extension %(ext_factory)s: '
-                           '%(exc)s'),
-                         {'ext_factory': ext_factory, 'exc': exc})
+                LOG.warning(_LW('Failed to load extension %(ext_factory)s: '
+                                '%(exc)s'),
+                            {'ext_factory': ext_factory, 'exc': exc})
 
 
 class ControllerExtension(object):
@@ -355,9 +342,9 @@ def load_standard_extensions(ext_mgr, logger, path, package, ext_list=None):
             try:
                 ext_mgr.load_extension(classpath)
             except Exception as exc:
-                logger.warn(_('Failed to load extension %(classpath)s: '
-                              '%(exc)s'),
-                            {'classpath': classpath, 'exc': exc})
+                logger.warning(_LW('Failed to load extension %(classpath)s: '
+                                   '%(exc)s'),
+                               {'classpath': classpath, 'exc': exc})
 
         # Now, let's consider any subdirectories we may have...
         subdirs = []
@@ -380,21 +367,24 @@ def load_standard_extensions(ext_mgr, logger, path, package, ext_list=None):
                 try:
                     ext(ext_mgr)
                 except Exception as exc:
-                    logger.warn(_('Failed to load extension %(ext_name)s: '
-                                  '%(exc)s'),
-                                {'ext_name': ext_name, 'exc': exc})
+                    logger.warning(_LW('Failed to load extension '
+                                       '%(ext_name)s: %(exc)s'),
+                                   {'ext_name': ext_name, 'exc': exc})
 
         # Update the list of directories we'll explore...
         dirnames[:] = subdirs
 
 
 def extension_authorizer(api_name, extension_name):
-    def authorize(context, target=None):
+    def authorize(context, target=None, action=None):
         if target is None:
             target = {'project_id': context.project_id,
                       'user_id': context.user_id}
-        action = '%s_extension:%s' % (api_name, extension_name)
-        cinder.policy.enforce(context, action, target)
+        if action is None:
+            act = '%s_extension:%s' % (api_name, extension_name)
+        else:
+            act = '%s_extension:%s:%s' % (api_name, extension_name, action)
+        cinder.policy.enforce(context, act, target)
     return authorize
 
 
