@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
+
 from cinder.api import common
-from cinder.openstack.common import log as logging
 
 
 LOG = logging.getLogger(__name__)
@@ -29,13 +30,15 @@ class ViewBuilder(common.ViewBuilder):
         """Initialize view builder."""
         super(ViewBuilder, self).__init__()
 
-    def summary_list(self, request, volumes):
+    def summary_list(self, request, volumes, volume_count):
         """Show a list of volumes without many details."""
-        return self._list_view(self.summary, request, volumes)
+        return self._list_view(self.summary, request, volumes,
+                               volume_count)
 
-    def detail_list(self, request, volumes):
+    def detail_list(self, request, volumes, volume_count):
         """Detailed view of a list of volumes."""
         return self._list_view(self.detail, request, volumes,
+                               volume_count,
                                coll_name=self._collection_name + '/detail')
 
     def summary(self, request, volume):
@@ -70,7 +73,8 @@ class ViewBuilder(common.ViewBuilder):
                 'bootable': str(volume.get('bootable')).lower(),
                 'encrypted': self._is_volume_encrypted(volume),
                 'replication_status': volume.get('replication_status'),
-                'consistencygroup_id': volume.get('consistencygroup_id')
+                'consistencygroup_id': volume.get('consistencygroup_id'),
+                'multiattach': volume.get('multiattach')
             }
         }
 
@@ -83,19 +87,17 @@ class ViewBuilder(common.ViewBuilder):
         attachments = []
 
         if volume['attach_status'] == 'attached':
-            d = {}
-            volume_id = volume['id']
-
-            # note(justinsb): we use the volume id as the id of the attachments
-            # object
-            d['id'] = volume_id
-
-            d['volume_id'] = volume_id
-            d['server_id'] = volume['instance_uuid']
-            d['host_name'] = volume['attached_host']
-            if volume.get('mountpoint'):
-                d['device'] = volume['mountpoint']
-            attachments.append(d)
+            attaches = volume.get('volume_attachment', [])
+            for attachment in attaches:
+                if attachment.get('attach_status') == 'attached':
+                    a = {'id': attachment.get('volume_id'),
+                         'attachment_id': attachment.get('id'),
+                         'volume_id': attachment.get('volume_id'),
+                         'server_id': attachment.get('instance_uuid'),
+                         'host_name': attachment.get('attached_host'),
+                         'device': attachment.get('mountpoint'),
+                         }
+                    attachments.append(a)
 
         return attachments
 
@@ -117,12 +119,14 @@ class ViewBuilder(common.ViewBuilder):
         else:
             return volume['volume_type_id']
 
-    def _list_view(self, func, request, volumes, coll_name=_collection_name):
+    def _list_view(self, func, request, volumes, volume_count,
+                   coll_name=_collection_name):
         """Provide a view for a list of volumes.
 
         :param func: Function used to format the volume data
         :param request: API request
-        :param servers: List of volumes in dictionary format
+        :param volumes: List of volumes in dictionary format
+        :param volume_count: Length of the original list of volumes
         :param coll_name: Name of collection, used to generate the next link
                           for a pagination query
         :returns: Volume data in dictionary format
@@ -130,7 +134,8 @@ class ViewBuilder(common.ViewBuilder):
         volumes_list = [func(request, volume)['volume'] for volume in volumes]
         volumes_links = self._get_collection_links(request,
                                                    volumes,
-                                                   coll_name)
+                                                   coll_name,
+                                                   volume_count)
         volumes_dict = dict(volumes=volumes_list)
 
         if volumes_links:
