@@ -17,7 +17,7 @@
 import datetime
 
 from lxml import etree
-from oslo.config import cfg
+from oslo_config import cfg
 
 from cinder.api.openstack import wsgi
 from cinder.api.views import versions as views_versions
