@@ -49,15 +49,15 @@ import subprocess
 import time
 
 import eventlet
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import encodeutils
+from oslo_utils import excutils
+from oslo_utils import units
 
-from cinder.backup.driver import BackupDriver
+from cinder.backup import driver
 from cinder import exception
-from cinder.i18n import _
-from cinder.openstack.common import excutils
-from cinder.openstack.common import log as logging
-from cinder.openstack.common import strutils
-from cinder.openstack.common import units
+from cinder.i18n import _, _LE, _LI, _LW
 from cinder import utils
 import cinder.volume.drivers.rbd as rbd_driver
 
@@ -103,7 +103,7 @@ class VolumeMetadataBackup(object):
 
     @property
     def name(self):
-        return strutils.safe_encode("backup.%s.meta" % self._backup_id)
+        return encodeutils.safe_encode("backup.%s.meta" % self._backup_id)
 
     @property
     def exists(self):
@@ -138,8 +138,7 @@ class VolumeMetadataBackup(object):
         """
         meta_obj = rados.Object(self._client.ioctx, self.name)
         if not self._exists(meta_obj):
-            msg = "Metadata backup object %s does not exist" % self.name
-            LOG.debug(msg)
+            LOG.debug("Metadata backup object %s does not exist", self.name)
             return None
 
         return meta_obj.read()
@@ -149,12 +148,11 @@ class VolumeMetadataBackup(object):
         try:
             meta_obj.remove()
         except rados.ObjectNotFound:
-            msg = ("Metadata backup object '%s' not found - ignoring" %
-                   self.name)
-            LOG.debug(msg)
+            LOG.debug("Metadata backup object '%s' not found - ignoring",
+                      self.name)
 
 
-class CephBackupDriver(BackupDriver):
+class CephBackupDriver(driver.BackupDriver):
     """Backup Cinder volumes to Ceph Object Store.
 
     This class enables backing up Cinder volumes to a Ceph object store.
@@ -177,14 +175,14 @@ class CephBackupDriver(BackupDriver):
             self.rbd_stripe_unit = CONF.backup_ceph_stripe_unit
             self.rbd_stripe_count = CONF.backup_ceph_stripe_count
         else:
-            LOG.info(_("RBD striping not supported - ignoring configuration "
-                       "settings for rbd striping"))
+            LOG.info(_LI("RBD striping not supported - ignoring configuration "
+                         "settings for rbd striping"))
             self.rbd_stripe_count = 0
             self.rbd_stripe_unit = 0
 
-        self._ceph_backup_user = strutils.safe_encode(CONF.backup_ceph_user)
-        self._ceph_backup_pool = strutils.safe_encode(CONF.backup_ceph_pool)
-        self._ceph_backup_conf = strutils.safe_encode(CONF.backup_ceph_conf)
+        self._ceph_backup_user = encodeutils.safe_encode(CONF.backup_ceph_user)
+        self._ceph_backup_pool = encodeutils.safe_encode(CONF.backup_ceph_pool)
+        self._ceph_backup_conf = encodeutils.safe_encode(CONF.backup_ceph_conf)
 
     def _validate_string_args(self, *args):
         """Ensure all args are non-None and non-empty."""
@@ -240,7 +238,8 @@ class CephBackupDriver(BackupDriver):
                                   conffile=self._ceph_backup_conf)
         try:
             client.connect()
-            pool_to_open = strutils.safe_encode(pool or self._ceph_backup_pool)
+            pool_to_open = encodeutils.safe_encode(pool or
+                                                   self._ceph_backup_pool)
             ioctx = client.open_ioctx(pool_to_open)
             return client, ioctx
         except self.rados.Error:
@@ -263,13 +262,13 @@ class CephBackupDriver(BackupDriver):
         """
         # Ensure no unicode
         if diff_format:
-            return strutils.safe_encode("volume-%s.backup.base" % volume_id)
+            return encodeutils.safe_encode("volume-%s.backup.base" % volume_id)
         else:
             if backup_id is None:
                 msg = _("Backup id required")
                 raise exception.InvalidParameterValue(msg)
-            return strutils.safe_encode("volume-%s.backup.%s" %
-                                        (volume_id, backup_id))
+            return encodeutils.safe_encode("volume-%s.backup.%s" %
+                                           (volume_id, backup_id))
 
     def _discard_bytes(self, volume, offset, length):
         """Trim length bytes from offset.
@@ -278,7 +277,7 @@ class CephBackupDriver(BackupDriver):
         and pad with zeroes.
         """
         if length:
-            LOG.debug("Discarding %(length)s bytes from offset %(offset)s" %
+            LOG.debug("Discarding %(length)s bytes from offset %(offset)s",
                       {'length': length, 'offset': offset})
             if self._file_is_rbd(volume):
                 volume.rbd_image.discard(offset, length)
@@ -286,7 +285,7 @@ class CephBackupDriver(BackupDriver):
                 zeroes = '\0' * length
                 chunks = int(length / self.chunk_size)
                 for chunk in xrange(0, chunks):
-                    LOG.debug("Writing zeroes chunk %d" % chunk)
+                    LOG.debug("Writing zeroes chunk %d", chunk)
                     volume.write(zeroes)
                     volume.flush()
                     # yield to any other pending backups
@@ -300,11 +299,11 @@ class CephBackupDriver(BackupDriver):
 
     def _transfer_data(self, src, src_name, dest, dest_name, length):
         """Transfer data between files (Python IO objects)."""
-        LOG.debug("Transferring data between '%(src)s' and '%(dest)s'" %
+        LOG.debug("Transferring data between '%(src)s' and '%(dest)s'",
                   {'src': src_name, 'dest': dest_name})
 
         chunks = int(length / self.chunk_size)
-        LOG.debug("%(chunks)s chunks of %(bytes)s bytes to be transferred" %
+        LOG.debug("%(chunks)s chunks of %(bytes)s bytes to be transferred",
                   {'chunks': chunks, 'bytes': self.chunk_size})
 
         for chunk in xrange(0, chunks):
@@ -323,17 +322,18 @@ class CephBackupDriver(BackupDriver):
             dest.flush()
             delta = (time.time() - before)
             rate = (self.chunk_size / delta) / 1024
-            LOG.debug((_("Transferred chunk %(chunk)s of %(chunks)s "
-                         "(%(rate)dK/s)") %
-                       {'chunk': chunk + 1, 'chunks': chunks,
-                        'rate': rate}))
+            LOG.debug("Transferred chunk %(chunk)s of %(chunks)s "
+                      "(%(rate)dK/s)",
+                      {'chunk': chunk + 1,
+                       'chunks': chunks,
+                       'rate': rate})
 
             # yield to any other pending backups
             eventlet.sleep(0)
 
         rem = int(length % self.chunk_size)
         if rem:
-            LOG.debug("Transferring remaining %s bytes" % rem)
+            LOG.debug("Transferring remaining %s bytes", rem)
             data = src.read(rem)
             if data == '':
                 if CONF.restore_discard_excess_bytes:
@@ -349,7 +349,7 @@ class CephBackupDriver(BackupDriver):
 
         This will be the base image used for storing differential exports.
         """
-        LOG.debug("Creating base image '%s'" % name)
+        LOG.debug("Creating base image '%s'", name)
         old_format, features = self._get_rbd_support()
         self.rbd.RBD().create(ioctx=rados_client.ioctx,
                               name=name,
@@ -376,7 +376,7 @@ class CephBackupDriver(BackupDriver):
             snap_name = self._get_backup_snap_name(base_rbd, base_name,
                                                    backup_id)
             if snap_name:
-                LOG.debug("Deleting backup snapshot='%s'" % snap_name)
+                LOG.debug("Deleting backup snapshot='%s'", snap_name)
                 base_rbd.remove_snap(snap_name)
             else:
                 LOG.debug("No backup snapshot to delete")
@@ -416,7 +416,7 @@ class CephBackupDriver(BackupDriver):
 
             base_name = self._get_backup_base_name(volume_id, backup_id)
             LOG.debug("Trying diff format basename='%(basename)s' for "
-                      "backup base image of volume %(volume)s." %
+                      "backup base image of volume %(volume)s.",
                       {'basename': base_name, 'volume': volume_id})
 
         with rbd_driver.RADOSClient(self) as client:
@@ -432,38 +432,39 @@ class CephBackupDriver(BackupDriver):
                 snap, rem = self._delete_backup_snapshot(client, base_name,
                                                          backup_id)
                 if rem:
-                    msg = (_("Backup base image of volume %(volume)s still "
-                             "has %(snapshots)s snapshots so skipping base "
-                             "image delete.") %
-                           {'snapshots': rem, 'volume': volume_id})
-                    LOG.info(msg)
+                    LOG.info(
+                        _LI("Backup base image of volume %(volume)s still "
+                            "has %(snapshots)s snapshots so skipping base "
+                            "image delete."),
+                        {'snapshots': rem, 'volume': volume_id})
                     return
 
-                LOG.info(_("Deleting backup base image='%(basename)s' of "
-                           "volume %(volume)s.") %
+                LOG.info(_LI("Deleting backup base image='%(basename)s' of "
+                             "volume %(volume)s."),
                          {'basename': base_name, 'volume': volume_id})
                 # Delete base if no more snapshots
                 try:
                     self.rbd.RBD().remove(client.ioctx, base_name)
-                except self.rbd.ImageBusy as exc:
+                except self.rbd.ImageBusy:
                     # Allow a retry if the image is busy
                     if retries > 0:
-                        LOG.info(_("Backup image of volume %(volume)s is "
-                                   "busy, retrying %(retries)s more time(s) "
-                                   "in %(delay)ss.") %
+                        LOG.info(_LI("Backup image of volume %(volume)s is "
+                                     "busy, retrying %(retries)s more time(s) "
+                                     "in %(delay)ss."),
                                  {'retries': retries,
                                   'delay': delay,
                                   'volume': volume_id})
                         eventlet.sleep(delay)
                     else:
-                        LOG.error(_("Max retries reached deleting backup "
-                                    "%(basename)s image of volume %(volume)s.")
-                                  % {'volume': volume_id,
-                                     'basename': base_name})
-                        raise exc
+                        LOG.error(_LE("Max retries reached deleting backup "
+                                      "%(basename)s image of volume "
+                                      "%(volume)s."),
+                                  {'volume': volume_id,
+                                   'basename': base_name})
+                        raise
                 else:
                     LOG.debug("Base backup image='%(basename)s' of volume "
-                              "%(volume)s deleted." %
+                              "%(volume)s deleted.",
                               {'basename': base_name, 'volume': volume_id})
                     retries = 0
                 finally:
@@ -471,10 +472,10 @@ class CephBackupDriver(BackupDriver):
 
             # Since we have deleted the base image we can delete the source
             # volume backup snapshot.
-            src_name = strutils.safe_encode(volume_id)
+            src_name = encodeutils.safe_encode(volume_id)
             if src_name in self.rbd.RBD().list(client.ioctx):
                 LOG.debug("Deleting source volume snapshot '%(snapshot)s' "
-                          "for backup %(basename)s." %
+                          "for backup %(basename)s.",
                           {'snapshot': snap, 'basename': base_name})
                 src_rbd = self.rbd.Image(client.ioctx, src_name)
                 try:
@@ -484,14 +485,14 @@ class CephBackupDriver(BackupDriver):
 
     def _piped_execute(self, cmd1, cmd2):
         """Pipe output of cmd1 into cmd2."""
-        LOG.debug("Piping cmd1='%s' into..." % ' '.join(cmd1))
-        LOG.debug("cmd2='%s'" % ' '.join(cmd2))
+        LOG.debug("Piping cmd1='%s' into...", ' '.join(cmd1))
+        LOG.debug("cmd2='%s'", ' '.join(cmd2))
 
         try:
             p1 = subprocess.Popen(cmd1, stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
         except OSError as e:
-            LOG.error(_("Pipe1 failed - %s ") % unicode(e))
+            LOG.error(_LE("Pipe1 failed - %s "), e)
             raise
 
         # NOTE(dosaboy): ensure that the pipe is blocking. This is to work
@@ -505,7 +506,7 @@ class CephBackupDriver(BackupDriver):
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
         except OSError as e:
-            LOG.error(_("Pipe2 failed - %s ") % unicode(e))
+            LOG.error(_LE("Pipe2 failed - %s "), e)
             raise
 
         p1.stdout.close()
@@ -522,7 +523,7 @@ class CephBackupDriver(BackupDriver):
         changed since the snapshot was created.
         """
         LOG.debug("Performing differential transfer from '%(src)s' to "
-                  "'%(dest)s'" %
+                  "'%(dest)s'",
                   {'src': src_name, 'dest': dest_name})
 
         # NOTE(dosaboy): Need to be tolerant of clusters/clients that do
@@ -536,15 +537,15 @@ class CephBackupDriver(BackupDriver):
         if from_snap is not None:
             cmd1.extend(['--from-snap', from_snap])
         if src_snap:
-            path = strutils.safe_encode("%s/%s@%s" %
-                                        (src_pool, src_name, src_snap))
+            path = encodeutils.safe_encode("%s/%s@%s" %
+                                           (src_pool, src_name, src_snap))
         else:
-            path = strutils.safe_encode("%s/%s" % (src_pool, src_name))
+            path = encodeutils.safe_encode("%s/%s" % (src_pool, src_name))
 
         cmd1.extend([path, '-'])
 
         cmd2 = ['rbd', 'import-diff'] + dest_ceph_args
-        rbd_path = strutils.safe_encode("%s/%s" % (dest_pool, dest_name))
+        rbd_path = encodeutils.safe_encode("%s/%s" % (dest_pool, dest_name))
         cmd2.extend(['-', rbd_path])
 
         ret, stderr = self._piped_execute(cmd1, cmd2)
@@ -559,13 +560,11 @@ class CephBackupDriver(BackupDriver):
         """Return tuple (exists, name)."""
         rbds = self.rbd.RBD().list(client.ioctx)
         if name not in rbds:
-            msg = "Image '%s' not found - trying diff format name" % name
-            LOG.debug(msg)
+            LOG.debug("Image '%s' not found - trying diff format name", name)
             if try_diff_format:
                 name = self._get_backup_base_name(volume_id, diff_format=True)
                 if name not in rbds:
-                    msg = "Diff format image '%s' not found" % name
-                    LOG.debug(msg)
+                    LOG.debug("Diff format image '%s' not found", name)
                     return False, name
             else:
                 return False, name
@@ -600,7 +599,7 @@ class CephBackupDriver(BackupDriver):
         # Identify our --from-snap point (if one exists)
         from_snap = self._get_most_recent_snap(source_rbd_image)
         LOG.debug("Using --from-snap '%(snap)s' for incremental backup of "
-                  "volume %(volume)s." %
+                  "volume %(volume)s.",
                   {'snap': from_snap, 'volume': volume_id})
 
         base_name = self._get_backup_base_name(volume_id, diff_format=True)
@@ -617,7 +616,7 @@ class CephBackupDriver(BackupDriver):
                 # ignore it since it is stale and waiting to be cleaned up.
                 if from_snap:
                     LOG.debug("Source snapshot '%(snapshot)s' of volume "
-                              "%(volume)s is stale so deleting." %
+                              "%(volume)s is stale so deleting.",
                               {'snapshot': from_snap, 'volume': volume_id})
                     source_rbd_image.remove_snap(from_snap)
                     from_snap = None
@@ -640,7 +639,7 @@ class CephBackupDriver(BackupDriver):
 
         # Snapshot source volume so that we have a new point-in-time
         new_snap = self._get_new_snap_name(backup_id)
-        LOG.debug("Creating backup snapshot='%s'" % new_snap)
+        LOG.debug("Creating backup snapshot='%s'", new_snap)
         source_rbd_image.create_snap(new_snap)
 
         # Attempt differential backup. If this fails, perhaps because librbd
@@ -660,7 +659,7 @@ class CephBackupDriver(BackupDriver):
                                     src_snap=new_snap,
                                     from_snap=from_snap)
 
-            LOG.debug("Differential backup transfer completed in %.4fs" %
+            LOG.debug("Differential backup transfer completed in %.4fs",
                       (time.time() - before))
 
             # We don't need the previous snapshot (if there was one) anymore so
@@ -679,7 +678,7 @@ class CephBackupDriver(BackupDriver):
 
                 # Delete snapshot
                 LOG.debug("Deleting diff backup snapshot='%(snapshot)s' of "
-                          "source volume='%(volume)s'." %
+                          "source volume='%(volume)s'.",
                           {'snapshot': new_snap, 'volume': volume_id})
                 source_rbd_image.remove_snap(new_snap)
 
@@ -700,8 +699,8 @@ class CephBackupDriver(BackupDriver):
             # First create base backup image
             old_format, features = self._get_rbd_support()
             LOG.debug("Creating backup base image='%(name)s' for volume "
-                      "%(volume)s."
-                      % {'name': backup_name, 'volume': volume_id})
+                      "%(volume)s.",
+                      {'name': backup_name, 'volume': volume_id})
             self.rbd.RBD().create(ioctx=client.ioctx,
                                   name=backup_name,
                                   size=length,
@@ -710,7 +709,7 @@ class CephBackupDriver(BackupDriver):
                                   stripe_unit=self.rbd_stripe_unit,
                                   stripe_count=self.rbd_stripe_count)
 
-            LOG.debug("Copying data from volume %s." % volume_id)
+            LOG.debug("Copying data from volume %s.", volume_id)
             dest_rbd = self.rbd.Image(client.ioctx, backup_name)
             try:
                 rbd_meta = rbd_driver.RBDImageMetadata(dest_rbd,
@@ -757,8 +756,8 @@ class CephBackupDriver(BackupDriver):
         return backup_snaps
 
     def _get_new_snap_name(self, backup_id):
-        return strutils.safe_encode("backup.%s.snap.%s" %
-                                    (backup_id, time.time()))
+        return encodeutils.safe_encode("backup.%s.snap.%s" %
+                                       (backup_id, time.time()))
 
     def _get_backup_snap_name(self, rbd_image, name, backup_id):
         """Return the name of the snapshot associated with backup_id.
@@ -771,17 +770,17 @@ class CephBackupDriver(BackupDriver):
         """
         snaps = self.get_backup_snaps(rbd_image)
 
-        LOG.debug("Looking for snapshot of backup base '%s'" % name)
+        LOG.debug("Looking for snapshot of backup base '%s'", name)
 
         if not snaps:
-            LOG.debug("Backup base '%s' has no snapshots" % name)
+            LOG.debug("Backup base '%s' has no snapshots", name)
             return None
 
         snaps = [snap['name'] for snap in snaps
                  if snap['backup_id'] == backup_id]
 
         if not snaps:
-            LOG.debug("Backup '%s' has no snapshot" % backup_id)
+            LOG.debug("Backup '%s' has no snapshot", backup_id)
             return None
 
         if len(snaps) > 1:
@@ -790,7 +789,7 @@ class CephBackupDriver(BackupDriver):
             LOG.error(msg)
             raise exception.BackupOperationError(msg)
 
-        LOG.debug("Found snapshot '%s'" % (snaps[0]))
+        LOG.debug("Found snapshot '%s'", snaps[0])
         return snaps[0]
 
     def _get_most_recent_snap(self, rbd_image):
@@ -826,11 +825,11 @@ class CephBackupDriver(BackupDriver):
         """
         json_meta = self.get_metadata(backup['volume_id'])
         if not json_meta:
-            LOG.debug("No metadata to backup for volume %s." %
+            LOG.debug("No metadata to backup for volume %s.",
                       backup['volume_id'])
             return
 
-        LOG.debug("Backing up metadata for volume %s." %
+        LOG.debug("Backing up metadata for volume %s.",
                   backup['volume_id'])
         try:
             with rbd_driver.RADOSClient(self) as client:
@@ -852,7 +851,7 @@ class CephBackupDriver(BackupDriver):
         volume_id = volume['id']
         volume_name = volume['name']
 
-        LOG.debug("Starting backup of volume='%s'." % volume_id)
+        LOG.debug("Starting backup of volume='%s'.", volume_id)
 
         # Ensure we are at the beginning of the volume
         volume_file.seek(0)
@@ -865,7 +864,7 @@ class CephBackupDriver(BackupDriver):
                 self._backup_rbd(backup_id, volume_id, volume_file,
                                  volume_name, length)
             except exception.BackupRBDOperationFailed:
-                LOG.debug("Forcing full backup of volume %s." % volume_id)
+                LOG.debug("Forcing full backup of volume %s.", volume_id)
                 do_full_backup = True
         else:
             do_full_backup = True
@@ -885,8 +884,8 @@ class CephBackupDriver(BackupDriver):
                     # Cleanup.
                     self.delete(backup)
 
-        LOG.debug("Backup '%(backup_id)s' of volume %(volume_id)s finished."
-                  % {'backup_id': backup_id, 'volume_id': volume_id})
+        LOG.debug("Backup '%(backup_id)s' of volume %(volume_id)s finished.",
+                  {'backup_id': backup_id, 'volume_id': volume_id})
 
     def _full_restore(self, backup_id, volume_id, dest_file, dest_name,
                       length, src_snap=None):
@@ -932,7 +931,7 @@ class CephBackupDriver(BackupDriver):
         with rbd_driver.RADOSClient(self, self._ceph_backup_pool) as client:
             adjust_size = 0
             base_image = self.rbd.Image(client.ioctx,
-                                        strutils.safe_encode(backup_base),
+                                        encodeutils.safe_encode(backup_base),
                                         read_only=True)
             try:
                 if restore_length != base_image.size():
@@ -942,8 +941,8 @@ class CephBackupDriver(BackupDriver):
 
         if adjust_size:
             with rbd_driver.RADOSClient(self, src_pool) as client:
-                dest_image = self.rbd.Image(client.ioctx,
-                                            strutils.safe_encode(restore_vol))
+                restore_vol_encode = encodeutils.safe_encode(restore_vol)
+                dest_image = self.rbd.Image(client.ioctx, restore_vol_encode)
                 try:
                     LOG.debug("Adjusting restore vol size")
                     dest_image.resize(adjust_size)
@@ -958,7 +957,7 @@ class CephBackupDriver(BackupDriver):
         rbd_conf = restore_file.rbd_conf
 
         LOG.debug("Attempting incremental restore from base='%(base)s' "
-                  "snap='%(snap)s'" %
+                  "snap='%(snap)s'",
                   {'base': base_name, 'snap': restore_point})
         before = time.time()
         try:
@@ -969,8 +968,8 @@ class CephBackupDriver(BackupDriver):
                                     dest_user=rbd_user, dest_conf=rbd_conf,
                                     src_snap=restore_point)
         except exception.BackupRBDOperationFailed:
-            LOG.exception(_("Differential restore failed, trying full "
-                            "restore"))
+            LOG.exception(_LE("Differential restore failed, trying full "
+                              "restore"))
             raise
 
         # If the volume we are restoring to is larger than the backup volume,
@@ -980,7 +979,7 @@ class CephBackupDriver(BackupDriver):
         self._check_restore_vol_size(base_name, restore_name, restore_length,
                                      rbd_pool)
 
-        LOG.debug("Restore transfer completed in %.4fs" %
+        LOG.debug("Restore transfer completed in %.4fs",
                   (time.time() - before))
 
     def _num_backup_snaps(self, backup_base_name):
@@ -1029,7 +1028,7 @@ class CephBackupDriver(BackupDriver):
         rbd_volume.diff_iterate(0, rbd_volume.size(), None, iter_cb)
 
         if extents:
-            LOG.debug("RBD has %s extents" % sum(extents))
+            LOG.debug("RBD has %s extents", sum(extents))
             return True
 
         return False
@@ -1066,9 +1065,8 @@ class CephBackupDriver(BackupDriver):
             # made from, force a full restore since a diff will not work in
             # this case.
             if volume['id'] == backup['volume_id']:
-                msg = ("Destination volume is same as backup source volume "
-                       "%s - forcing full copy." % volume['id'])
-                LOG.debug(msg)
+                LOG.debug("Destination volume is same as backup source volume "
+                          "%s - forcing full copy.", volume['id'])
                 return False, restore_point
 
             if restore_point:
@@ -1082,8 +1080,9 @@ class CephBackupDriver(BackupDriver):
 
                 return True, restore_point
             else:
-                LOG.info(_("No restore point found for backup='%(backup)s' of "
-                           "volume %(volume)s - forcing full copy.") %
+                LOG.info(_LI("No restore point found for "
+                             "backup='%(backup)s' of "
+                             "volume %(volume)s - forcing full copy."),
                          {'backup': backup['id'],
                           'volume': backup['volume_id']})
 
@@ -1115,7 +1114,7 @@ class CephBackupDriver(BackupDriver):
                                        restore_point, length)
                 do_full_restore = False
             except exception.BackupRBDOperationFailed:
-                LOG.debug("Forcing full restore to volume %s." %
+                LOG.debug("Forcing full restore to volume %s.",
                           volume['id'])
 
         if do_full_restore:
@@ -1136,7 +1135,7 @@ class CephBackupDriver(BackupDriver):
                 if meta is not None:
                     self.put_metadata(volume_id, meta)
                 else:
-                    LOG.debug("Volume %s has no backed up metadata." %
+                    LOG.debug("Volume %s has no backed up metadata.",
                               backup['volume_id'])
         except exception.BackupMetadataUnsupportedVersion:
             msg = _("Metadata restore failed due to incompatible version")
@@ -1150,7 +1149,7 @@ class CephBackupDriver(BackupDriver):
         """
         target_volume = self.db.volume_get(self.context, volume_id)
         LOG.debug('Starting restore from Ceph backup=%(src)s to '
-                  'volume=%(dest)s' %
+                  'volume=%(dest)s',
                   {'src': backup['id'], 'dest': target_volume['name']})
 
         try:
@@ -1167,37 +1166,38 @@ class CephBackupDriver(BackupDriver):
 
             self._restore_metadata(backup, volume_id)
 
-            LOG.debug('Restore to volume %s finished successfully.' %
+            LOG.debug('Restore to volume %s finished successfully.',
                       volume_id)
         except exception.BackupOperationError as e:
-            LOG.error(_('Restore to volume %(volume)s finished with error - '
-                        '%(error)s.') % {'error': e, 'volume': volume_id})
+            LOG.error(_LE('Restore to volume %(volume)s finished with error - '
+                          '%(error)s.'), {'error': e, 'volume': volume_id})
             raise
 
     def delete(self, backup):
         """Delete the given backup from Ceph object store."""
-        LOG.debug('Delete started for backup=%s' % backup['id'])
+        LOG.debug('Delete started for backup=%s', backup['id'])
 
         delete_failed = False
         try:
             self._try_delete_base_image(backup['id'], backup['volume_id'])
         except self.rbd.ImageNotFound:
-            msg = (_("RBD image for backup %(backup)s of volume %(volume)s "
-                     "not found. Deleting backup metadata.")
-                   % {'backup': backup['id'], 'volume': backup['volume_id']})
-            LOG.warning(msg)
+            LOG.warning(
+                _LW("RBD image for backup %(backup)s of volume %(volume)s "
+                    "not found. Deleting backup metadata."),
+                {'backup': backup['id'], 'volume': backup['volume_id']})
             delete_failed = True
 
         with rbd_driver.RADOSClient(self) as client:
             VolumeMetadataBackup(client, backup['id']).remove_if_exists()
 
         if delete_failed:
-            LOG.info(_("Delete of backup '%(backup)s' for volume '%(volume)s' "
-                       "finished with warning.") %
+            LOG.info(_LI("Delete of backup '%(backup)s' "
+                         "for volume '%(volume)s' "
+                         "finished with warning."),
                      {'backup': backup['id'], 'volume': backup['volume_id']})
         else:
             LOG.debug("Delete of backup '%(backup)s' for volume "
-                      "'%(volume)s' finished." %
+                      "'%(volume)s' finished.",
                       {'backup': backup['id'], 'volume': backup['volume_id']})
 
 
