@@ -13,30 +13,63 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import os
+import platform
+import re
 import socket
+import sys
 import time
 
+from oslo_concurrency import lockutils
+from oslo_concurrency import processutils as putils
+from oslo_log import log as logging
+import six
+
 from cinder.brick import exception
 from cinder.brick import executor
 from cinder.brick.initiator import host_driver
 from cinder.brick.initiator import linuxfc
 from cinder.brick.initiator import linuxscsi
 from cinder.brick.remotefs import remotefs
-from cinder.i18n import _
-from cinder.openstack.common import lockutils
-from cinder.openstack.common import log as logging
+from cinder.i18n import _, _LE, _LW
 from cinder.openstack.common import loopingcall
-from cinder.openstack.common import processutils as putils
+
+S390X = "s390x"
+S390 = "s390"
 
 LOG = logging.getLogger(__name__)
 
 synchronized = lockutils.synchronized_with_prefix('brick-')
 DEVICE_SCAN_ATTEMPTS_DEFAULT = 3
+MULTIPATH_ERROR_REGEX = re.compile("\w{3} \d+ \d\d:\d\d:\d\d \|.*$")
 
 
-def get_connector_properties(root_helper, my_ip):
-    """Get the connection properties for all protocols."""
+def _check_multipathd_running(root_helper, enforce_multipath):
+    try:
+        putils.execute('multipathd', 'show', 'status',
+                       run_as_root=True, root_helper=root_helper)
+    except putils.ProcessExecutionError as err:
+        LOG.error(_LE('multipathd is not running: exit code %(err)s'),
+                  {'err': err.exit_code})
+        if enforce_multipath:
+            raise
+        return False
+
+    return True
+
+
+def get_connector_properties(root_helper, my_ip, multipath, enforce_multipath):
+    """Get the connection properties for all protocols.
+
+    When the connector wants to use multipath, multipath=True should be
+    specified. If enforce_multipath=True is specified too, an exception is
+    thrown when multipathd is not running. Otherwise, it falls back to
+    multipath=False and only the first path shown up is used.
+    For the compatibility reason, even if multipath=False is specified,
+    some cinder storage drivers may export the target for multipath, which
+    can be found via sendtargets discovery.
+    """
 
     iscsi = ISCSIConnector(root_helper=root_helper)
     fc = linuxfc.LinuxFibreChannel(root_helper=root_helper)
@@ -53,7 +86,11 @@ def get_connector_properties(root_helper, my_ip):
     wwnns = fc.get_fc_wwnns()
     if wwnns:
         props['wwnns'] = wwnns
-
+    props['multipath'] = (multipath and
+                          _check_multipathd_running(root_helper,
+                                                    enforce_multipath))
+    props['platform'] = platform.machine()
+    props['os_type'] = sys.platform
     return props
 
 
@@ -78,9 +115,10 @@ class InitiatorConnector(executor.Executor):
     def factory(protocol, root_helper, driver=None,
                 execute=putils.execute, use_multipath=False,
                 device_scan_attempts=DEVICE_SCAN_ATTEMPTS_DEFAULT,
+                arch=platform.machine(),
                 *args, **kwargs):
-        """Build a Connector object based upon protocol."""
-        LOG.debug("Factory for %s" % protocol)
+        """Build a Connector object based upon protocol and architecture."""
+        LOG.debug("Factory for %s on %s" % (protocol, arch))
         protocol = protocol.upper()
         if protocol == "ISCSI":
             return ISCSIConnector(root_helper=root_helper,
@@ -97,13 +135,22 @@ class InitiatorConnector(executor.Executor):
                                  device_scan_attempts=device_scan_attempts,
                                  *args, **kwargs)
         elif protocol == "FIBRE_CHANNEL":
-            return FibreChannelConnector(root_helper=root_helper,
-                                         driver=driver,
-                                         execute=execute,
-                                         use_multipath=use_multipath,
-                                         device_scan_attempts=
-                                         device_scan_attempts,
-                                         *args, **kwargs)
+            if arch in (S390, S390X):
+                return FibreChannelConnectorS390X(root_helper=root_helper,
+                                                  driver=driver,
+                                                  execute=execute,
+                                                  use_multipath=use_multipath,
+                                                  device_scan_attempts=
+                                                  device_scan_attempts,
+                                                  *args, **kwargs)
+            else:
+                return FibreChannelConnector(root_helper=root_helper,
+                                             driver=driver,
+                                             execute=execute,
+                                             use_multipath=use_multipath,
+                                             device_scan_attempts=
+                                             device_scan_attempts,
+                                             *args, **kwargs)
         elif protocol == "AOE":
             return AoEConnector(root_helper=root_helper,
                                 driver=driver,
@@ -123,22 +170,29 @@ class InitiatorConnector(executor.Executor):
                                   execute=execute,
                                   device_scan_attempts=device_scan_attempts,
                                   *args, **kwargs)
+        elif protocol == "HUAWEISDSHYPERVISOR":
+            return HuaweiStorHyperConnector(root_helper=root_helper,
+                                            driver=driver,
+                                            execute=execute,
+                                            device_scan_attempts=
+                                            device_scan_attempts,
+                                            *args, **kwargs)
         else:
             msg = (_("Invalid InitiatorConnector protocol "
                      "specified %(protocol)s") %
                    dict(protocol=protocol))
             raise ValueError(msg)
 
-    def check_valid_device(self, path):
+    def check_valid_device(self, path, run_as_root=True):
         cmd = ('dd', 'if=%(path)s' % {"path": path},
                'of=/dev/null', 'count=1')
         out, info = None, None
         try:
-            out, info = self._execute(*cmd, run_as_root=True,
+            out, info = self._execute(*cmd, run_as_root=run_as_root,
                                       root_helper=self._root_helper)
         except putils.ProcessExecutionError as e:
-            LOG.error(_("Failed to access the device on the path "
-                        "%(path)s: %(error)s %(info)s.") %
+            LOG.error(_LE("Failed to access the device on the path "
+                          "%(path)s: %(error)s %(info)s.") %
                       {"path": path, "error": e.stderr,
                        "info": info})
             return False
@@ -183,68 +237,117 @@ class ISCSIConnector(InitiatorConnector):
         super(ISCSIConnector, self).set_execute(execute)
         self._linuxscsi.set_execute(execute)
 
+    def _iterate_all_targets(self, connection_properties):
+        for ip, iqn, lun in self._get_all_targets(connection_properties):
+            props = copy.deepcopy(connection_properties)
+            props['target_portal'] = ip
+            props['target_iqn'] = iqn
+            props['target_lun'] = lun
+            for key in ('target_portals', 'target_iqns', 'target_luns'):
+                props.pop(key, None)
+            yield props
+
+    def _get_all_targets(self, connection_properties):
+        if all([key in connection_properties for key in ('target_portals',
+                                                         'target_iqns',
+                                                         'target_luns')]):
+            return zip(connection_properties['target_portals'],
+                       connection_properties['target_iqns'],
+                       connection_properties['target_luns'])
+
+        return [(connection_properties['target_portal'],
+                 connection_properties['target_iqn'],
+                 connection_properties.get('target_lun', 0))]
+
+    def _discover_iscsi_portals(self, connection_properties):
+        if all([key in connection_properties for key in ('target_portals',
+                                                         'target_iqns')]):
+            # Use targets specified by connection_properties
+            return zip(connection_properties['target_portals'],
+                       connection_properties['target_iqns'])
+
+        # Discover and return every available target
+        out = self._run_iscsiadm_bare(['-m',
+                                       'discovery',
+                                       '-t',
+                                       'sendtargets',
+                                       '-p',
+                                       connection_properties['target_portal']],
+                                      check_exit_code=[0, 255])[0] \
+            or ""
+
+        return self._get_target_portals_from_iscsiadm_output(out)
+
     @synchronized('connect_volume')
     def connect_volume(self, connection_properties):
         """Attach the volume to instance_name.
 
         connection_properties for iSCSI must include:
-        target_portal - ip and optional port
-        target_iqn - iSCSI Qualified Name
-        target_lun - LUN id of the volume
+        target_portal(s) - ip and optional port
+        target_iqn(s) - iSCSI Qualified Name
+        target_lun(s) - LUN id of the volume
+        Note that plural keys may be used when use_multipath=True
         """
 
         device_info = {'type': 'block'}
 
         if self.use_multipath:
-            #multipath installed, discovering other targets if available
-            target_portal = connection_properties['target_portal']
-            out = self._run_iscsiadm_bare(['-m',
-                                           'discovery',
-                                           '-t',
-                                           'sendtargets',
-                                           '-p',
-                                           target_portal],
-                                          check_exit_code=[0, 255])[0] \
-                or ""
-
-            for ip, iqn in self._get_target_portals_from_iscsiadm_output(out):
-                props = connection_properties.copy()
+            # multipath installed, discovering other targets if available
+            for ip, iqn in self._discover_iscsi_portals(connection_properties):
+                props = copy.deepcopy(connection_properties)
                 props['target_portal'] = ip
                 props['target_iqn'] = iqn
                 self._connect_to_iscsi_portal(props)
 
             self._rescan_iscsi()
+            host_devices = self._get_device_path(connection_properties)
         else:
-            self._connect_to_iscsi_portal(connection_properties)
+            target_props = connection_properties
+            for props in self._iterate_all_targets(connection_properties):
+                if self._connect_to_iscsi_portal(props):
+                    target_props = props
+                    break
+                else:
+                    LOG.warn(_LW(
+                        'Failed to login to any of the iSCSI targets.'))
 
-        host_device = self._get_device_path(connection_properties)
+            host_devices = self._get_device_path(target_props)
 
         # The /dev/disk/by-path/... node is not always present immediately
         # TODO(justinsb): This retry-with-delay is a pattern, move to utils?
         tries = 0
-        while not os.path.exists(host_device):
+        # Loop until at least 1 path becomes available
+        while all(map(lambda x: not os.path.exists(x), host_devices)):
             if tries >= self.device_scan_attempts:
-                raise exception.VolumeDeviceNotFound(device=host_device)
+                raise exception.VolumeDeviceNotFound(device=host_devices)
 
-            LOG.warn(_("ISCSI volume not yet found at: %(host_device)s. "
-                       "Will rescan & retry.  Try number: %(tries)s"),
-                     {'host_device': host_device,
+            LOG.warn(_LW("ISCSI volume not yet found at: %(host_devices)s. "
+                         "Will rescan & retry.  Try number: %(tries)s"),
+                     {'host_devices': host_devices,
                       'tries': tries})
 
             # The rescan isn't documented as being necessary(?), but it helps
-            self._run_iscsiadm(connection_properties, ("--rescan",))
+            if self.use_multipath:
+                self._rescan_iscsi()
+            else:
+                self._run_iscsiadm(target_props, ("--rescan",))
 
             tries = tries + 1
-            if not os.path.exists(host_device):
+            if all(map(lambda x: not os.path.exists(x), host_devices)):
                 time.sleep(tries ** 2)
+            else:
+                break
 
         if tries != 0:
-            LOG.debug("Found iSCSI node %(host_device)s "
+            LOG.debug("Found iSCSI node %(host_devices)s "
                       "(after %(tries)s rescans)",
-                      {'host_device': host_device, 'tries': tries})
+                      {'host_devices': host_devices, 'tries': tries})
+
+        # Choose an accessible host device
+        host_device = next(dev for dev in host_devices if os.path.exists(dev))
 
         if self.use_multipath:
-            #we use the multipath device instead of the single path device
+            # we use the multipath device instead of the single path device
             self._rescan_multipath()
             multipath_device = self._get_multipath_device_name(host_device)
             if multipath_device is not None:
@@ -258,9 +361,9 @@ class ISCSIConnector(InitiatorConnector):
         """Detach the volume from instance_name.
 
         connection_properties for iSCSI must include:
-        target_portal - IP and optional port
-        target_iqn - iSCSI Qualified Name
-        target_lun - LUN id of the volume
+        target_portal(s) - IP and optional port
+        target_iqn(s) - iSCSI Qualified Name
+        target_lun(s) - LUN id of the volume
         """
         # Moved _rescan_iscsi and _rescan_multipath
         # from _disconnect_volume_multipath_iscsi to here.
@@ -268,23 +371,48 @@ class ISCSIConnector(InitiatorConnector):
         # but before logging out, the removed devices under /dev/disk/by-path
         # will reappear after rescan.
         self._rescan_iscsi()
-        host_device = self._get_device_path(connection_properties)
-        multipath_device = None
         if self.use_multipath:
             self._rescan_multipath()
-            multipath_device = self._get_multipath_device_name(host_device)
+            host_device = multipath_device = None
+            host_devices = self._get_device_path(connection_properties)
+            # Choose an accessible host device
+            for dev in host_devices:
+                if os.path.exists(dev):
+                    host_device = dev
+                    multipath_device = self._get_multipath_device_name(dev)
+                    if multipath_device:
+                        break
+            if not host_device:
+                LOG.error(_LE("No accessible volume device: %(host_devices)s"),
+                          {'host_devices': host_devices})
+                raise exception.VolumeDeviceNotFound(device=host_devices)
+
             if multipath_device:
                 device_realpath = os.path.realpath(host_device)
                 self._linuxscsi.remove_multipath_device(device_realpath)
                 return self._disconnect_volume_multipath_iscsi(
                     connection_properties, multipath_device)
 
+        # When multiple portals/iqns/luns are specified, we need to remove
+        # unused devices created by logging into other LUNs' session.
+        for props in self._iterate_all_targets(connection_properties):
+            self._disconnect_volume_iscsi(props)
+
+    def _disconnect_volume_iscsi(self, connection_properties):
         # remove the device from the scsi subsystem
         # this eliminates any stale entries until logout
+        host_device = self._get_device_path(connection_properties)[0]
         dev_name = self._linuxscsi.get_name_from_path(host_device)
         if dev_name:
             self._linuxscsi.remove_scsi_device(dev_name)
 
+            # NOTE(jdg): On busy systems we can have a race here
+            # where remove_iscsi_device is called before the device file
+            # has actually been removed.  The result is an orphaned
+            # iscsi session that never gets logged out.  The following
+            # call to wait addresses that issue.
+            self._linuxscsi.wait_for_volume_removal(host_device)
+
         # NOTE(vish): Only disconnect from the target if no luns from the
         #             target are in use.
         device_prefix = ("/dev/disk/by-path/ip-%(portal)s-iscsi-%(iqn)s-lun-" %
@@ -298,11 +426,8 @@ class ISCSIConnector(InitiatorConnector):
             self._disconnect_from_iscsi_portal(connection_properties)
 
     def _get_device_path(self, connection_properties):
-        path = ("/dev/disk/by-path/ip-%(portal)s-iscsi-%(iqn)s-lun-%(lun)s" %
-                {'portal': connection_properties['target_portal'],
-                 'iqn': connection_properties['target_iqn'],
-                 'lun': connection_properties.get('target_lun', 0)})
-        return path
+        return ["/dev/disk/by-path/ip-%s-iscsi-%s-lun-%s" % x for x in
+                self._get_all_targets(connection_properties)]
 
     def get_initiator(self):
         """Secure helper to read file as root."""
@@ -362,16 +487,7 @@ class ISCSIConnector(InitiatorConnector):
         # Do a discovery to find all targets.
         # Targets for multiple paths for the same multipath device
         # may not be the same.
-        out = self._run_iscsiadm_bare(['-m',
-                                       'discovery',
-                                       '-t',
-                                       'sendtargets',
-                                       '-p',
-                                      connection_properties['target_portal']],
-                                      check_exit_code=[0, 255])[0] \
-            or ""
-
-        ips_iqns = self._get_target_portals_from_iscsiadm_output(out)
+        ips_iqns = self._discover_iscsi_portals(connection_properties)
 
         if not devices:
             # disconnect if no other multipath devices
@@ -428,8 +544,8 @@ class ISCSIConnector(InitiatorConnector):
                                   "node.session.auth.password",
                                   connection_properties['auth_password'])
 
-        #duplicate logins crash iscsiadm after load,
-        #so we scan active sessions to see if the node is logged in.
+        # duplicate logins crash iscsiadm after load,
+        # so we scan active sessions to see if the node is logged in.
         out = self._run_iscsiadm_bare(["-m", "session"],
                                       run_as_root=True,
                                       check_exit_code=[0, 1, 21])[0] or ""
@@ -450,17 +566,20 @@ class ISCSIConnector(InitiatorConnector):
                                    ("--login",),
                                    check_exit_code=[0, 255])
             except putils.ProcessExecutionError as err:
-                #as this might be one of many paths,
-                #only set successful logins to startup automatically
-                if err.exit_code in [15]:
-                    self._iscsiadm_update(connection_properties,
-                                          "node.startup",
-                                          "automatic")
-                    return
+                # exit_code=15 means the session already exists, so it should
+                # be regarded as successful login.
+                if err.exit_code not in [15]:
+                    LOG.warn(_LW('Failed to login iSCSI target %(iqn)s '
+                                 'on portal %(portal)s (exit code %(err)s).'),
+                             {'iqn': connection_properties['target_iqn'],
+                              'portal': connection_properties['target_portal'],
+                              'err': err.exit_code})
+                    return False
 
             self._iscsiadm_update(connection_properties,
                                   "node.startup",
                                   "automatic")
+        return True
 
     def _disconnect_from_iscsi_portal(self, connection_properties):
         self._iscsiadm_update(connection_properties, "node.startup", "manual",
@@ -476,7 +595,7 @@ class ISCSIConnector(InitiatorConnector):
                                   device],
                                   check_exit_code=[0, 1])[0]
         mpath_line = [line for line in out.splitlines()
-                      if "scsi_id" not in line]  # ignore udev errors
+                      if not re.match(MULTIPATH_ERROR_REGEX, line)]
         if len(mpath_line) > 0 and len(mpath_line[0]) > 0:
             return "/dev/mapper/%s" % mpath_line[0].split(" ")[0]
 
@@ -491,7 +610,7 @@ class ISCSIConnector(InitiatorConnector):
 
     def _disconnect_mpath(self, connection_properties, ips_iqns):
         for ip, iqn in ips_iqns:
-            props = connection_properties.copy()
+            props = copy.deepcopy(connection_properties)
             props['target_portal'] = ip
             props['target_iqn'] = iqn
             self._disconnect_from_iscsi_portal(props)
@@ -536,7 +655,7 @@ class ISCSIConnector(InitiatorConnector):
                                 check_exit_code=[0, 1, 21, 255])
 
     def _rescan_multipath(self):
-        self._run_multipath('-r', check_exit_code=[0, 1, 21])
+        self._run_multipath(['-r'], check_exit_code=[0, 1, 21])
 
 
 class ISERConnector(ISCSIConnector):
@@ -581,31 +700,12 @@ class FibreChannelConnector(InitiatorConnector):
         LOG.debug("execute = %s" % self._execute)
         device_info = {'type': 'block'}
 
-        ports = connection_properties['target_wwn']
-        wwns = []
-        # we support a list of wwns or a single wwn
-        if isinstance(ports, list):
-            for wwn in ports:
-                wwns.append(str(wwn))
-        elif isinstance(ports, basestring):
-            wwns.append(str(ports))
-
-        # We need to look for wwns on every hba
-        # because we don't know ahead of time
-        # where they will show up.
         hbas = self._linuxfc.get_fc_hbas_info()
-        host_devices = []
-        for hba in hbas:
-            pci_num = self._get_pci_num(hba)
-            if pci_num is not None:
-                for wwn in wwns:
-                    target_wwn = "0x%s" % wwn.lower()
-                    host_device = ("/dev/disk/by-path/pci-%s-fc-%s-lun-%s" %
-                                   (pci_num,
-                                    target_wwn,
-                                    connection_properties.get(
-                                        'target_lun', 0)))
-                    host_devices.append(host_device)
+        ports = connection_properties['target_wwn']
+        possible_devs = self._get_possible_devices(hbas, ports)
+
+        lun = connection_properties.get('target_lun', 0)
+        host_devices = self._get_host_devices(possible_devs, lun)
 
         if len(host_devices) == 0:
             # this is empty because we don't have any FC HBAs
@@ -633,8 +733,8 @@ class FibreChannelConnector(InitiatorConnector):
                 LOG.error(msg)
                 raise exception.NoFibreChannelVolumeDeviceFound()
 
-            LOG.warn(_("Fibre volume not yet found. "
-                       "Will rescan & retry.  Try number: %(tries)s"),
+            LOG.warn(_LW("Fibre volume not yet found. "
+                         "Will rescan & retry.  Try number: %(tries)s"),
                      {'tries': tries})
 
             self._linuxfc.rescan_hosts(hbas)
@@ -678,6 +778,50 @@ class FibreChannelConnector(InitiatorConnector):
         device_info['devices'] = devices
         return device_info
 
+    def _get_host_devices(self, possible_devs, lun):
+        host_devices = []
+        for pci_num, target_wwn in possible_devs:
+            host_device = "/dev/disk/by-path/pci-%s-fc-%s-lun-%s" % (
+                pci_num,
+                target_wwn,
+                lun)
+            host_devices.append(host_device)
+        return host_devices
+
+    def _get_possible_devices(self, hbas, wwnports):
+        """Compute the possible valid fibre channel device options.
+
+        :param hbas: available hba devices.
+        :param wwnports: possible wwn addresses. Can either be string
+        or list of strings.
+
+        :returns: list of (pci_id, wwn) tuples
+
+        Given one or more wwn (mac addresses for fibre channel) ports
+        do the matrix math to figure out a set of pci device, wwn
+        tuples that are potentially valid (they won't all be). This
+        provides a search space for the device connection.
+
+        """
+        # the wwn (think mac addresses for fiber channel devices) can
+        # either be a single value or a list. Normalize it to a list
+        # for further operations.
+        wwns = []
+        if isinstance(wwnports, list):
+            for wwn in wwnports:
+                wwns.append(str(wwn))
+        elif isinstance(wwnports, six.string_types):
+            wwns.append(str(wwnports))
+
+        raw_devices = []
+        for hba in hbas:
+            pci_num = self._get_pci_num(hba)
+            if pci_num is not None:
+                for wwn in wwns:
+                    target_wwn = "0x%s" % wwn.lower()
+                    raw_devices.append((pci_num, target_wwn))
+        return raw_devices
+
     @synchronized('connect_volume')
     def disconnect_volume(self, connection_properties, device_info):
         """Detach the volume from instance_name.
@@ -698,6 +842,9 @@ class FibreChannelConnector(InitiatorConnector):
             LOG.debug("devices to remove = %s" % devices)
             self._linuxscsi.flush_multipath_device(multipath_id)
 
+        self._remove_devices(connection_properties, devices)
+
+    def _remove_devices(self, connection_properties, devices):
         # There may have been more than 1 device mounted
         # by the kernel for this volume.  We have to remove
         # all of them
@@ -726,6 +873,69 @@ class FibreChannelConnector(InitiatorConnector):
         return pci_num
 
 
+class FibreChannelConnectorS390X(FibreChannelConnector):
+    """Connector class to attach/detach Fibre Channel volumes on S390X arch."""
+
+    def __init__(self, root_helper, driver=None,
+                 execute=putils.execute, use_multipath=False,
+                 device_scan_attempts=DEVICE_SCAN_ATTEMPTS_DEFAULT,
+                 *args, **kwargs):
+        super(FibreChannelConnectorS390X, self).__init__(root_helper,
+                                                         driver=driver,
+                                                         execute=execute,
+                                                         device_scan_attempts=
+                                                         device_scan_attempts,
+                                                         *args, **kwargs)
+        LOG.debug("Initializing Fibre Channel connector for S390")
+        self._linuxscsi = linuxscsi.LinuxSCSI(root_helper, execute)
+        self._linuxfc = linuxfc.LinuxFibreChannelS390X(root_helper, execute)
+        self.use_multipath = use_multipath
+
+    def set_execute(self, execute):
+        super(FibreChannelConnectorS390X, self).set_execute(execute)
+        self._linuxscsi.set_execute(execute)
+        self._linuxfc.set_execute(execute)
+
+    def _get_host_devices(self, possible_devs, lun):
+        host_devices = []
+        for pci_num, target_wwn in possible_devs:
+            target_lun = self._get_lun_string(lun)
+            host_device = self._get_device_file_path(
+                pci_num,
+                target_wwn,
+                target_lun)
+            self._linuxfc.configure_scsi_device(pci_num, target_wwn,
+                                                target_lun)
+            host_devices.append(host_device)
+        return host_devices
+
+    def _get_lun_string(self, lun):
+        target_lun = 0
+        if lun <= 0xffff:
+            target_lun = "0x%04x000000000000" % lun
+        elif lun <= 0xffffffff:
+            target_lun = "0x%08x00000000" % lun
+        return target_lun
+
+    def _get_device_file_path(self, pci_num, target_wwn, target_lun):
+        host_device = "/dev/disk/by-path/ccw-%s-zfcp-%s:%s" % (
+            pci_num,
+            target_wwn,
+            target_lun)
+        return host_device
+
+    def _remove_devices(self, connection_properties, devices):
+        hbas = self._linuxfc.get_fc_hbas_info()
+        ports = connection_properties['target_wwn']
+        possible_devs = self._get_possible_devices(hbas, ports)
+        lun = connection_properties.get('target_lun', 0)
+        target_lun = self._get_lun_string(lun)
+        for pci_num, target_wwn in possible_devs:
+            self._linuxfc.deconfigure_scsi_device(pci_num,
+                                                  target_wwn,
+                                                  target_lun)
+
+
 class AoEConnector(InitiatorConnector):
     """Connector class to attach/detach AoE volumes."""
     def __init__(self, root_helper, driver=None,
@@ -769,7 +979,7 @@ class AoEConnector(InitiatorConnector):
 
         waiting_status = {'tries': 0}
 
-        #NOTE(jbr_): Device path is not always present immediately
+        # NOTE(jbr_): Device path is not always present immediately
         def _wait_for_discovery(aoe_path):
             if os.path.exists(aoe_path):
                 raise loopingcall.LoopingCallDone
@@ -777,8 +987,8 @@ class AoEConnector(InitiatorConnector):
             if waiting_status['tries'] >= self.device_scan_attempts:
                 raise exception.VolumeDeviceNotFound(device=aoe_path)
 
-            LOG.warn(_("AoE volume not yet found at: %(path)s. "
-                       "Try number: %(tries)s"),
+            LOG.warn(_LW("AoE volume not yet found at: %(path)s. "
+                         "Try number: %(tries)s"),
                      {'path': aoe_device,
                       'tries': waiting_status['tries']})
 
@@ -859,8 +1069,8 @@ class RemoteFsConnector(InitiatorConnector):
                     kwargs.get('glusterfs_mount_point_base') or\
                     mount_point_base
         else:
-            LOG.warn(_("Connection details not present."
-                       " RemoteFsClient may not initialize properly."))
+            LOG.warn(_LW("Connection details not present."
+                         " RemoteFsClient may not initialize properly."))
         self._remotefsclient = remotefs.RemoteFsClient(mount_type, root_helper,
                                                        execute=execute,
                                                        *args, **kwargs)
@@ -927,3 +1137,116 @@ class LocalConnector(InitiatorConnector):
     def disconnect_volume(self, connection_properties, device_info):
         """Disconnect a volume from the local host."""
         pass
+
+
+class HuaweiStorHyperConnector(InitiatorConnector):
+    """"Connector class to attach/detach SDSHypervisor volumes."""
+    attached_success_code = 0
+    has_been_attached_code = 50151401
+    attach_mnid_done_code = 50151405
+    vbs_unnormal_code = 50151209
+    not_mount_node_code = 50155007
+    iscliexist = True
+
+    def __init__(self, root_helper, driver=None, execute=putils.execute,
+                 *args, **kwargs):
+        self.cli_path = os.getenv('HUAWEISDSHYPERVISORCLI_PATH')
+        if not self.cli_path:
+            self.cli_path = '/usr/local/bin/sds/sds_cli'
+            LOG.debug("CLI path is not configured, using default %s."
+                      % self.cli_path)
+        if not os.path.isfile(self.cli_path):
+            self.iscliexist = False
+            LOG.error(_LE('SDS CLI file not found, '
+                          'HuaweiStorHyperConnector init failed.'))
+        super(HuaweiStorHyperConnector, self).__init__(root_helper,
+                                                       driver=driver,
+                                                       execute=execute,
+                                                       *args, **kwargs)
+
+    @synchronized('connect_volume')
+    def connect_volume(self, connection_properties):
+        """Connect to a volume."""
+        LOG.debug("Connect_volume connection properties: %s."
+                  % connection_properties)
+        out = self._attach_volume(connection_properties['volume_id'])
+        if not out or int(out['ret_code']) not in (self.attached_success_code,
+                                                   self.has_been_attached_code,
+                                                   self.attach_mnid_done_code):
+            msg = (_("Attach volume failed, "
+                   "error code is %s") % out['ret_code'])
+            raise exception.BrickException(msg=msg)
+        out = self._query_attached_volume(
+            connection_properties['volume_id'])
+        if not out or int(out['ret_code']) != 0:
+            msg = _("query attached volume failed or volume not attached.")
+            raise exception.BrickException(msg=msg)
+
+        device_info = {'type': 'block',
+                       'path': out['dev_addr']}
+        return device_info
+
+    @synchronized('connect_volume')
+    def disconnect_volume(self, connection_properties, device_info):
+        """Disconnect a volume from the local host."""
+        LOG.debug("Disconnect_volume: %s." % connection_properties)
+        out = self._detach_volume(connection_properties['volume_id'])
+        if not out or int(out['ret_code']) not in (self.attached_success_code,
+                                                   self.vbs_unnormal_code,
+                                                   self.not_mount_node_code):
+            msg = (_("Disconnect_volume failed, "
+                   "error code is %s") % out['ret_code'])
+            raise exception.BrickException(msg=msg)
+
+    def is_volume_connected(self, volume_name):
+        """Check if volume already connected to host"""
+        LOG.debug('Check if volume %s already connected to a host.'
+                  % volume_name)
+        out = self._query_attached_volume(volume_name)
+        if out:
+            return int(out['ret_code']) == 0
+        return False
+
+    def _attach_volume(self, volume_name):
+        return self._cli_cmd('attach', volume_name)
+
+    def _detach_volume(self, volume_name):
+        return self._cli_cmd('detach', volume_name)
+
+    def _query_attached_volume(self, volume_name):
+        return self._cli_cmd('querydev', volume_name)
+
+    def _cli_cmd(self, method, volume_name):
+        LOG.debug("Enter into _cli_cmd.")
+        if not self.iscliexist:
+            msg = _("SDS command line doesn't exist, "
+                    "can't execute SDS command.")
+            raise exception.BrickException(msg=msg)
+        if not method or volume_name is None:
+            return
+        cmd = [self.cli_path, '-c', method, '-v', volume_name]
+        out, clilog = self._execute(*cmd, run_as_root=False,
+                                    root_helper=self._root_helper)
+        analyse_result = self._analyze_output(out)
+        LOG.debug('%(method)s volume returns %(analyse_result)s.'
+                  % {'method': method, 'analyse_result': analyse_result})
+        if clilog:
+            LOG.error(_LE("SDS CLI output some log: %s.")
+                      % clilog)
+        return analyse_result
+
+    def _analyze_output(self, out):
+        LOG.debug("Enter into _analyze_output.")
+        if out:
+            analyse_result = {}
+            out_temp = out.split('\n')
+            for line in out_temp:
+                LOG.debug("Line is %s." % line)
+                if line.find('=') != -1:
+                    key, val = line.split('=', 1)
+                    LOG.debug(key + " = " + val)
+                    if key in ['ret_code', 'ret_desc', 'dev_addr']:
+                        analyse_result[key] = val
+            return analyse_result
+        else:
+            return None
