@@ -19,12 +19,12 @@ import hashlib
 import os
 import re
 
+from oslo_concurrency import processutils as putils
+from oslo_log import log as logging
 import six
 
 from cinder.brick import exception
-from cinder.i18n import _
-from cinder.openstack.common import log as logging
-from cinder.openstack.common import processutils as putils
+from cinder.i18n import _, _LI
 
 LOG = logging.getLogger(__name__)
 
@@ -77,7 +77,7 @@ class RemoteFsClient(object):
                             self._get_hash_str(device_name))
 
     def _read_mounts(self):
-        (out, err) = self._execute('mount', check_exit_code=0)
+        (out, _err) = self._execute('mount', check_exit_code=0)
         lines = out.split('\n')
         mounts = {}
         for line in lines:
@@ -93,7 +93,7 @@ class RemoteFsClient(object):
         mount_path = self.get_mount_point(share)
 
         if mount_path in self._read_mounts():
-            LOG.info(_('Already mounted: %s') % mount_path)
+            LOG.info(_LI('Already mounted: %s') % mount_path)
             return
 
         self._execute('mkdir', '-p', mount_path, check_exit_code=0)
