@@ -17,29 +17,22 @@
 
 """Starter script for Cinder Scheduler."""
 
-import logging as python_logging
-import sys
-
 import eventlet
 eventlet.monkey_patch()
-# Monkey patch the original current_thread to use the up-to-date _active
-# global variable. See https://bugs.launchpad.net/bugs/1863021 and
-# https://github.com/eventlet/eventlet/issues/592
-import __original_module_threading as orig_threading  # pylint: disable=E0401
-import threading # noqa
-orig_threading.current_thread.__globals__['_active'] = \
-    threading._active  # type: ignore
+
+import sys
+import warnings
+
+warnings.simplefilter('once', DeprecationWarning)
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_reports import guru_meditation_report as gmr
-from oslo_reports import opts as gmr_opts
+
+from cinder import i18n
+i18n.enable_lazy()
 
 # Need to register global_opts
 from cinder.common import config  # noqa
-from cinder import i18n
-i18n.enable_lazy()
-from cinder import objects
 from cinder import service
 from cinder import utils
 from cinder import version
@@ -48,15 +41,11 @@ from cinder import version
 CONF = cfg.CONF
 
 
-def main() -> None:
-    objects.register_all()
-    gmr_opts.set_defaults(CONF)
+def main():
     CONF(sys.argv[1:], project='cinder',
          version=version.version_string())
     logging.setup(CONF, "cinder")
-    python_logging.captureWarnings(True)
     utils.monkey_patch()
-    gmr.TextGuruMeditation.setup_autorun(version, conf=CONF)
     server = service.Service.create(binary='cinder-scheduler')
     service.serve(server)
     service.wait()
