@@ -13,11 +13,11 @@
 #    under the License.
 
 from migrate import ForeignKeyConstraint
+from oslo_log import log as logging
 from sqlalchemy import Boolean, Column, DateTime
 from sqlalchemy import MetaData, Integer, String, Table, ForeignKey
 
-from cinder.i18n import _
-from cinder.openstack.common import log as logging
+from cinder.i18n import _LE
 
 LOG = logging.getLogger(__name__)
 
@@ -47,7 +47,7 @@ def upgrade(migrate_engine):
     try:
         quota_classes.create()
     except Exception:
-        LOG.error(_("Table |%s| not created!"), repr(quota_classes))
+        LOG.error(_LE("Table |%s| not created!"), repr(quota_classes))
         raise
 
     quota_usages = Table('quota_usages', meta,
@@ -72,7 +72,7 @@ def upgrade(migrate_engine):
     try:
         quota_usages.create()
     except Exception:
-        LOG.error(_("Table |%s| not created!"), repr(quota_usages))
+        LOG.error(_LE("Table |%s| not created!"), repr(quota_usages))
         raise
 
     reservations = Table('reservations', meta,
@@ -103,7 +103,7 @@ def upgrade(migrate_engine):
     try:
         reservations.create()
     except Exception:
-        LOG.error(_("Table |%s| not created!"), repr(reservations))
+        LOG.error(_LE("Table |%s| not created!"), repr(reservations))
         raise
 
 
@@ -111,39 +111,47 @@ def downgrade(migrate_engine):
     meta = MetaData()
     meta.bind = migrate_engine
 
-    if migrate_engine.name == 'mysql':
-        # NOTE(jsbryant): MySQL Cannot drop the quota_usages table
-        # until the foreign key reservations_ibfk_1 is removed.  We
-        # remove the foreign key first, and then we drop the table.
-        table = Table('reservations', meta, autoload=True)
-        ref_table = Table('reservations', meta, autoload=True)
-        params = {'columns': [table.c['usage_id']],
-                  'refcolumns': [ref_table.c['id']],
-                  'name': 'reservations_ibfk_1'}
+    fk_name = None
 
+    if migrate_engine.name == 'mysql':
+        fk_name = 'reservations_ibfk_1'
+    elif migrate_engine.name == 'postgresql':
+        fk_name = 'reservations_usage_id_fkey'
+
+    # NOTE: MySQL and PostgreSQL Cannot drop the quota_usages table
+    # until the foreign key is removed.  We remove the foreign key first,
+    # and then we drop the table.
+    table = Table('reservations', meta, autoload=True)
+    ref_table = Table('reservations', meta, autoload=True)
+    params = {'columns': [table.c['usage_id']],
+              'refcolumns': [ref_table.c['id']],
+              'name': fk_name}
+
+    if fk_name:
         try:
             fkey = ForeignKeyConstraint(**params)
             fkey.drop()
         except Exception:
-            LOG.error(_("Dropping foreign key reservations_ibfk_1 failed."))
+            msg = _LE("Dropping foreign key %s failed.")
+            LOG.error(msg, fk_name)
 
     quota_classes = Table('quota_classes', meta, autoload=True)
     try:
         quota_classes.drop()
     except Exception:
-        LOG.error(_("quota_classes table not dropped"))
+        LOG.error(_LE("quota_classes table not dropped"))
         raise
 
     quota_usages = Table('quota_usages', meta, autoload=True)
     try:
         quota_usages.drop()
     except Exception:
-        LOG.error(_("quota_usages table not dropped"))
+        LOG.error(_LE("quota_usages table not dropped"))
         raise
 
     reservations = Table('reservations', meta, autoload=True)
     try:
         reservations.drop()
     except Exception:
-        LOG.error(_("reservations table not dropped"))
+        LOG.error(_LE("reservations table not dropped"))
         raise
