@@ -10,11 +10,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 from sqlalchemy import Boolean, Column, DateTime
 from sqlalchemy import MetaData, String, Table, ForeignKey
 
 from cinder.i18n import _
-from cinder.openstack.common import log as logging
 
 LOG = logging.getLogger(__name__)
 
