@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 from sqlalchemy import Column, ForeignKey, MetaData, Table
 from sqlalchemy import Boolean, DateTime, Integer, String
 
 from cinder.i18n import _
-from cinder.openstack.common import log as logging
 
 
 LOG = logging.getLogger(__name__)
