@@ -14,8 +14,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_log import log as logging
 from sqlalchemy import Column, MetaData, String, Table
 
+from cinder.i18n import _LE
+
+LOG = logging.getLogger(__name__)
+
 
 def upgrade(migrate_engine):
     meta = MetaData()
@@ -24,5 +29,23 @@ def upgrade(migrate_engine):
     backups = Table('backups', meta, autoload=True)
     parent_id = Column('parent_id', String(length=36))
 
-    backups.create_column(parent_id)
-    backups.update().values(parent_id=None).execute()
+    try:
+        backups.create_column(parent_id)
+        backups.update().values(parent_id=None).execute()
+    except Exception:
+        LOG.error(_LE("Adding parent_id column to backups table failed."))
+        raise
+
+
+def downgrade(migrate_engine):
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    backups = Table('backups', meta, autoload=True)
+    parent_id = backups.columns.parent_id
+
+    try:
+        backups.drop_column(parent_id)
+    except Exception:
+        LOG.error(_LE("Dropping parent_id column from backups table failed."))
+        raise
