@@ -19,15 +19,14 @@
 SQLAlchemy models for cinder data.
 """
 
-from oslo.config import cfg
-from oslo.db.sqlalchemy import models
+from oslo_config import cfg
+from oslo_db.sqlalchemy import models
+from oslo_utils import timeutils
 from sqlalchemy import Column, Integer, String, Text, schema
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import ForeignKey, DateTime, Boolean
 from sqlalchemy.orm import relationship, backref, validates
 
-from cinder.openstack.common import timeutils
-
 
 CONF = cfg.CONF
 BASE = declarative_base()
@@ -64,6 +63,11 @@ class Service(BASE, CinderBase):
     disabled = Column(Boolean, default=False)
     availability_zone = Column(String(255), default='cinder')
     disabled_reason = Column(String(255))
+    # adding column modified_at to contain timestamp
+    # for manual enable/disable of cinder services
+    # updated_at column will now contain timestamps for
+    # periodic updates
+    modified_at = Column(DateTime)
 
 
 class ConsistencyGroup(BASE, CinderBase):
@@ -80,6 +84,7 @@ class ConsistencyGroup(BASE, CinderBase):
     description = Column(String(255))
     volume_type_id = Column(String(255))
     status = Column(String(255))
+    cgsnapshot_id = Column(String(36))
 
 
 class Cgsnapshot(BASE, CinderBase):
@@ -129,10 +134,6 @@ class Volume(BASE, CinderBase):
     host = Column(String(255))  # , ForeignKey('hosts.id'))
     size = Column(Integer)
     availability_zone = Column(String(255))  # TODO(vish): foreign key?
-    instance_uuid = Column(String(36))
-    attached_host = Column(String(255))
-    mountpoint = Column(String(255))
-    attach_time = Column(String(255))  # TODO(vish): datetime
     status = Column(String(255))  # TODO(vish): enum?
     attach_status = Column(String(255))  # TODO(vish): enum
     migration_status = Column(String(255))
@@ -147,6 +148,7 @@ class Volume(BASE, CinderBase):
     provider_location = Column(String(255))
     provider_auth = Column(String(255))
     provider_geometry = Column(String(255))
+    provider_id = Column(String(255))
 
     volume_type_id = Column(String(36))
     source_volid = Column(String(36))
@@ -156,6 +158,7 @@ class Volume(BASE, CinderBase):
 
     deleted = Column(Boolean, default=False)
     bootable = Column(Boolean, default=False)
+    multiattach = Column(Boolean, default=False)
 
     replication_status = Column(String(255))
     replication_extended_status = Column(String(255))
@@ -196,14 +199,36 @@ class VolumeAdminMetadata(BASE, CinderBase):
                           'VolumeAdminMetadata.deleted == False)')
 
 
+class VolumeAttachment(BASE, CinderBase):
+    """Represents a volume attachment for a vm."""
+    __tablename__ = 'volume_attachment'
+    id = Column(String(36), primary_key=True)
+
+    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False)
+    volume = relationship(Volume, backref="volume_attachment",
+                          foreign_keys=volume_id,
+                          primaryjoin='and_('
+                          'VolumeAttachment.volume_id == Volume.id,'
+                          'VolumeAttachment.deleted == False)')
+    instance_uuid = Column(String(36))
+    attached_host = Column(String(255))
+    mountpoint = Column(String(255))
+    attach_time = Column(DateTime)
+    detach_time = Column(DateTime)
+    attach_status = Column(String(255))
+    attach_mode = Column(String(255))
+
+
 class VolumeTypes(BASE, CinderBase):
     """Represent possible volume_types of volumes offered."""
     __tablename__ = "volume_types"
     id = Column(String(36), primary_key=True)
     name = Column(String(255))
+    description = Column(String(255))
     # A reference to qos_specs entity
     qos_specs_id = Column(String(36),
                           ForeignKey('quality_of_service_specs.id'))
+    is_public = Column(Boolean, default=True)
     volumes = relationship(Volume,
                            backref=backref('volume_type', uselist=False),
                            foreign_keys=id,
@@ -212,6 +237,27 @@ class VolumeTypes(BASE, CinderBase):
                            'VolumeTypes.deleted == False)')
 
 
+class VolumeTypeProjects(BASE, CinderBase):
+    """Represent projects associated volume_types."""
+    __tablename__ = "volume_type_projects"
+    __table_args__ = (schema.UniqueConstraint(
+        "volume_type_id", "project_id", "deleted",
+        name="uniq_volume_type_projects0volume_type_id0project_id0deleted"),
+    )
+    id = Column(Integer, primary_key=True)
+    volume_type_id = Column(Integer, ForeignKey('volume_types.id'),
+                            nullable=False)
+    project_id = Column(String(255))
+
+    volume_type = relationship(
+        VolumeTypes,
+        backref="projects",
+        foreign_keys=volume_type_id,
+        primaryjoin='and_('
+        'VolumeTypeProjects.volume_type_id == VolumeTypes.id,'
+        'VolumeTypeProjects.deleted == False)')
+
+
 class VolumeTypeExtraSpecs(BASE, CinderBase):
     """Represents additional specs as key/value pairs for a volume_type."""
     __tablename__ = 'volume_type_extra_specs'
@@ -409,6 +455,7 @@ class Snapshot(BASE, CinderBase):
     volume_type_id = Column(String(36))
 
     provider_location = Column(String(255))
+    provider_id = Column(String(255))
 
     volume = relationship(Volume, backref="snapshots",
                           foreign_keys=volume_id,
@@ -471,6 +518,7 @@ class Backup(BASE, CinderBase):
     display_name = Column(String(255))
     display_description = Column(String(255))
     container = Column(String(255))
+    parent_id = Column(String(36))
     status = Column(String(255))
     fail_reason = Column(String(255))
     service_metadata = Column(String(255))
@@ -491,13 +539,12 @@ class Encryption(BASE, CinderBase):
     """
 
     __tablename__ = 'encryption'
+    encryption_id = Column(String(36), primary_key=True)
     cipher = Column(String(255))
     key_size = Column(Integer)
     provider = Column(String(255))
     control_location = Column(String(255))
-    volume_type_id = Column(String(36),
-                            ForeignKey('volume_types.id'),
-                            primary_key=True)
+    volume_type_id = Column(String(36), ForeignKey('volume_types.id'))
     volume_type = relationship(
         VolumeTypes,
         backref="encryption",
@@ -524,6 +571,20 @@ class Transfer(BASE, CinderBase):
                           'Transfer.deleted == False)')
 
 
+class DriverInitiatorData(BASE, models.TimestampMixin, models.ModelBase):
+    """Represents private key-value pair specific an initiator for drivers"""
+    __tablename__ = 'driver_initiator_data'
+    __table_args__ = (
+        schema.UniqueConstraint("initiator", "namespace", "key"),
+        {'mysql_engine': 'InnoDB'}
+    )
+    id = Column(Integer, primary_key=True, nullable=False)
+    initiator = Column(String(255), index=True, nullable=False)
+    namespace = Column(String(255), nullable=False)
+    key = Column(String(255), nullable=False)
+    value = Column(String(255))
+
+
 def register_models():
     """Register Models and create metadata.
 
@@ -537,6 +598,7 @@ def register_models():
               Volume,
               VolumeMetadata,
               VolumeAdminMetadata,
+              VolumeAttachment,
               SnapshotMetadata,
               Transfer,
               VolumeTypeExtraSpecs,
