@@ -17,6 +17,9 @@ Handles all requests relating to volume replication.
 """
 import functools
 
+from oslo_config import cfg
+from oslo_log import log as logging
+
 from cinder.db import base
 from cinder import exception
 from cinder.i18n import _
@@ -25,6 +28,9 @@ from cinder import volume as cinder_volume
 from cinder.volume import rpcapi as volume_rpcapi
 from cinder.volume import utils as volume_utils
 
+CONF = cfg.CONF
+
+LOG = logging.getLogger(__name__)
 
 PROMOTE_PROCEED_STATUS = ('active', 'active-stopped')
 REENABLE_PROCEED_STATUS = ('inactive', 'active-stopped', 'error')
