@@ -15,9 +15,9 @@
 
 import operator
 import re
-from typing import Callable, Dict  # noqa: H301
 
 import pyparsing
+import six
 
 from cinder import exception
 from cinder.i18n import _
@@ -25,7 +25,7 @@ from cinder.i18n import _
 
 def _operatorOperands(tokenList):
     it = iter(tokenList)
-    while True:
+    while 1:
         try:
             op1 = next(it)
             op2 = next(it)
@@ -40,26 +40,26 @@ class EvalConstant(object):
 
     def eval(self):
         result = self.value
-        if (isinstance(result, str) and
-                re.match(r"^[a-zA-Z_]+\.[a-zA-Z_]+$", result)):
+        if (isinstance(result, six.string_types) and
+                re.match("^[a-zA-Z_]+\.[a-zA-Z_]+$", result)):
             (which_dict, entry) = result.split('.')
             try:
                 result = _vars[which_dict][entry]
-            except KeyError:
-                raise exception.EvaluatorParseException(
-                    _("KeyError evaluating string"))
-            except TypeError:
-                raise exception.EvaluatorParseException(
-                    _("TypeError evaluating string"))
+            except KeyError as e:
+                msg = _("KeyError: %s") % e
+                raise exception.EvaluatorParseException(msg)
+            except TypeError as e:
+                msg = _("TypeError: %s") % e
+                raise exception.EvaluatorParseException(msg)
 
         try:
             result = int(result)
         except ValueError:
             try:
                 result = float(result)
-            except ValueError:
-                if isinstance(result, str):
-                    result = result.replace('"', '').replace('\'', '')
+            except ValueError as e:
+                msg = _("ValueError: %s") % e
+                raise exception.EvaluatorParseException(msg)
 
         return result
 
@@ -104,8 +104,8 @@ class EvalMultOp(object):
                 elif op == '/':
                     prod /= float(val.eval())
             except ZeroDivisionError as e:
-                raise exception.EvaluatorParseException(
-                    _("ZeroDivisionError: %s") % e)
+                msg = _("ZeroDivisionError: %s") % e
+                raise exception.EvaluatorParseException(msg)
         return prod
 
 
@@ -168,7 +168,7 @@ class EvalTernaryOp(object):
 
 
 class EvalFunction(object):
-    functions: Dict[str, Callable] = {
+    functions = {
         "abs": abs,
         "max": max,
         "min": min,
@@ -221,7 +221,6 @@ class EvalBoolOrOp(object):
         right = self.value[2].eval()
         return left or right
 
-
 _parser = None
 _vars = {}
 
@@ -232,20 +231,20 @@ def _def_parser():
 
     alphas = pyparsing.alphas
     Combine = pyparsing.Combine
+    Forward = pyparsing.Forward
     nums = pyparsing.nums
-    quoted_string = pyparsing.quotedString
-
     oneOf = pyparsing.oneOf
     opAssoc = pyparsing.opAssoc
-    infixNotation = pyparsing.infixNotation
+    operatorPrecedence = pyparsing.operatorPrecedence
     Word = pyparsing.Word
 
     integer = Word(nums)
     real = Combine(Word(nums) + '.' + Word(nums))
     variable = Word(alphas + '_' + '.')
     number = real | integer
+    expr = Forward()
     fn = Word(alphas + '_' + '.')
-    operand = number | variable | fn | quoted_string
+    operand = number | variable | fn
 
     signop = oneOf('+ -')
     addop = oneOf('+ -')
@@ -257,7 +256,7 @@ def _def_parser():
     negateop = oneOf('NOT not !')
 
     operand.setParseAction(EvalConstant)
-    expr = infixNotation(operand, [
+    expr = operatorPrecedence(operand, [
         (fn, 1, opAssoc.RIGHT, EvalFunction),
         ("^", 2, opAssoc.RIGHT, EvalPowerOp),
         (signop, 1, opAssoc.RIGHT, EvalSignOp),
@@ -292,7 +291,7 @@ def evaluate(expression, **kwargs):
     try:
         result = _parser.parseString(expression, parseAll=True)[0]
     except pyparsing.ParseException as e:
-        raise exception.EvaluatorParseException(
-            _("ParseException: %s") % e)
+        msg = _("ParseException: %s") % e
+        raise exception.EvaluatorParseException(msg)
 
     return result.eval()
