@@ -11,16 +11,18 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+"""
+Chance Weigher.  Assign random weights to hosts.
+
+Used to spread volumes randomly across a list of equally suitable hosts.
+"""
+
 
 import random
 
-from cinder.scheduler import weights
+from cinder.openstack.common.scheduler import weights
 
 
 class ChanceWeigher(weights.BaseHostWeigher):
-    """Chance Weigher assigns random weights to hosts.
-
-    Used to spread volumes randomly across a list of equally suitable hosts.
-    """
     def _weigh_object(self, host_state, weight_properties):
         return random.random()
