@@ -11,9 +11,14 @@
 # under the License.
 
 import ast
-import tempfile
 
-from oslo.config import cfg
+import fixtures
+import mock
+from oslo_concurrency import lockutils
+from oslo_config import cfg
+from oslo_config import fixture as config_fixture
+from oslo_serialization import jsonutils
+from oslo_utils import timeutils
 import webob
 from webob import exc
 
@@ -22,13 +27,13 @@ from cinder.brick.local_dev import lvm as brick_lvm
 from cinder import context
 from cinder import db
 from cinder import exception
-from cinder.openstack.common import jsonutils
-from cinder.openstack.common import timeutils
 from cinder import test
 from cinder.tests.api import fakes
 from cinder.tests.api.v2 import stubs
 from cinder.tests import cast_as_call
+from cinder.tests import fake_snapshot
 from cinder.volume import api as volume_api
+from cinder.volume.targets import tgt
 
 CONF = cfg.CONF
 
@@ -45,15 +50,25 @@ class AdminActionsTest(test.TestCase):
     def setUp(self):
         super(AdminActionsTest, self).setUp()
 
-        self.tempdir = tempfile.mkdtemp()
+        self.tempdir = self.useFixture(fixtures.TempDir()).path
+        self.fixture = self.useFixture(config_fixture.Config(lockutils.CONF))
+        self.fixture.config(lock_path=self.tempdir,
+                            group='oslo_concurrency')
+        self.fixture.config(disable_process_locking=True,
+                            group='oslo_concurrency')
         self.flags(rpc_backend='cinder.openstack.common.rpc.impl_fake')
-        self.flags(lock_path=self.tempdir,
-                   disable_process_locking=True)
 
         self.volume_api = volume_api.API()
         cast_as_call.mock_cast_as_call(self.volume_api.volume_rpcapi.client)
         cast_as_call.mock_cast_as_call(self.volume_api.scheduler_rpcapi.client)
         self.stubs.Set(brick_lvm.LVM, '_vg_exists', lambda x: True)
+        self.stubs.Set(tgt.TgtAdm,
+                       'create_iscsi_target',
+                       self._fake_create_iscsi_target)
+
+    def _fake_create_iscsi_target(self, name, tid, lun,
+                                  path, chap_auth=None, **kwargs):
+        return 1
 
     def _issue_volume_reset(self, ctx, volume, updated_status):
         req = webob.Request.blank('/v2/fake/volumes/%s/action' % volume['id'])
@@ -341,12 +356,20 @@ class AdminActionsTest(test.TestCase):
         # volume is deleted
         self.assertRaises(exception.NotFound, db.volume_get, ctx, volume['id'])
 
-    def test_force_delete_snapshot(self):
+    @mock.patch.object(volume_api.API, 'delete_snapshot', return_value=True)
+    @mock.patch('cinder.objects.Snapshot.get_by_id')
+    @mock.patch.object(db, 'snapshot_get')
+    @mock.patch.object(db, 'volume_get')
+    def test_force_delete_snapshot(self, volume_get, snapshot_get, get_by_id,
+                                   delete_snapshot):
         ctx = context.RequestContext('admin', 'fake', True)
-        snapshot = stubs.stub_snapshot(1, host='foo')
-        self.stubs.Set(db, 'volume_get', lambda x, y: snapshot)
-        self.stubs.Set(db, 'snapshot_get', lambda x, y: snapshot)
-        self.stubs.Set(volume_api.API, 'delete_snapshot', lambda *x, **y: True)
+        volume = stubs.stub_volume(1)
+        snapshot = stubs.stub_snapshot(1)
+        snapshot_obj = fake_snapshot.fake_snapshot_obj(ctx, **snapshot)
+        volume_get.return_value = volume
+        snapshot_get.return_value = snapshot
+        get_by_id.return_value = snapshot_obj
+
         path = '/v2/fake/snapshots/%s/action' % snapshot['id']
         req = webob.Request.blank(path)
         req.method = 'POST'
@@ -368,15 +391,14 @@ class AdminActionsTest(test.TestCase):
         svc = self.start_service('volume', host='test')
         self.volume_api.reserve_volume(ctx, volume)
         mountpoint = '/dev/vbd'
-        self.volume_api.attach(ctx, volume, stubs.FAKE_UUID, None,
-                               mountpoint, 'rw')
+        attachment = self.volume_api.attach(ctx, volume, stubs.FAKE_UUID,
+                                            None, mountpoint, 'rw')
         # volume is attached
         volume = db.volume_get(ctx, volume['id'])
         self.assertEqual(volume['status'], 'in-use')
-        self.assertEqual(volume['instance_uuid'], stubs.FAKE_UUID)
-        self.assertIsNone(volume['attached_host'])
-        self.assertEqual(volume['mountpoint'], mountpoint)
-        self.assertEqual(volume['attach_status'], 'attached')
+        self.assertEqual(attachment['instance_uuid'], stubs.FAKE_UUID)
+        self.assertEqual(attachment['mountpoint'], mountpoint)
+        self.assertEqual(attachment['attach_status'], 'attached')
         admin_metadata = volume['volume_admin_metadata']
         self.assertEqual(len(admin_metadata), 2)
         self.assertEqual(admin_metadata[0]['key'], 'readonly')
@@ -384,14 +406,16 @@ class AdminActionsTest(test.TestCase):
         self.assertEqual(admin_metadata[1]['key'], 'attached_mode')
         self.assertEqual(admin_metadata[1]['value'], 'rw')
         conn_info = self.volume_api.initialize_connection(ctx,
-                                                          volume, connector)
+                                                          volume,
+                                                          connector)
         self.assertEqual(conn_info['data']['access_mode'], 'rw')
         # build request to force detach
         req = webob.Request.blank('/v2/fake/volumes/%s/action' % volume['id'])
         req.method = 'POST'
         req.headers['content-type'] = 'application/json'
         # request status of 'error'
-        req.body = jsonutils.dumps({'os-force_detach': None})
+        req.body = jsonutils.dumps({'os-force_detach':
+                                    {'attachment_id': attachment['id']}})
         # attach admin context to request
         req.environ['cinder.context'] = ctx
         # make request
@@ -399,12 +423,12 @@ class AdminActionsTest(test.TestCase):
         # request is accepted
         self.assertEqual(resp.status_int, 202)
         volume = db.volume_get(ctx, volume['id'])
+        self.assertRaises(exception.VolumeAttachmentNotFound,
+                          db.volume_attachment_get,
+                          ctx, attachment['id'])
+
         # status changed to 'available'
         self.assertEqual(volume['status'], 'available')
-        self.assertIsNone(volume['instance_uuid'])
-        self.assertIsNone(volume['attached_host'])
-        self.assertIsNone(volume['mountpoint'])
-        self.assertEqual(volume['attach_status'], 'detached')
         admin_metadata = volume['volume_admin_metadata']
         self.assertEqual(len(admin_metadata), 1)
         self.assertEqual(admin_metadata[0]['key'], 'readonly')
@@ -421,17 +445,18 @@ class AdminActionsTest(test.TestCase):
         connector = {'initiator': 'iqn.2012-07.org.fake:01'}
         # start service to handle rpc messages for attach requests
         svc = self.start_service('volume', host='test')
-        self.volume_api.reserve_volume(ctx, volume)
+        self.volume_api.initialize_connection(ctx, volume, connector)
         mountpoint = '/dev/vbd'
         host_name = 'fake-host'
-        self.volume_api.attach(ctx, volume, None, host_name, mountpoint, 'ro')
+        attachment = self.volume_api.attach(ctx, volume, None, host_name,
+                                            mountpoint, 'ro')
         # volume is attached
         volume = db.volume_get(ctx, volume['id'])
         self.assertEqual(volume['status'], 'in-use')
-        self.assertIsNone(volume['instance_uuid'])
-        self.assertEqual(volume['attached_host'], host_name)
-        self.assertEqual(volume['mountpoint'], mountpoint)
-        self.assertEqual(volume['attach_status'], 'attached')
+        self.assertIsNone(attachment['instance_uuid'])
+        self.assertEqual(attachment['attached_host'], host_name)
+        self.assertEqual(attachment['mountpoint'], mountpoint)
+        self.assertEqual(attachment['attach_status'], 'attached')
         admin_metadata = volume['volume_admin_metadata']
         self.assertEqual(len(admin_metadata), 2)
         self.assertEqual(admin_metadata[0]['key'], 'readonly')
@@ -446,7 +471,8 @@ class AdminActionsTest(test.TestCase):
         req.method = 'POST'
         req.headers['content-type'] = 'application/json'
         # request status of 'error'
-        req.body = jsonutils.dumps({'os-force_detach': None})
+        req.body = jsonutils.dumps({'os-force_detach':
+                                    {'attachment_id': attachment['id']}})
         # attach admin context to request
         req.environ['cinder.context'] = ctx
         # make request
@@ -454,12 +480,11 @@ class AdminActionsTest(test.TestCase):
         # request is accepted
         self.assertEqual(resp.status_int, 202)
         volume = db.volume_get(ctx, volume['id'])
+        self.assertRaises(exception.VolumeAttachmentNotFound,
+                          db.volume_attachment_get,
+                          ctx, attachment['id'])
         # status changed to 'available'
         self.assertEqual(volume['status'], 'available')
-        self.assertIsNone(volume['instance_uuid'])
-        self.assertIsNone(volume['attached_host'])
-        self.assertIsNone(volume['mountpoint'])
-        self.assertEqual(volume['attach_status'], 'detached')
         admin_metadata = volume['volume_admin_metadata']
         self.assertEqual(len(admin_metadata), 1)
         self.assertEqual(admin_metadata[0]['key'], 'readonly')
@@ -478,11 +503,10 @@ class AdminActionsTest(test.TestCase):
         # start service to handle rpc messages for attach requests
         svc = self.start_service('volume', host='test')
         self.volume_api.reserve_volume(ctx, volume)
-        mountpoint = '/dev/vbd'
-        self.volume_api.attach(ctx, volume, stubs.FAKE_UUID, None,
-                               mountpoint, 'rw')
         conn_info = self.volume_api.initialize_connection(ctx,
                                                           volume, connector)
+        self.volume_api.attach(ctx, volume, fakes.get_fake_uuid(), None,
+                               '/dev/vbd0', 'rw')
         self.assertEqual(conn_info['data']['access_mode'], 'rw')
         self.assertRaises(exception.InvalidVolume,
                           self.volume_api.attach,
@@ -490,15 +514,7 @@ class AdminActionsTest(test.TestCase):
                           volume,
                           fakes.get_fake_uuid(),
                           None,
-                          mountpoint,
-                          'rw')
-        self.assertRaises(exception.InvalidVolume,
-                          self.volume_api.attach,
-                          ctx,
-                          volume,
-                          fakes.get_fake_uuid(),
-                          None,
-                          mountpoint,
+                          '/dev/vdb1',
                           'ro')
         # cleanup
         svc.stop()
@@ -514,9 +530,9 @@ class AdminActionsTest(test.TestCase):
         # start service to handle rpc messages for attach requests
         svc = self.start_service('volume', host='test')
         self.volume_api.reserve_volume(ctx, volume)
-        mountpoint = '/dev/vbd'
-        host_name = 'fake_host'
-        self.volume_api.attach(ctx, volume, None, host_name, mountpoint, 'rw')
+        self.volume_api.initialize_connection(ctx, volume, connector)
+        self.volume_api.attach(ctx, volume, None, 'fake_host1',
+                               '/dev/vbd0', 'rw')
         conn_info = self.volume_api.initialize_connection(ctx,
                                                           volume, connector)
         conn_info['data']['access_mode'] = 'rw'
@@ -525,16 +541,8 @@ class AdminActionsTest(test.TestCase):
                           ctx,
                           volume,
                           None,
-                          host_name,
-                          mountpoint,
-                          'rw')
-        self.assertRaises(exception.InvalidVolume,
-                          self.volume_api.attach,
-                          ctx,
-                          volume,
-                          None,
-                          host_name,
-                          mountpoint,
+                          'fake_host2',
+                          '/dev/vbd1',
                           'ro')
         # cleanup
         svc.stop()
@@ -549,7 +557,7 @@ class AdminActionsTest(test.TestCase):
         connector = {}
         # start service to handle rpc messages for attach requests
         svc = self.start_service('volume', host='test')
-        self.assertRaises(exception.VolumeBackendAPIException,
+        self.assertRaises(exception.InvalidInput,
                           self.volume_api.initialize_connection,
                           ctx, volume, connector)
         # cleanup
@@ -563,19 +571,23 @@ class AdminActionsTest(test.TestCase):
                                         'provider_location': '', 'size': 1})
         # start service to handle rpc messages for attach requests
         svc = self.start_service('volume', host='test')
-        values = {'status': 'attaching',
-                  'instance_uuid': fakes.get_fake_uuid()}
-        db.volume_update(ctx, volume['id'], values)
+        self.volume_api.reserve_volume(ctx, volume)
+        values = {'volume_id': volume['id'],
+                  'attach_status': 'attaching',
+                  'attach_time': timeutils.utcnow(),
+                  'instance_uuid': 'abc123',
+                  }
+        db.volume_attach(ctx, values)
+        db.volume_admin_metadata_update(ctx, volume['id'],
+                                        {"attached_mode": 'rw'}, False)
         mountpoint = '/dev/vbd'
-        self.assertRaises(exception.InvalidVolume,
-                          self.volume_api.attach,
-                          ctx,
-                          volume,
-                          stubs.FAKE_UUID,
-                          None,
-                          mountpoint,
-                          'rw')
-        # cleanup
+        attachment = self.volume_api.attach(ctx, volume,
+                                            stubs.FAKE_UUID, None,
+                                            mountpoint, 'rw')
+
+        self.assertEqual(stubs.FAKE_UUID, attachment['instance_uuid'])
+        self.assertEqual(volume['id'], attachment['volume_id'], volume['id'])
+        self.assertEqual('attached', attachment['attach_status'])
         svc.stop()
 
     def test_attach_attaching_volume_with_different_mode(self):
