@@ -16,7 +16,9 @@
 import datetime
 
 from lxml import etree
-from oslo.config import cfg
+import mock
+from oslo_config import cfg
+from oslo_utils import timeutils
 import webob
 
 from cinder.api import extensions
@@ -29,7 +31,6 @@ from cinder.tests.api import fakes
 from cinder.tests.api.v2 import stubs
 from cinder.tests import fake_notifier
 from cinder.tests.image import fake as fake_image
-from cinder import utils
 from cinder.volume import api as volume_api
 
 
@@ -85,18 +86,15 @@ class VolumeApiTest(test.TestCase):
                                'availability_zone': 'zone1:host1',
                                'display_name': 'Volume Test Name',
                                'encrypted': False,
-                               'attachments': [{'device': '/',
-                                                'server_id': 'fakeuuid',
-                                                'host_name': None,
-                                                'id': '1',
-                                                'volume_id': '1'}],
+                               'attachments': [],
+                               'multiattach': 'false',
                                'bootable': 'false',
                                'volume_type': 'vol_type_name',
                                'snapshot_id': None,
                                'source_volid': None,
                                'metadata': {},
                                'id': '1',
-                               'created_at': datetime.datetime(1, 1, 1,
+                               'created_at': datetime.datetime(1900, 1, 1,
                                                                1, 1, 1),
                                'size': 100,
                                'encrypted': False}}
@@ -176,11 +174,8 @@ class VolumeApiTest(test.TestCase):
                                'availability_zone': 'nova',
                                'display_name': 'Volume Test Name',
                                'encrypted': False,
-                               'attachments': [{'device': '/',
-                                                'server_id': 'fakeuuid',
-                                                'host_name': None,
-                                                'id': '1',
-                                                'volume_id': '1'}],
+                               'attachments': [],
+                               'multiattach': 'false',
                                'bootable': 'false',
                                'volume_type': 'vol_type_name',
                                'image_id': test_id,
@@ -188,7 +183,7 @@ class VolumeApiTest(test.TestCase):
                                'source_volid': None,
                                'metadata': {},
                                'id': '1',
-                               'created_at': datetime.datetime(1, 1, 1,
+                               'created_at': datetime.datetime(1900, 1, 1,
                                                                1, 1, 1),
                                'size': '1'}}
         body = {"volume": vol}
@@ -258,13 +253,8 @@ class VolumeApiTest(test.TestCase):
             'availability_zone': 'fakeaz',
             'display_name': 'Updated Test Name',
             'encrypted': False,
-            'attachments': [{
-                'id': '1',
-                'volume_id': '1',
-                'server_id': 'fakeuuid',
-                'host_name': None,
-                'device': '/'
-            }],
+            'attachments': [],
+            'multiattach': 'false',
             'bootable': 'false',
             'volume_type': 'vol_type_name',
             'snapshot_id': None,
@@ -272,7 +262,7 @@ class VolumeApiTest(test.TestCase):
             'metadata': {'attached_mode': 'rw',
                          'readonly': 'False'},
             'id': '1',
-            'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+            'created_at': datetime.datetime(1900, 1, 1, 1, 1, 1),
             'size': 1}}
         self.assertEqual(res_dict, expected)
         self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
@@ -294,13 +284,8 @@ class VolumeApiTest(test.TestCase):
             'availability_zone': 'fakeaz',
             'display_name': 'displayname',
             'encrypted': False,
-            'attachments': [{
-                'id': '1',
-                'volume_id': '1',
-                'server_id': 'fakeuuid',
-                'host_name': None,
-                'device': '/'
-            }],
+            'attachments': [],
+            'multiattach': 'false',
             'bootable': 'false',
             'volume_type': 'vol_type_name',
             'snapshot_id': None,
@@ -309,7 +294,7 @@ class VolumeApiTest(test.TestCase):
                          "readonly": "False",
                          "attached_mode": "rw"},
             'id': '1',
-            'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+            'created_at': datetime.datetime(1900, 1, 1, 1, 1, 1),
             'size': 1
         }}
         self.assertEqual(res_dict, expected)
@@ -328,6 +313,10 @@ class VolumeApiTest(test.TestCase):
                                         {"readonly": "True",
                                          "invisible_key": "invisible_value"},
                                         False)
+        values = {'volume_id': '1', }
+        attachment = db.volume_attach(context.get_admin_context(), values)
+        db.volume_attached(context.get_admin_context(),
+                           attachment['id'], stubs.FAKE_UUID, None, '/')
 
         updates = {
             "display_name": "Updated Test Name",
@@ -339,26 +328,28 @@ class VolumeApiTest(test.TestCase):
         req.environ['cinder.context'] = admin_ctx
         res_dict = self.controller.update(req, '1', body)
         expected = {'volume': {
-            'status': 'fakestatus',
+            'status': 'in-use',
             'display_description': 'displaydesc',
             'availability_zone': 'fakeaz',
             'display_name': 'Updated Test Name',
             'encrypted': False,
             'attachments': [{
+                'attachment_id': attachment['id'],
                 'id': '1',
                 'volume_id': '1',
-                'server_id': 'fakeuuid',
+                'server_id': stubs.FAKE_UUID,
                 'host_name': None,
                 'device': '/'
             }],
+            'multiattach': 'false',
             'bootable': 'false',
-            'volume_type': 'None',
+            'volume_type': None,
             'snapshot_id': None,
             'source_volid': None,
             'metadata': {'key': 'value',
                          'readonly': 'True'},
             'id': '1',
-            'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
+            'created_at': datetime.datetime(1900, 1, 1, 1, 1, 1),
             'size': 1}}
         self.assertEqual(res_dict, expected)
         self.assertEqual(len(fake_notifier.NOTIFICATIONS), 2)
@@ -400,11 +391,8 @@ class VolumeApiTest(test.TestCase):
                                  'availability_zone': 'fakeaz',
                                  'display_name': 'displayname',
                                  'encrypted': False,
-                                 'attachments': [{'device': '/',
-                                                  'server_id': 'fakeuuid',
-                                                  'host_name': None,
-                                                  'id': '1',
-                                                  'volume_id': '1'}],
+                                 'attachments': [],
+                                 'multiattach': 'false',
                                  'bootable': 'false',
                                  'volume_type': 'vol_type_name',
                                  'snapshot_id': None,
@@ -412,7 +400,7 @@ class VolumeApiTest(test.TestCase):
                                  'metadata': {'attached_mode': 'rw',
                                               'readonly': 'False'},
                                  'id': '1',
-                                 'created_at': datetime.datetime(1, 1, 1,
+                                 'created_at': datetime.datetime(1900, 1, 1,
                                                                  1, 1, 1),
                                  'size': 1}]}
         self.assertEqual(res_dict, expected)
@@ -430,29 +418,36 @@ class VolumeApiTest(test.TestCase):
                                         {"readonly": "True",
                                          "invisible_key": "invisible_value"},
                                         False)
+        values = {'volume_id': '1', }
+        attachment = db.volume_attach(context.get_admin_context(), values)
+        db.volume_attached(context.get_admin_context(),
+                           attachment['id'], stubs.FAKE_UUID, None, '/')
 
         req = fakes.HTTPRequest.blank('/v1/volumes')
         admin_ctx = context.RequestContext('admin', 'fakeproject', True)
         req.environ['cinder.context'] = admin_ctx
         res_dict = self.controller.index(req)
-        expected = {'volumes': [{'status': 'fakestatus',
+        expected = {'volumes': [{'status': 'in-use',
                                  'display_description': 'displaydesc',
                                  'availability_zone': 'fakeaz',
                                  'display_name': 'displayname',
                                  'encrypted': False,
-                                 'attachments': [{'device': '/',
-                                                  'server_id': 'fakeuuid',
-                                                  'host_name': None,
-                                                  'id': '1',
-                                                  'volume_id': '1'}],
+                                 'attachments': [
+                                     {'attachment_id': attachment['id'],
+                                      'device': '/',
+                                      'server_id': stubs.FAKE_UUID,
+                                      'host_name': None,
+                                      'id': '1',
+                                      'volume_id': '1'}],
+                                 'multiattach': 'false',
                                  'bootable': 'false',
-                                 'volume_type': 'None',
+                                 'volume_type': None,
                                  'snapshot_id': None,
                                  'source_volid': None,
                                  'metadata': {'key': 'value',
                                               'readonly': 'True'},
                                  'id': '1',
-                                 'created_at': datetime.datetime(1, 1, 1,
+                                 'created_at': datetime.datetime(1900, 1, 1,
                                                                  1, 1, 1),
                                  'size': 1}]}
         self.assertEqual(res_dict, expected)
@@ -469,11 +464,8 @@ class VolumeApiTest(test.TestCase):
                                  'availability_zone': 'fakeaz',
                                  'display_name': 'displayname',
                                  'encrypted': False,
-                                 'attachments': [{'device': '/',
-                                                  'server_id': 'fakeuuid',
-                                                  'host_name': None,
-                                                  'id': '1',
-                                                  'volume_id': '1'}],
+                                 'attachments': [],
+                                 'multiattach': 'false',
                                  'bootable': 'false',
                                  'volume_type': 'vol_type_name',
                                  'snapshot_id': None,
@@ -481,7 +473,7 @@ class VolumeApiTest(test.TestCase):
                                  'metadata': {'attached_mode': 'rw',
                                               'readonly': 'False'},
                                  'id': '1',
-                                 'created_at': datetime.datetime(1, 1, 1,
+                                 'created_at': datetime.datetime(1900, 1, 1,
                                                                  1, 1, 1),
                                  'size': 1}]}
         self.assertEqual(res_dict, expected)
@@ -499,29 +491,36 @@ class VolumeApiTest(test.TestCase):
                                         {"readonly": "True",
                                          "invisible_key": "invisible_value"},
                                         False)
+        values = {'volume_id': '1', }
+        attachment = db.volume_attach(context.get_admin_context(), values)
+        db.volume_attached(context.get_admin_context(),
+                           attachment['id'], stubs.FAKE_UUID, None, '/')
 
         req = fakes.HTTPRequest.blank('/v1/volumes/detail')
         admin_ctx = context.RequestContext('admin', 'fakeproject', True)
         req.environ['cinder.context'] = admin_ctx
         res_dict = self.controller.index(req)
-        expected = {'volumes': [{'status': 'fakestatus',
+        expected = {'volumes': [{'status': 'in-use',
                                  'display_description': 'displaydesc',
                                  'availability_zone': 'fakeaz',
                                  'display_name': 'displayname',
                                  'encrypted': False,
-                                 'attachments': [{'device': '/',
-                                                  'server_id': 'fakeuuid',
-                                                  'host_name': None,
-                                                  'id': '1',
-                                                  'volume_id': '1'}],
+                                 'attachments': [
+                                     {'attachment_id': attachment['id'],
+                                      'device': '/',
+                                      'server_id': stubs.FAKE_UUID,
+                                      'host_name': None,
+                                      'id': '1',
+                                      'volume_id': '1'}],
+                                 'multiattach': 'false',
                                  'bootable': 'false',
-                                 'volume_type': 'None',
+                                 'volume_type': None,
                                  'snapshot_id': None,
                                  'source_volid': None,
                                  'metadata': {'key': 'value',
                                               'readonly': 'True'},
                                  'id': '1',
-                                 'created_at': datetime.datetime(1, 1, 1,
+                                 'created_at': datetime.datetime(1900, 1, 1,
                                                                  1, 1, 1),
                                  'size': 1}]}
         self.assertEqual(res_dict, expected)
@@ -536,11 +535,8 @@ class VolumeApiTest(test.TestCase):
                                'availability_zone': 'fakeaz',
                                'display_name': 'displayname',
                                'encrypted': False,
-                               'attachments': [{'device': '/',
-                                                'server_id': 'fakeuuid',
-                                                'host_name': None,
-                                                'id': '1',
-                                                'volume_id': '1'}],
+                               'attachments': [],
+                               'multiattach': 'false',
                                'bootable': 'false',
                                'volume_type': 'vol_type_name',
                                'snapshot_id': None,
@@ -548,7 +544,7 @@ class VolumeApiTest(test.TestCase):
                                'metadata': {'attached_mode': 'rw',
                                             'readonly': 'False'},
                                'id': '1',
-                               'created_at': datetime.datetime(1, 1, 1,
+                               'created_at': datetime.datetime(1900, 1, 1,
                                                                1, 1, 1),
                                'size': 1}}
         self.assertEqual(res_dict, expected)
@@ -569,13 +565,14 @@ class VolumeApiTest(test.TestCase):
                                'display_name': 'displayname',
                                'encrypted': False,
                                'attachments': [],
+                               'multiattach': 'false',
                                'bootable': 'false',
                                'volume_type': 'vol_type_name',
                                'snapshot_id': None,
                                'source_volid': None,
                                'metadata': {'readonly': 'False'},
                                'id': '1',
-                               'created_at': datetime.datetime(1, 1, 1,
+                               'created_at': datetime.datetime(1900, 1, 1,
                                                                1, 1, 1),
                                'size': 1}}
         self.assertEqual(res_dict, expected)
@@ -594,11 +591,8 @@ class VolumeApiTest(test.TestCase):
                                'availability_zone': 'fakeaz',
                                'display_name': 'displayname',
                                'encrypted': False,
-                               'attachments': [{'device': '/',
-                                                'server_id': 'fakeuuid',
-                                                'host_name': None,
-                                                'id': '1',
-                                                'volume_id': '1'}],
+                               'attachments': [],
+                               'multiattach': 'false',
                                'bootable': 'true',
                                'volume_type': 'vol_type_name',
                                'snapshot_id': None,
@@ -606,7 +600,7 @@ class VolumeApiTest(test.TestCase):
                                'metadata': {'attached_mode': 'rw',
                                             'readonly': 'False'},
                                'id': '1',
-                               'created_at': datetime.datetime(1, 1, 1,
+                               'created_at': datetime.datetime(1900, 1, 1,
                                                                1, 1, 1),
                                'size': 1}}
         self.assertEqual(res_dict, expected)
@@ -625,8 +619,8 @@ class VolumeApiTest(test.TestCase):
     def test_volume_detail_limit_offset(self):
         def volume_detail_limit_offset(is_admin):
             def stub_volume_get_all_by_project(context, project_id, marker,
-                                               limit, sort_key, sort_dir,
-                                               filters=None,
+                                               limit, sort_keys=None,
+                                               sort_dirs=None, filters=None,
                                                viewable_admin_meta=False):
                 return [
                     stubs.stub_volume(1, display_name='vol1'),
@@ -645,9 +639,9 @@ class VolumeApiTest(test.TestCase):
             self.assertEqual(len(volumes), 1)
             self.assertEqual(volumes[0]['id'], 2)
 
-        #admin case
+        # admin case
         volume_detail_limit_offset(is_admin=True)
-        #non_admin case
+        # non_admin case
         volume_detail_limit_offset(is_admin=False)
 
     def test_volume_show_with_admin_metadata(self):
@@ -661,29 +655,36 @@ class VolumeApiTest(test.TestCase):
                                         {"readonly": "True",
                                          "invisible_key": "invisible_value"},
                                         False)
+        values = {'volume_id': '1', }
+        attachment = db.volume_attach(context.get_admin_context(), values)
+        db.volume_attached(context.get_admin_context(),
+                           attachment['id'], stubs.FAKE_UUID, None, '/')
 
         req = fakes.HTTPRequest.blank('/v1/volumes/1')
         admin_ctx = context.RequestContext('admin', 'fakeproject', True)
         req.environ['cinder.context'] = admin_ctx
         res_dict = self.controller.show(req, '1')
-        expected = {'volume': {'status': 'fakestatus',
+        expected = {'volume': {'status': 'in-use',
                                'display_description': 'displaydesc',
                                'availability_zone': 'fakeaz',
                                'display_name': 'displayname',
                                'encrypted': False,
-                               'attachments': [{'device': '/',
-                                                'server_id': 'fakeuuid',
-                                                'host_name': None,
-                                                'id': '1',
-                                                'volume_id': '1'}],
+                               'attachments': [
+                                   {'attachment_id': attachment['id'],
+                                    'device': '/',
+                                    'server_id': stubs.FAKE_UUID,
+                                    'host_name': None,
+                                    'id': '1',
+                                    'volume_id': '1'}],
+                               'multiattach': 'false',
                                'bootable': 'false',
-                               'volume_type': 'None',
+                               'volume_type': None,
                                'snapshot_id': None,
                                'source_volid': None,
                                'metadata': {'key': 'value',
                                             'readonly': 'True'},
                                'id': '1',
-                               'created_at': datetime.datetime(1, 1, 1,
+                               'created_at': datetime.datetime(1900, 1, 1,
                                                                1, 1, 1),
                                'size': 1}}
         self.assertEqual(res_dict, expected)
@@ -762,30 +763,41 @@ class VolumeApiTest(test.TestCase):
         self.assertIn('volumes', res)
         self.assertEqual(1, len(res['volumes']))
 
-    def test_add_visible_admin_metadata_visible_key_only(self):
-        admin_metadata = [{"key": "invisible_key", "value": "invisible_value"},
-                          {"key": "readonly", "value": "visible"},
-                          {"key": "attached_mode", "value": "visible"}]
-        metadata = [{"key": "key", "value": "value"}]
-        volume = dict(volume_admin_metadata=admin_metadata,
-                      volume_metadata=metadata)
-        utils.add_visible_admin_metadata(volume)
-        self.assertEqual(volume['volume_metadata'],
-                         [{"key": "key", "value": "value"},
-                          {"key": "readonly", "value": "visible"},
-                          {"key": "attached_mode", "value": "visible"}])
+    @mock.patch('cinder.volume.api.API.get_all')
+    def test_get_volumes_filter_with_string(self, get_all):
+        req = mock.MagicMock()
+        req.GET.copy.return_value = {'display_name': 'Volume-573108026'}
+        context = mock.Mock()
+        req.environ = {'cinder.context': context}
+        self.controller._items(req, mock.Mock)
+        get_all.assert_called_once_with(
+            context, sort_dirs=['desc'], viewable_admin_meta=True,
+            sort_keys=['created_at'], limit=None,
+            filters={'display_name': 'Volume-573108026'}, marker=None)
 
-        admin_metadata = {"invisible_key": "invisible_value",
-                          "readonly": "visible",
-                          "attached_mode": "visible"}
-        metadata = {"key": "value"}
-        volume = dict(admin_metadata=admin_metadata,
-                      metadata=metadata)
-        utils.add_visible_admin_metadata(volume)
-        self.assertEqual(volume['metadata'],
-                         {'key': 'value',
-                          'attached_mode': 'visible',
-                          'readonly': 'visible'})
+    @mock.patch('cinder.volume.api.API.get_all')
+    def test_get_volumes_filter_with_list(self, get_all):
+        req = mock.MagicMock()
+        req.GET.copy.return_value = {'id': "['1', '2', '3']"}
+        context = mock.Mock()
+        req.environ = {'cinder.context': context}
+        self.controller._items(req, mock.Mock)
+        get_all.assert_called_once_with(
+            context, sort_dirs=['desc'], viewable_admin_meta=True,
+            sort_keys=['created_at'], limit=None,
+            filters={'id': ['1', '2', '3']}, marker=None)
+
+    @mock.patch('cinder.volume.api.API.get_all')
+    def test_get_volumes_filter_with_expression(self, get_all):
+        req = mock.MagicMock()
+        req.GET.copy.return_value = {'id': "d+"}
+        context = mock.Mock()
+        req.environ = {'cinder.context': context}
+        self.controller._items(req, mock.Mock)
+        get_all.assert_called_once_with(
+            context, sort_dirs=['desc'], viewable_admin_meta=True,
+            sort_keys=['created_at'], limit=None, filters={'id': 'd+'},
+            marker=None)
 
 
 class VolumeSerializerTest(test.TestCase):
@@ -824,7 +836,7 @@ class VolumeSerializerTest(test.TestCase):
             size=1024,
             availability_zone='vol_availability',
             bootable='false',
-            created_at=datetime.datetime.now(),
+            created_at=timeutils.utcnow(),
             attachments=[dict(id='vol_id',
                               volume_id='vol_id',
                               server_id='instance_uuid',
@@ -849,7 +861,7 @@ class VolumeSerializerTest(test.TestCase):
                             size=1024,
                             availability_zone='vol1_availability',
                             bootable='true',
-                            created_at=datetime.datetime.now(),
+                            created_at=timeutils.utcnow(),
                             attachments=[dict(id='vol1_id',
                                               volume_id='vol1_id',
                                               server_id='instance_uuid',
@@ -866,7 +878,7 @@ class VolumeSerializerTest(test.TestCase):
                             size=1024,
                             availability_zone='vol2_availability',
                             bootable='true',
-                            created_at=datetime.datetime.now(),
+                            created_at=timeutils.utcnow(),
                             attachments=[dict(id='vol2_id',
                                               volume_id='vol2_id',
                                               server_id='instance_uuid',
