@@ -15,11 +15,11 @@
 
 import mock
 import mox
+from oslo_log import log as logging
 
 from cinder.brick import exception
 from cinder.brick.remotefs import remotefs
 from cinder.i18n import _
-from cinder.openstack.common import log as logging
 from cinder import test
 
 LOG = logging.getLogger(__name__)
