@@ -12,10 +12,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from cinder.openstack.common import log as logging
-from cinder.tests.brick.fake_lvm import FakeBrickLVM
+from oslo_log import log as logging
+
+from cinder.i18n import _LE
+from cinder.tests.brick import fake_lvm
 from cinder.volume import driver
 from cinder.volume.drivers import lvm
+from cinder.zonemanager import utils as fczm_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -26,9 +29,9 @@ class FakeISCSIDriver(lvm.LVMISCSIDriver):
     def __init__(self, *args, **kwargs):
         super(FakeISCSIDriver, self).__init__(execute=self.fake_execute,
                                               *args, **kwargs)
-        self.vg = FakeBrickLVM('cinder-volumes', False,
-                               None, 'default',
-                               self.fake_execute)
+        self.vg = fake_lvm.FakeBrickLVM('cinder-volumes', False,
+                                        None, 'default',
+                                        self.fake_execute)
 
     def check_for_setup_error(self):
         """No setup necessary in fake mode."""
@@ -36,17 +39,18 @@ class FakeISCSIDriver(lvm.LVMISCSIDriver):
 
     def initialize_connection(self, volume, connector):
         volume_metadata = {}
+
         for metadata in volume['volume_admin_metadata']:
             volume_metadata[metadata['key']] = metadata['value']
+
         access_mode = volume_metadata.get('attached_mode')
         if access_mode is None:
             access_mode = ('ro'
                            if volume_metadata.get('readonly') == 'True'
                            else 'rw')
-        return {
-            'driver_volume_type': 'iscsi',
-            'data': {'access_mode': access_mode}
-        }
+
+        return {'driver_volume_type': 'iscsi',
+                'data': {'access_mode': access_mode}}
 
     def terminate_connection(self, volume, connector, **kwargs):
         pass
@@ -54,7 +58,7 @@ class FakeISCSIDriver(lvm.LVMISCSIDriver):
     @staticmethod
     def fake_execute(cmd, *_args, **_kwargs):
         """Execute that simply logs the command."""
-        LOG.debug(_("FAKE ISCSI: %s"), cmd)
+        LOG.debug("FAKE ISCSI: %s", cmd)
         return (None, None)
 
 
@@ -73,10 +77,46 @@ class FakeISERDriver(FakeISCSIDriver):
     @staticmethod
     def fake_execute(cmd, *_args, **_kwargs):
         """Execute that simply logs the command."""
-        LOG.debug(_("FAKE ISER: %s"), cmd)
+        LOG.debug("FAKE ISER: %s", cmd)
         return (None, None)
 
 
+class FakeFibreChannelDriver(driver.FibreChannelDriver):
+
+    @fczm_utils.AddFCZone
+    def initialize_connection(self, volume, connector):
+        return {
+            'driver_volume_type': 'fibre_channel',
+            'data': {
+                'initiator_target_map': {'fake_wwn': ['fake_wwn2']},
+            }}
+
+    @fczm_utils.AddFCZone
+    def no_zone_initialize_connection(self, volume, connector):
+        """This shouldn't call the ZM."""
+        return {
+            'driver_volume_type': 'bogus',
+            'data': {
+                'initiator_target_map': {'fake_wwn': ['fake_wwn2']},
+            }}
+
+    @fczm_utils.RemoveFCZone
+    def terminate_connection(self, volume, connector, **kwargs):
+        return {
+            'driver_volume_type': 'fibre_channel',
+            'data': {
+                'initiator_target_map': {'fake_wwn': ['fake_wwn2']},
+            }}
+
+    @fczm_utils.RemoveFCZone
+    def no_zone_terminate_connection(self, volume, connector, **kwargs):
+        return {
+            'driver_volume_type': 'bogus',
+            'data': {
+                'initiator_target_map': {'fake_wwn': ['fake_wwn2']},
+            }}
+
+
 class LoggingVolumeDriver(driver.VolumeDriver):
     """Logs and records calls, for unit tests."""
 
@@ -94,7 +134,7 @@ class LoggingVolumeDriver(driver.VolumeDriver):
         self.log_action('clear_volume', volume)
 
     def local_path(self, volume):
-        LOG.error(_("local_path not implemented"))
+        LOG.error(_LE("local_path not implemented"))
         raise NotImplementedError()
 
     def ensure_export(self, context, volume):
@@ -121,12 +161,12 @@ class LoggingVolumeDriver(driver.VolumeDriver):
     @staticmethod
     def log_action(action, parameters):
         """Logs the command."""
-        LOG.debug(_("LoggingVolumeDriver: %s") % (action))
+        LOG.debug("LoggingVolumeDriver: %s" % (action))
         log_dictionary = {}
         if parameters:
             log_dictionary = dict(parameters)
         log_dictionary['action'] = action
-        LOG.debug(_("LoggingVolumeDriver: %s") % (log_dictionary))
+        LOG.debug("LoggingVolumeDriver: %s" % (log_dictionary))
         LoggingVolumeDriver._LOGS.append(log_dictionary)
 
     @staticmethod
