@@ -19,14 +19,18 @@ Self test for Hitachi Unified Storage (HUS-HNAS) platform.
 """
 
 import os
+import StringIO
 import tempfile
 
 import mock
+from oslo_log import log as logging
 
-from cinder.openstack.common import log as logging
+from cinder import exception
 from cinder import test
 from cinder.volume import configuration as conf
 from cinder.volume.drivers.hds import iscsi
+from cinder.volume import volume_types
+
 LOG = logging.getLogger(__name__)
 
 HNASCONF = """<?xml version="1.0" encoding="UTF-8" ?>
@@ -49,9 +53,38 @@ HNASCONF = """<?xml version="1.0" encoding="UTF-8" ?>
 </config>
 """
 
+HNAS_WRONG_CONF1 = """<?xml version="1.0" encoding="UTF-8" ?>
+<config>
+  <hnas_cmd>ssc</hnas_cmd>
+  <mgmt_ip0>172.17.44.15</mgmt_ip0>
+  <username>supervisor</username>
+  <password>supervisor</password>
+    <volume_type>default</volume_type>
+    <hdp>172.17.39.132:/cinder</hdp>
+  </svc_0>
+</config>
+"""
+
+HNAS_WRONG_CONF2 = """<?xml version="1.0" encoding="UTF-8" ?>
+<config>
+  <hnas_cmd>ssc</hnas_cmd>
+  <mgmt_ip0>172.17.44.15</mgmt_ip0>
+  <username>supervisor</username>
+  <password>supervisor</password>
+  <svc_0>
+    <volume_type>default</volume_type>
+  </svc_0>
+  <svc_1>
+    <volume_type>silver</volume_type>
+  </svc_1>
+</config>
+"""
+
 # The following information is passed on to tests, when creating a volume
 _VOLUME = {'name': 'testvol', 'volume_id': '1234567890', 'size': 128,
-           'volume_type': None, 'provider_location': None, 'id': 'abcdefg'}
+           'volume_type': 'silver', 'volume_type_id': '1',
+           'provider_location': None, 'id': 'abcdefg',
+           'host': 'host1@hnas-iscsi-backend#silver'}
 
 
 class SimulatedHnasBackend(object):
@@ -240,7 +273,7 @@ class SimulatedHnasBackend(object):
             "CTL: 1 Port: 5 IP: 172.17.39.133 Port: 3260 Link: Up"
         return self.out
 
-    def get_hdp_info(self, cmd, ip0, user, pw):
+    def get_hdp_info(self, cmd, ip0, user, pw, fslabel=None):
         self.out = "HDP: 1024  272384 MB    33792 MB  12 %  LUs:  " \
             "70  Normal  fs1\n" \
             "HDP: 1025  546816 MB    73728 MB  13 %  LUs:  194  Normal  fs2"
@@ -291,6 +324,26 @@ class HNASiSCSIDriverTest(test.TestCase):
         vol['provider_location'] = loc['provider_location']
         return vol
 
+    @mock.patch('__builtin__.open')
+    @mock.patch.object(os, 'access')
+    def test_read_config(self, m_access, m_open):
+        # Test exception when file is not found
+        m_access.return_value = False
+        m_open.return_value = StringIO.StringIO(HNASCONF)
+        self.assertRaises(exception.NotFound, iscsi._read_config, '')
+
+        # Test exception when config file has parsing errors
+        # due to missing <svc> tag
+        m_access.return_value = True
+        m_open.return_value = StringIO.StringIO(HNAS_WRONG_CONF1)
+        self.assertRaises(exception.ConfigNotFound, iscsi._read_config, '')
+
+        # Test exception when config file has parsing errors
+        # due to missing <hdp> tag
+        m_open.return_value = StringIO.StringIO(HNAS_WRONG_CONF2)
+        self.configuration.hds_hnas_iscsi_config_file = ''
+        self.assertRaises(exception.ParameterNotFound, iscsi._read_config, '')
+
     def test_create_volume(self):
         loc = self.driver.create_volume(_VOLUME)
         self.assertNotEqual(loc, None)
@@ -302,7 +355,7 @@ class HNASiSCSIDriverTest(test.TestCase):
         stats = self.driver.get_volume_stats(True)
         self.assertEqual(stats["vendor_name"], "HDS")
         self.assertEqual(stats["storage_protocol"], "iSCSI")
-        self.assertTrue(stats["total_capacity_gb"] > 0)
+        self.assertEqual(len(stats['pools']), 2)
 
     def test_delete_volume(self):
         vol = self._create_volume()
@@ -394,3 +447,9 @@ class HNASiSCSIDriverTest(test.TestCase):
         self.assertNotEqual(num_conn_before, num_conn_after)
         # cleanup
         self.backend.deleteVolumebyProvider(vol['provider_location'])
+
+    @mock.patch.object(volume_types, 'get_volume_type_extra_specs',
+                       return_value={'key': 'type', 'service_label': 'silver'})
+    def test_get_pool(self, m_ext_spec):
+        label = self.driver.get_pool(_VOLUME)
+        self.assertEqual('silver', label)
