@@ -18,11 +18,12 @@ Test suite for VMware VMDK driver volumeops module.
 """
 
 import mock
+from oslo_utils import units
+from oslo_vmware import exceptions
+from oslo_vmware import vim_util
 
-from cinder.openstack.common import units
 from cinder import test
-from cinder.volume.drivers.vmware import error_util
-from cinder.volume.drivers.vmware import vim_util
+from cinder.volume.drivers.vmware import exceptions as vmdk_exceptions
 from cinder.volume.drivers.vmware import volumeops
 
 
@@ -237,6 +238,30 @@ class VolumeOpsTestCase(test.TestCase):
             hosts = self.vops.get_connected_hosts(datastore)
             self.assertEqual([], hosts)
 
+    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
+                'get_connected_hosts')
+    def test_is_datastore_accessible(self, get_connected_hosts):
+        host_1 = mock.sentinel.host_1
+        host_2 = mock.sentinel.host_2
+        get_connected_hosts.return_value = [host_1, host_2]
+
+        ds = mock.sentinel.datastore
+        host = mock.Mock(value=mock.sentinel.host_1)
+        self.assertTrue(self.vops.is_datastore_accessible(ds, host))
+        get_connected_hosts.assert_called_once_with(ds)
+
+    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
+                'get_connected_hosts')
+    def test_is_datastore_accessible_with_inaccessible(self,
+                                                       get_connected_hosts):
+        host_1 = mock.sentinel.host_1
+        get_connected_hosts.return_value = [host_1]
+
+        ds = mock.sentinel.datastore
+        host = mock.Mock(value=mock.sentinel.host_2)
+        self.assertFalse(self.vops.is_datastore_accessible(ds, host))
+        get_connected_hosts.assert_called_once_with(ds)
+
     def test_is_valid(self):
         with mock.patch.object(self.vops, 'get_summary') as get_summary:
             summary = mock.Mock(spec=object)
@@ -319,7 +344,7 @@ class VolumeOpsTestCase(test.TestCase):
                                                    host_mounts,
                                                    host_mounts,
                                                    resource_pool]
-            self.assertRaises(error_util.VimException,
+            self.assertRaises(exceptions.VimException,
                               self.vops.get_dss_rp,
                               host)
 
@@ -378,6 +403,24 @@ class VolumeOpsTestCase(test.TestCase):
                                                         mock.sentinel.dc,
                                                         'vmFolder')
 
+    def test_create_folder_with_empty_vmfolder(self):
+        """Test create_folder when the datacenter vmFolder is empty"""
+        child_folder = mock.sentinel.child_folder
+        self.session.invoke_api.side_effect = [None, child_folder]
+
+        parent_folder = mock.sentinel.parent_folder
+        child_name = 'child_folder'
+        ret = self.vops.create_folder(parent_folder, child_name)
+
+        self.assertEqual(child_folder, ret)
+        expected_calls = [mock.call(vim_util, 'get_object_property',
+                                    self.session.vim, parent_folder,
+                                    'childEntity'),
+                          mock.call(self.session.vim, 'CreateFolder',
+                                    parent_folder, name=child_name)]
+        self.assertEqual(expected_calls,
+                         self.session.invoke_api.call_args_list)
+
     def test_create_folder_not_present(self):
         """Test create_folder when child not present."""
         parent_folder = mock.sentinel.parent_folder
@@ -983,7 +1026,7 @@ class VolumeOpsTestCase(test.TestCase):
         self.assertEqual(mock.sentinel.new_backing, ret)
         disk_move_type = 'moveAllDiskBackingsAndDisallowSharing'
         get_clone_spec.assert_called_with(datastore, disk_move_type, snapshot,
-                                          backing, None, None)
+                                          backing, None, None, None)
         expected = [mock.call(vim_util, 'get_object_property',
                               self.session.vim, backing, 'parent'),
                     mock.call(self.session.vim, 'CloneVM_Task', backing,
@@ -999,7 +1042,7 @@ class VolumeOpsTestCase(test.TestCase):
         self.assertEqual(mock.sentinel.new_backing, ret)
         disk_move_type = 'createNewChildDiskBacking'
         get_clone_spec.assert_called_with(datastore, disk_move_type, snapshot,
-                                          backing, None, None)
+                                          backing, None, None, None)
         expected = [mock.call(vim_util, 'get_object_property',
                               self.session.vim, backing, 'parent'),
                     mock.call(self.session.vim, 'CloneVM_Task', backing,
@@ -1010,14 +1053,15 @@ class VolumeOpsTestCase(test.TestCase):
         clone_type = None
         disk_type = 'thin'
         host = mock.sentinel.host
+        rp = mock.sentinel.rp
         self.session.invoke_api.reset_mock()
         ret = self.vops.clone_backing(name, backing, snapshot, clone_type,
-                                      datastore, disk_type, host)
+                                      datastore, disk_type, host, rp)
 
         self.assertEqual(mock.sentinel.new_backing, ret)
         disk_move_type = 'moveAllDiskBackingsAndDisallowSharing'
         get_clone_spec.assert_called_with(datastore, disk_move_type, snapshot,
-                                          backing, disk_type, host)
+                                          backing, disk_type, host, rp)
         expected = [mock.call(vim_util, 'get_object_property',
                               self.session.vim, backing, 'parent'),
                     mock.call(self.session.vim, 'CloneVM_Task', backing,
@@ -1131,6 +1175,63 @@ class VolumeOpsTestCase(test.TestCase):
                                            datacenter=datacenter)
         self.session.wait_for_task.assert_called_once_with(task)
 
+    def test_create_datastore_folder(self):
+        file_manager = mock.sentinel.file_manager
+        self.session.vim.service_content.fileManager = file_manager
+        invoke_api = self.session.invoke_api
+
+        ds_name = "nfs"
+        folder_path = "test/"
+        datacenter = mock.sentinel.datacenter
+
+        self.vops.create_datastore_folder(ds_name, folder_path, datacenter)
+        invoke_api.assert_called_once_with(self.session.vim,
+                                           'MakeDirectory',
+                                           file_manager,
+                                           name="[nfs] test/",
+                                           datacenter=datacenter)
+
+    def test_create_datastore_folder_with_existing_folder(self):
+        file_manager = mock.sentinel.file_manager
+        self.session.vim.service_content.fileManager = file_manager
+        invoke_api = self.session.invoke_api
+        invoke_api.side_effect = exceptions.FileAlreadyExistsException
+
+        ds_name = "nfs"
+        folder_path = "test/"
+        datacenter = mock.sentinel.datacenter
+
+        self.vops.create_datastore_folder(ds_name, folder_path, datacenter)
+        invoke_api.assert_called_once_with(self.session.vim,
+                                           'MakeDirectory',
+                                           file_manager,
+                                           name="[nfs] test/",
+                                           datacenter=datacenter)
+        invoke_api.side_effect = None
+
+    def test_create_datastore_folder_with_invoke_api_error(self):
+        file_manager = mock.sentinel.file_manager
+        self.session.vim.service_content.fileManager = file_manager
+        invoke_api = self.session.invoke_api
+        invoke_api.side_effect = exceptions.VimFaultException(
+            ["FileFault"], "error")
+
+        ds_name = "nfs"
+        folder_path = "test/"
+        datacenter = mock.sentinel.datacenter
+
+        self.assertRaises(exceptions.VimFaultException,
+                          self.vops.create_datastore_folder,
+                          ds_name,
+                          folder_path,
+                          datacenter)
+        invoke_api.assert_called_once_with(self.session.vim,
+                                           'MakeDirectory',
+                                           file_manager,
+                                           name="[nfs] test/",
+                                           datacenter=datacenter)
+        invoke_api.side_effect = None
+
     def test_get_path_name(self):
         path = mock.Mock(spec=object)
         path_name = mock.sentinel.vm_path_name
@@ -1176,7 +1277,7 @@ class VolumeOpsTestCase(test.TestCase):
 
         # Test with no disk device.
         invoke_api.return_value = []
-        self.assertRaises(error_util.VirtualDiskNotFoundException,
+        self.assertRaises(vmdk_exceptions.VirtualDiskNotFoundException,
                           self.vops.get_vmdk_path,
                           backing)
 
@@ -1195,7 +1296,7 @@ class VolumeOpsTestCase(test.TestCase):
         # Test with no disk device.
         invoke_api.return_value = []
 
-        self.assertRaises(error_util.VirtualDiskNotFoundException,
+        self.assertRaises(vmdk_exceptions.VirtualDiskNotFoundException,
                           self.vops.get_disk_size,
                           mock.sentinel.backing)
 
@@ -1255,19 +1356,44 @@ class VolumeOpsTestCase(test.TestCase):
         task = mock.sentinel.task
         invoke_api = self.session.invoke_api
         invoke_api.return_value = task
+
         disk_mgr = self.session.vim.service_content.virtualDiskManager
-        dc_ref = self.session.dc_ref
-        src_vmdk_file_path = self.session.src
-        dest_vmdk_file_path = self.session.dest
-        self.vops.copy_vmdk_file(dc_ref, src_vmdk_file_path,
-                                 dest_vmdk_file_path)
+        src_dc_ref = mock.sentinel.src_dc_ref
+        src_vmdk_file_path = mock.sentinel.src_vmdk_file_path
+        dest_dc_ref = mock.sentinel.dest_dc_ref
+        dest_vmdk_file_path = mock.sentinel.dest_vmdk_file_path
+        self.vops.copy_vmdk_file(src_dc_ref, src_vmdk_file_path,
+                                 dest_vmdk_file_path, dest_dc_ref)
+
         invoke_api.assert_called_once_with(self.session.vim,
                                            'CopyVirtualDisk_Task',
                                            disk_mgr,
                                            sourceName=src_vmdk_file_path,
-                                           sourceDatacenter=dc_ref,
+                                           sourceDatacenter=src_dc_ref,
                                            destName=dest_vmdk_file_path,
-                                           destDatacenter=dc_ref,
+                                           destDatacenter=dest_dc_ref,
+                                           force=True)
+        self.session.wait_for_task.assert_called_once_with(task)
+
+    def test_copy_vmdk_file_with_default_dest_datacenter(self):
+        task = mock.sentinel.task
+        invoke_api = self.session.invoke_api
+        invoke_api.return_value = task
+
+        disk_mgr = self.session.vim.service_content.virtualDiskManager
+        src_dc_ref = mock.sentinel.src_dc_ref
+        src_vmdk_file_path = mock.sentinel.src_vmdk_file_path
+        dest_vmdk_file_path = mock.sentinel.dest_vmdk_file_path
+        self.vops.copy_vmdk_file(src_dc_ref, src_vmdk_file_path,
+                                 dest_vmdk_file_path)
+
+        invoke_api.assert_called_once_with(self.session.vim,
+                                           'CopyVirtualDisk_Task',
+                                           disk_mgr,
+                                           sourceName=src_vmdk_file_path,
+                                           sourceDatacenter=src_dc_ref,
+                                           destName=dest_vmdk_file_path,
+                                           destDatacenter=src_dc_ref,
                                            force=True)
         self.session.wait_for_task.assert_called_once_with(task)
 
@@ -1286,6 +1412,52 @@ class VolumeOpsTestCase(test.TestCase):
                                            datacenter=dc_ref)
         self.session.wait_for_task.assert_called_once_with(task)
 
+    def test_get_profile(self):
+        server_obj = mock.Mock()
+        self.session.pbm.client.factory.create.return_value = server_obj
+
+        profile_ids = [mock.sentinel.profile_id]
+        profile_name = mock.sentinel.profile_name
+        profile = mock.Mock()
+        profile.name = profile_name
+        self.session.invoke_api.side_effect = [profile_ids, [profile]]
+
+        value = mock.sentinel.value
+        backing = mock.Mock(value=value)
+        self.assertEqual(profile_name, self.vops.get_profile(backing))
+
+        pbm = self.session.pbm
+        profile_manager = pbm.service_content.profileManager
+        exp_calls = [mock.call(pbm, 'PbmQueryAssociatedProfile',
+                               profile_manager, entity=server_obj),
+                     mock.call(pbm, 'PbmRetrieveContent', profile_manager,
+                               profileIds=profile_ids)]
+        self.assertEqual(exp_calls, self.session.invoke_api.call_args_list)
+
+        self.assertEqual(value, server_obj.key)
+        self.assertEqual('virtualMachine', server_obj.objectType)
+        self.session.invoke_api.side_effect = None
+
+    def test_get_profile_with_no_profile(self):
+        server_obj = mock.Mock()
+        self.session.pbm.client.factory.create.return_value = server_obj
+
+        self.session.invoke_api.side_effect = [[]]
+
+        value = mock.sentinel.value
+        backing = mock.Mock(value=value)
+        self.assertIsNone(self.vops.get_profile(backing))
+
+        pbm = self.session.pbm
+        profile_manager = pbm.service_content.profileManager
+        exp_calls = [mock.call(pbm, 'PbmQueryAssociatedProfile',
+                               profile_manager, entity=server_obj)]
+        self.assertEqual(exp_calls, self.session.invoke_api.call_args_list)
+
+        self.assertEqual(value, server_obj.key)
+        self.assertEqual('virtualMachine', server_obj.objectType)
+        self.session.invoke_api.side_effect = None
+
     def test_extend_virtual_disk(self):
         """Test volumeops.extend_virtual_disk."""
         task = mock.sentinel.task
@@ -1307,84 +1479,6 @@ class VolumeOpsTestCase(test.TestCase):
                                            eagerZero=False)
         self.session.wait_for_task.assert_called_once_with(task)
 
-    def test_get_all_profiles(self):
-        profile_ids = [1, 2]
-        methods = ['PbmQueryProfile', 'PbmRetrieveContent']
-
-        def invoke_api_side_effect(module, method, *args, **kwargs):
-            self.assertEqual(self.session.pbm, module)
-            self.assertEqual(methods.pop(0), method)
-            self.assertEqual(self.session.pbm.service_content.profileManager,
-                             args[0])
-            if method == 'PbmQueryProfile':
-                self.assertEqual('STORAGE',
-                                 kwargs['resourceType'].resourceType)
-                return profile_ids
-            self.assertEqual(profile_ids, kwargs['profileIds'])
-
-        self.session.invoke_api.side_effect = invoke_api_side_effect
-        self.vops.get_all_profiles()
-
-        self.assertEqual(2, self.session.invoke_api.call_count)
-
-        # Clear side effects.
-        self.session.invoke_api.side_effect = None
-
-    def test_get_all_profiles_with_no_profiles(self):
-        self.session.invoke_api.return_value = []
-        res_type = mock.sentinel.res_type
-        self.session.pbm.client.factory.create.return_value = res_type
-
-        profiles = self.vops.get_all_profiles()
-        self.session.invoke_api.assert_called_once_with(
-            self.session.pbm,
-            'PbmQueryProfile',
-            self.session.pbm.service_content.profileManager,
-            resourceType=res_type)
-        self.assertEqual([], profiles)
-
-    def _create_profile(self, profile_id, name):
-        profile = mock.Mock()
-        profile.profileId = profile_id
-        profile.name = name
-        return profile
-
-    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
-                'get_all_profiles')
-    def test_retrieve_profile_id(self, get_all_profiles):
-        profiles = [self._create_profile(str(i), 'profile-%d' % i)
-                    for i in range(0, 10)]
-        get_all_profiles.return_value = profiles
-
-        exp_profile_id = '5'
-        profile_id = self.vops.retrieve_profile_id(
-            'profile-%s' % exp_profile_id)
-        self.assertEqual(exp_profile_id, profile_id)
-        get_all_profiles.assert_called_once_with()
-
-    @mock.patch('cinder.volume.drivers.vmware.volumeops.VMwareVolumeOps.'
-                'get_all_profiles')
-    def test_retrieve_profile_id_with_invalid_profile(self, get_all_profiles):
-        profiles = [self._create_profile(str(i), 'profile-%d' % i)
-                    for i in range(0, 10)]
-        get_all_profiles.return_value = profiles
-
-        profile_id = self.vops.retrieve_profile_id('profile-%s' % (i + 1))
-        self.assertIsNone(profile_id)
-        get_all_profiles.assert_called_once_with()
-
-    def test_filter_matching_hubs(self):
-        hubs = mock.Mock()
-        profile_id = 'profile-0'
-
-        self.vops.filter_matching_hubs(hubs, profile_id)
-        self.session.invoke_api.assert_called_once_with(
-            self.session.pbm,
-            'PbmQueryMatchingHub',
-            self.session.pbm.service_content.placementSolver,
-            hubsToSearch=hubs,
-            profile=profile_id)
-
 
 class VirtualDiskPathTest(test.TestCase):
     """Unit tests for VirtualDiskPath."""
@@ -1436,7 +1530,7 @@ class VirtualDiskTypeTest(test.TestCase):
         volumeops.VirtualDiskType.validate("thick")
         volumeops.VirtualDiskType.validate("thin")
         volumeops.VirtualDiskType.validate("eagerZeroedThick")
-        self.assertRaises(error_util.InvalidDiskTypeException,
+        self.assertRaises(vmdk_exceptions.InvalidDiskTypeException,
                           volumeops.VirtualDiskType.validate,
                           "preallocated")
 
@@ -1450,7 +1544,7 @@ class VirtualDiskTypeTest(test.TestCase):
         self.assertEqual("eagerZeroedThick",
                          volumeops.VirtualDiskType.get_virtual_disk_type(
                              "eagerZeroedThick"))
-        self.assertRaises(error_util.InvalidDiskTypeException,
+        self.assertRaises(vmdk_exceptions.InvalidDiskTypeException,
                           volumeops.VirtualDiskType.get_virtual_disk_type,
                           "preallocated")
 
@@ -1471,7 +1565,7 @@ class VirtualDiskAdapterTypeTest(test.TestCase):
         volumeops.VirtualDiskAdapterType.validate("busLogic")
         volumeops.VirtualDiskAdapterType.validate("lsiLogicsas")
         volumeops.VirtualDiskAdapterType.validate("ide")
-        self.assertRaises(error_util.InvalidAdapterTypeException,
+        self.assertRaises(vmdk_exceptions.InvalidAdapterTypeException,
                           volumeops.VirtualDiskAdapterType.validate,
                           "pvscsi")
 
@@ -1488,7 +1582,7 @@ class VirtualDiskAdapterTypeTest(test.TestCase):
         self.assertEqual("ide",
                          volumeops.VirtualDiskAdapterType.get_adapter_type(
                              "ide"))
-        self.assertRaises(error_util.InvalidAdapterTypeException,
+        self.assertRaises(vmdk_exceptions.InvalidAdapterTypeException,
                           volumeops.VirtualDiskAdapterType.get_adapter_type,
                           "pvscsi")
 
@@ -1509,7 +1603,7 @@ class ControllerTypeTest(test.TestCase):
         self.assertEqual(volumeops.ControllerType.IDE,
                          volumeops.ControllerType.get_controller_type(
                              'ide'))
-        self.assertRaises(error_util.InvalidAdapterTypeException,
+        self.assertRaises(vmdk_exceptions.InvalidAdapterTypeException,
                           volumeops.ControllerType.get_controller_type,
                           'invalid_type')
 
