@@ -10,16 +10,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import copy
 import os
 import shutil
 import tempfile
-from unittest import mock
 
-from oslo_utils import fileutils
+import mock
 from oslo_utils import timeutils
 
-from cinder.tests.unit import test
+from cinder.openstack.common import fileutils
+from cinder import test
 from cinder.volume import configuration as conf
 
 
@@ -29,8 +28,8 @@ class TargetDriverFixture(test.TestCase):
         self.configuration = conf.Configuration(None)
         self.configuration.append_config_values = mock.Mock(return_value=0)
         self.configuration.safe_get = mock.Mock(side_effect=self.fake_safe_get)
-        self.configuration.target_ip_address = '10.9.8.7'
-        self.configuration.target_port = 3260
+        self.configuration.iscsi_ip_address = '10.9.8.7'
+        self.configuration.iscsi_port = 3260
 
         self.fake_volumes_dir = tempfile.mkdtemp()
         fileutils.ensure_tree(self.fake_volumes_dir)
@@ -56,9 +55,6 @@ class TargetDriverFixture(test.TestCase):
              'created_at': timeutils.utcnow(),
              'host': 'fake_host@lvm#lvm'}
 
-        self.testvol_no_prov_loc = copy.copy(self.testvol)
-        self.testvol_no_prov_loc['provider_location'] = None
-
         self.iscsi_target_prefix = 'iqn.2010-10.org.openstack:'
         self.target_string = ('127.0.0.1:3260,1 ' +
                               self.iscsi_target_prefix +
@@ -71,8 +67,8 @@ class TargetDriverFixture(test.TestCase):
              'id': self.fake_volume_id,
              'volume_type_id': None,
              'provider_location': ('%(ip)s:%(port)d%(iqn)svolume-%(vol)s 2' %
-                                   {'ip': self.configuration.target_ip_address,
-                                    'port': self.configuration.target_port,
+                                   {'ip': self.configuration.iscsi_ip_address,
+                                    'port': self.configuration.iscsi_port,
                                     'iqn': self.iscsi_target_prefix,
                                     'vol': self.fake_volume_id}),
              'provider_auth': 'CHAP stack-1-a60e2611875f40199931f2'
@@ -95,10 +91,9 @@ class TargetDriverFixture(test.TestCase):
              'target_portal': '10.10.7.1:3260',
              'volume_id': self.fake_volume_id}
 
-        self.VOLUME_ID = '83c2e877-feed-46be-8435-77884fe55b45'
-        self.VOLUME_NAME = 'volume-' + self.VOLUME_ID
+        self.volume_name = 'volume-83c2e877-feed-46be-8435-77884fe55b45'
         self.test_vol = (self.iscsi_target_prefix +
-                         self.VOLUME_NAME)
+                         self.volume_name)
 
     def _cleanup(self):
         if os.path.exists(self.fake_volumes_dir):
@@ -107,7 +102,7 @@ class TargetDriverFixture(test.TestCase):
     def fake_safe_get(self, value):
         if value == 'volumes_dir':
             return self.fake_volumes_dir
-        elif value == 'target_protocol':
-            return self.configuration.target_protocol
-        elif value == 'target_prefix':
+        elif value == 'iscsi_protocol':
+            return self.configuration.iscsi_protocol
+        elif value == 'iscsi_target_prefix':
             return self.iscsi_target_prefix
