@@ -13,9 +13,11 @@
 #    under the License.
 #
 
-
 from cinder import context
 from cinder import db
+from cinder.openstack.common import loopingcall
+
+from oslo_utils import timeutils
 
 
 def get_test_admin_context():
@@ -61,10 +63,26 @@ def create_volume(ctxt,
     return db.volume_create(ctxt, vol)
 
 
+def attach_volume(ctxt, volume_id, instance_uuid, attached_host,
+                  mountpoint, mode='rw'):
+
+    now = timeutils.utcnow()
+    values = {}
+    values['volume_id'] = volume_id
+    values['attached_host'] = attached_host
+    values['mountpoint'] = mountpoint
+    values['attach_time'] = now
+
+    attachment = db.volume_attach(ctxt, values)
+    return db.volume_attached(ctxt, attachment['id'], instance_uuid,
+                              attached_host, mountpoint, mode)
+
+
 def create_snapshot(ctxt,
                     volume_id,
                     display_name='test_snapshot',
                     display_description='this is a test snapshot',
+                    cgsnapshot_id = None,
                     status='creating'):
     vol = db.volume_get(ctxt, volume_id)
     snap = {}
@@ -75,6 +93,7 @@ def create_snapshot(ctxt,
     snap['volume_size'] = vol['size']
     snap['display_name'] = display_name
     snap['display_description'] = display_description
+    snap['cgsnapshot_id'] = cgsnapshot_id
     return db.snapshot_create(ctxt, snap)
 
 
@@ -85,6 +104,7 @@ def create_consistencygroup(ctxt,
                             status='available',
                             availability_zone='fake_az',
                             volume_type_id=None,
+                            cgsnapshot_id=None,
                             **kwargs):
     """Create a consistencygroup object in the DB."""
     cg = {}
@@ -119,3 +139,9 @@ def create_cgsnapshot(ctxt,
     for key in kwargs:
         cgsnap[key] = kwargs[key]
     return db.cgsnapshot_create(ctxt, cgsnap)
+
+
+class ZeroIntervalLoopingCall(loopingcall.FixedIntervalLoopingCall):
+    def start(self, interval, **kwargs):
+        kwargs['initial_delay'] = 0
+        return super(ZeroIntervalLoopingCall, self).start(0, **kwargs)
