@@ -189,8 +189,7 @@ class BlockDeviceDriver(driver.VolumeDriver):
     # #######  Interface methods for DataPath (Target Driver) ########
 
     def ensure_export(self, context, volume):
-        volume_path = "/dev/%s/%s" % (self.configuration.volume_group,
-                                      volume['name'])
+        volume_path = self.local_path(volume)
         model_update = \
             self.target_driver.ensure_export(
                 context,
@@ -199,8 +198,7 @@ class BlockDeviceDriver(driver.VolumeDriver):
         return model_update
 
     def create_export(self, context, volume):
-        volume_path = "/dev/%s/%s" % (self.configuration.volume_group,
-                                      volume['name'])
+        volume_path = self.local_path(volume)
         export_info = self.target_driver.create_export(context,
                                                        volume,
                                                        volume_path)
