@@ -16,11 +16,9 @@
 import time
 
 from oslo_log import log as logging
-import six
 
-from cinder import coordination
 from cinder import exception
-from cinder.i18n import _, _LE, _LW
+from cinder.i18n import _, _LE
 from cinder.volume.drivers.emc import emc_vmax_utils
 
 LOG = logging.getLogger(__name__)
@@ -31,10 +29,6 @@ POSTGROUPTYPE = 3
 EMC_ROOT = 'root/emc'
 THINPROVISIONINGCOMPOSITE = 32768
 THINPROVISIONING = 5
-INFO_SRC_V3 = 3
-ACTIVATESNAPVX = 4
-DEACTIVATESNAPVX = 19
-SNAPSYNCTYPE = 7
 
 
 class EMCVMAXProvisionV3(object):
@@ -52,7 +46,7 @@ class EMCVMAXProvisionV3(object):
             extraSpecs):
         """Given the volume instance remove it from the pool.
 
-        :param conn: connection to the ecom server
+        :param conn: connection the the ecom server
         :param storageConfigservice: volume created from job
         :param volumeInstanceName: the volume instance name
         :param volumeName: the volume name (String)
@@ -72,10 +66,10 @@ class EMCVMAXProvisionV3(object):
             'ReturnElementsToStoragePool', storageConfigservice,
             TheElements=theElements)
 
-        if rc != 0:
+        if rc != 0L:
             rc, errordesc = self.utils.wait_for_job_complete(conn, job,
                                                              extraSpecs)
-            if rc != 0:
+            if rc != 0L:
                 exceptionMessage = (_(
                     "Error Delete Volume: %(volumeName)s. "
                     "Return code: %(rc)lu.  Error: %(error)s.")
@@ -112,55 +106,41 @@ class EMCVMAXProvisionV3(object):
         :returns: int -- return code
         :raises: VolumeBackendAPIException
         """
-        try:
-            storageGroupInstance = conn.GetInstance(sgInstanceName)
-        except Exception:
-            exceptionMessage = (_(
-                "Unable to get the name of the storage group"))
-            LOG.error(exceptionMessage)
-            raise exception.VolumeBackendAPIException(
-                data=exceptionMessage)
-        sgName = storageGroupInstance['ElementName']
+        startTime = time.time()
 
-        @coordination.synchronized("emc-sg-{storageGroup}")
-        def do_create_volume_from_sg(storageGroup):
-            startTime = time.time()
+        rc, job = conn.InvokeMethod(
+            'CreateOrModifyElementFromStoragePool',
+            storageConfigService, ElementName=volumeName,
+            EMCCollections=[sgInstanceName],
+            ElementType=self.utils.get_num(THINPROVISIONING, '16'),
+            Size=self.utils.get_num(volumeSize, '64'))
 
-            rc, job = conn.InvokeMethod(
-                'CreateOrModifyElementFromStoragePool',
-                storageConfigService, ElementName=volumeName,
-                EMCCollections=[sgInstanceName],
-                ElementType=self.utils.get_num(THINPROVISIONING, '16'),
-                Size=self.utils.get_num(volumeSize, '64'))
+        LOG.debug("Create Volume: %(volumename)s. Return code: %(rc)lu.",
+                  {'volumename': volumeName,
+                   'rc': rc})
 
-            LOG.debug("Create Volume: %(volumename)s. Return code: %(rc)lu.",
-                      {'volumename': volumeName,
-                       'rc': rc})
+        if rc != 0L:
+            rc, errordesc = self.utils.wait_for_job_complete(conn, job,
+                                                             extraSpecs)
+            if rc != 0L:
+                exceptionMessage = (_(
+                    "Error Create Volume: %(volumeName)s. "
+                    "Return code: %(rc)lu.  Error: %(error)s.")
+                    % {'volumeName': volumeName,
+                       'rc': rc,
+                       'error': errordesc})
+                LOG.error(exceptionMessage)
+                raise exception.VolumeBackendAPIException(
+                    data=exceptionMessage)
 
-            if rc != 0:
-                rc, errordesc = self.utils.wait_for_job_complete(conn, job,
-                                                                 extraSpecs)
-                if rc != 0:
-                    exceptionMessage = (_(
-                        "Error Create Volume: %(volumeName)s. "
-                        "Return code: %(rc)lu.  Error: %(error)s.")
-                        % {'volumeName': volumeName,
-                           'rc': rc,
-                           'error': errordesc})
-                    LOG.error(exceptionMessage)
-                    raise exception.VolumeBackendAPIException(
-                        data=exceptionMessage)
+        LOG.debug("InvokeMethod CreateOrModifyElementFromStoragePool "
+                  "took: %(delta)s H:MM:SS.",
+                  {'delta': self.utils.get_time_delta(startTime,
+                                                      time.time())})
 
-            LOG.debug("InvokeMethod CreateOrModifyElementFromStoragePool "
-                      "took: %(delta)s H:MM:SS.",
-                      {'delta': self.utils.get_time_delta(startTime,
-                                                          time.time())})
-
-            # Find the newly created volume.
-            volumeDict = self.get_volume_dict_from_job(conn, job['Job'])
-            return volumeDict, rc
-
-        return do_create_volume_from_sg(sgName)
+        # Find the newly created volume.
+        volumeDict = self.get_volume_dict_from_job(conn, job['Job'])
+        return volumeDict, rc
 
     def _find_new_storage_group(
             self, conn, maskingGroupDict, storageGroupName):
@@ -187,39 +167,7 @@ class EMCVMAXProvisionV3(object):
         associators = conn.Associators(
             jobInstance,
             ResultClass='EMC_StorageVolume')
-        if len(associators) > 0:
-            return self.create_volume_dict(associators[0].path)
-        else:
-            exceptionMessage = (_(
-                "Unable to get storage volume from job."))
-            LOG.error(exceptionMessage)
-            raise exception.VolumeBackendAPIException(data=exceptionMessage)
-
-    def get_volume_from_job(self, conn, jobInstance):
-        """Given the jobInstance determine the volume Instance.
-
-        :param conn: the ecom connection
-        :param jobInstance: the instance of a job
-        :returns: dict -- volumeDict - an instance of a volume
-        """
-        associators = conn.Associators(
-            jobInstance,
-            ResultClass='EMC_StorageVolume')
-        if len(associators) > 0:
-            return associators[0]
-        else:
-            exceptionMessage = (_(
-                "Unable to get storage volume from job."))
-            LOG.error(exceptionMessage)
-            raise exception.VolumeBackendAPIException(data=exceptionMessage)
-
-    def create_volume_dict(self, volumeInstanceName):
-        """Create volume dictionary
-
-        :param volumeInstanceName: the instance of a job
-        :returns: dict -- volumeDict - an instance of a volume
-        """
-        volpath = volumeInstanceName
+        volpath = associators[0].path
         volumeDict = {}
         volumeDict['classname'] = volpath.classname
         keys = {}
@@ -231,30 +179,10 @@ class EMCVMAXProvisionV3(object):
 
         return volumeDict
 
-    def get_or_create_default_sg(self, conn, extraSpecs, storageSystemName):
-        """Get or create default storage group for a replica.
-
-        :param conn: the connection to the ecom server
-        :param extraSpecs: the extra specifications
-        :param storageSystemName: the storage system name
-        :returns: sgInstanceName, instance of storage group
-        """
-        pool = extraSpecs[self.utils.POOL]
-        slo = extraSpecs[self.utils.SLO]
-        workload = extraSpecs[self.utils.WORKLOAD]
-        storageGroupName, controllerConfigService, sgInstanceName = (
-            self.utils.get_v3_default_sg_instance_name(
-                conn, pool, slo, workload, storageSystemName))
-        if sgInstanceName is None:
-            sgInstanceName = self.create_storage_group_v3(
-                conn, controllerConfigService, storageGroupName,
-                pool, slo, workload, extraSpecs)
-        return sgInstanceName
-
     def create_element_replica(
             self, conn, repServiceInstanceName,
             cloneName, syncType, sourceInstance, extraSpecs,
-            targetInstance=None, rsdInstance=None, copyState=None):
+            targetInstance=None):
         """Make SMI-S call to create replica for source element.
 
         :param conn: the connection to the ecom server
@@ -263,116 +191,55 @@ class EMCVMAXProvisionV3(object):
         :param syncType: 7=snapshot, 8=clone
         :param sourceInstance: source volume instance
         :param extraSpecs: additional info
-        :param targetInstance: Target volume instance. Default None
-        :param rsdInstance: replication settingdata instance. Default None
+        :param targetInstance: target volume instance. Defaults to None
         :returns: int -- rc - return code
         :returns: job - job object of the replica creation operation
         :raises: VolumeBackendAPIException
         """
         startTime = time.time()
-        LOG.debug("Create replica: %(clone)s "
-                  "syncType: %(syncType)s  Source: %(source)s.",
-                  {'clone': cloneName,
-                   'syncType': syncType,
-                   'source': sourceInstance.path})
-        storageSystemName = sourceInstance['SystemName']
-        sgInstanceName = (
-            self.get_or_create_default_sg(
-                conn, extraSpecs, storageSystemName))
-        try:
-            storageGroupInstance = conn.GetInstance(sgInstanceName)
-        except Exception:
-            exceptionMessage = (_(
-                "Unable to get the name of the storage group"))
-            LOG.error(exceptionMessage)
-            raise exception.VolumeBackendAPIException(
-                data=exceptionMessage)
 
-        @coordination.synchronized("emc-sg-{storageGroupName}")
-        def do_create_element_replica(storageGroupName):
-            if targetInstance is None and rsdInstance is None:
-                rc, job = conn.InvokeMethod(
-                    'CreateElementReplica', repServiceInstanceName,
-                    ElementName=cloneName,
-                    SyncType=self.utils.get_num(syncType, '16'),
-                    SourceElement=sourceInstance.path,
-                    Collections=[sgInstanceName])
-            else:
-                rc, job = self._create_element_replica_extra_params(
-                    conn, repServiceInstanceName, cloneName, syncType,
-                    sourceInstance, targetInstance, rsdInstance,
-                    sgInstanceName, copyState=copyState)
-
-            if rc != 0:
-                rc, errordesc = self.utils.wait_for_job_complete(conn, job,
-                                                                 extraSpecs)
-                if rc != 0:
-                    exceptionMessage = (_(
-                        "Error Create Cloned Volume: %(cloneName)s "
-                        "Return code: %(rc)lu. Error: %(error)s.")
-                        % {'cloneName': cloneName,
-                           'rc': rc,
-                           'error': errordesc})
-                    LOG.error(exceptionMessage)
-                    raise exception.VolumeBackendAPIException(
-                        data=exceptionMessage)
-
-            LOG.debug("InvokeMethod CreateElementReplica "
-                      "took: %(delta)s H:MM:SS.",
-                      {'delta': self.utils.get_time_delta(startTime,
-                                                          time.time())})
-            return rc, job
-        return do_create_element_replica(storageGroupInstance['ElementName'])
-
-    def _create_element_replica_extra_params(
-            self, conn, repServiceInstanceName, cloneName, syncType,
-            sourceInstance, targetInstance, rsdInstance, sgInstanceName,
-            copyState=None):
-        """CreateElementReplica using extra parameters.
-
-        :param conn: the connection to the ecom server
-        :param repServiceInstanceName: replication service
-        :param cloneName: clone volume name
-        :param syncType: 7=snapshot, 8=clone
-        :param sourceInstance: source volume instance
-        :param targetInstance: Target volume instance. Default None
-        :param rsdInstance: replication settingdata instance. Default None
-        :param sgInstanceName: pool instance name
-        :returns: int -- rc - return code
-        :returns: job - job object of the replica creation operation
-        """
-        syncType = self.utils.get_num(syncType, '16')
-        if targetInstance and rsdInstance:
+        if targetInstance is None:
+            LOG.debug("Create targetless replica: %(clone)s "
+                      "syncType: %(syncType)s  Source: %(source)s.",
+                      {'clone': cloneName,
+                       'syncType': syncType,
+                       'source': sourceInstance.path})
             rc, job = conn.InvokeMethod(
                 'CreateElementReplica', repServiceInstanceName,
-                ElementName=cloneName,
-                SyncType=syncType,
-                SourceElement=sourceInstance.path,
-                TargetElement=targetInstance.path,
-                ReplicationSettingData=rsdInstance)
-        elif rsdInstance:
+                ElementName=cloneName, SyncType=syncType,
+                SourceElement=sourceInstance.path)
+        else:
+            LOG.debug(
+                "Create replica: %(clone)s syncType: %(syncType)s "
+                "Source: %(source)s target: %(target)s.",
+                {'clone': cloneName,
+                 'syncType': syncType,
+                 'source': sourceInstance.path,
+                 'target': targetInstance.path})
             rc, job = conn.InvokeMethod(
                 'CreateElementReplica', repServiceInstanceName,
-                ElementName=cloneName,
-                SyncType=syncType,
-                SourceElement=sourceInstance.path,
-                ReplicationSettingData=rsdInstance,
-                Collections=[sgInstanceName])
-        elif targetInstance and copyState:
-            rc, job = conn.InvokeMethod(
-                'CreateElementReplica', repServiceInstanceName,
-                ElementName=cloneName,
-                SyncType=syncType,
-                SourceElement=sourceInstance.path,
-                TargetElement=targetInstance.path,
-                WaitForCopyState=copyState)
-        elif targetInstance:
-            rc, job = conn.InvokeMethod(
-                'CreateElementReplica', repServiceInstanceName,
-                ElementName=cloneName,
-                SyncType=syncType,
+                ElementName=cloneName, SyncType=syncType,
                 SourceElement=sourceInstance.path,
                 TargetElement=targetInstance.path)
+
+        if rc != 0L:
+            rc, errordesc = self.utils.wait_for_job_complete(conn, job,
+                                                             extraSpecs)
+            if rc != 0L:
+                exceptionMessage = (_(
+                    "Error Create Cloned Volume: %(cloneName)s "
+                    "Return code: %(rc)lu. Error: %(error)s.")
+                    % {'cloneName': cloneName,
+                       'rc': rc,
+                       'error': errordesc})
+                LOG.error(exceptionMessage)
+                raise exception.VolumeBackendAPIException(
+                    data=exceptionMessage)
+
+        LOG.debug("InvokeMethod CreateElementReplica "
+                  "took: %(delta)s H:MM:SS.",
+                  {'delta': self.utils.get_time_delta(startTime,
+                                                      time.time())})
         return rc, job
 
     def break_replication_relationship(
@@ -416,48 +283,38 @@ class EMCVMAXProvisionV3(object):
         """
         startTime = time.time()
 
-        @coordination.synchronized("emc-sg-{sgGroupName}")
-        def do_create_storage_group_v3(sgGroupName):
-            if slo and workload:
-                rc, job = conn.InvokeMethod(
-                    'CreateGroup',
-                    controllerConfigService,
-                    GroupName=groupName,
-                    Type=self.utils.get_num(4, '16'),
-                    EMCSRP=srp,
-                    EMCSLO=slo,
-                    EMCWorkload=workload)
-            else:
-                rc, job = conn.InvokeMethod(
-                    'CreateGroup',
-                    controllerConfigService,
-                    GroupName=groupName,
-                    Type=self.utils.get_num(4, '16'))
+        rc, job = conn.InvokeMethod(
+            'CreateGroup',
+            controllerConfigService,
+            GroupName=groupName,
+            Type=self.utils.get_num(4, '16'),
+            EMCSRP=srp,
+            EMCSLO=slo,
+            EMCWorkload=workload)
 
-            if rc != 0:
-                rc, errordesc = self.utils.wait_for_job_complete(
-                    conn, job, extraSpecs)
-                if rc != 0:
-                    LOG.error(_LE(
-                        "Error Create Group: %(groupName)s. "
-                        "Return code: %(rc)lu.  Error: %(error)s."),
-                        {'groupName': groupName,
-                         'rc': rc,
-                         'error': errordesc})
-                    raise
+        if rc != 0L:
+            rc, errordesc = self.utils.wait_for_job_complete(conn, job,
+                                                             extraSpecs)
+            if rc != 0L:
+                LOG.error(_LE(
+                    "Error Create Group: %(groupName)s. "
+                    "Return code: %(rc)lu.  Error: %(error)s."),
+                    {'groupName': groupName,
+                     'rc': rc,
+                     'error': errordesc})
+                raise
 
-            LOG.debug("InvokeMethod CreateGroup "
-                      "took: %(delta)s H:MM:SS.",
-                      {'delta': self.utils.get_time_delta(startTime,
-                                                          time.time())})
+        LOG.debug("InvokeMethod CreateGroup "
+                  "took: %(delta)s H:MM:SS.",
+                  {'delta': self.utils.get_time_delta(startTime,
+                                                      time.time())})
 
-            foundStorageGroupInstanceName = self._find_new_storage_group(
-                conn, job, groupName)
-            return foundStorageGroupInstanceName
+        foundStorageGroupInstanceName = self._find_new_storage_group(
+            conn, job, groupName)
 
-        return do_create_storage_group_v3(groupName)
+        return foundStorageGroupInstanceName
 
-    def get_storage_pool_capability(self, conn, poolInstanceName):
+    def _get_storage_pool_capability(self, conn, poolInstanceName):
         """Get the pool capability.
 
         :param conn: the connection information to the ecom server
@@ -475,7 +332,7 @@ class EMCVMAXProvisionV3(object):
 
         return storagePoolCapability
 
-    def get_storage_pool_setting(
+    def _get_storage_pool_setting(
             self, conn, storagePoolCapability, slo, workload):
         """Get the pool setting for pool capability.
 
@@ -499,16 +356,6 @@ class EMCVMAXProvisionV3(object):
             if matchString in settingInstanceID:
                 foundStoragePoolSetting = storagePoolSetting
                 break
-        if foundStoragePoolSetting is None:
-            exceptionMessage = (_(
-                "The array does not support the storage pool setting "
-                "for SLO %(slo)s and workload %(workload)s.  Please "
-                "check the array for valid SLOs and workloads.")
-                % {'slo': slo,
-                   'workload': workload})
-            LOG.error(exceptionMessage)
-            raise exception.VolumeBackendAPIException(
-                data=exceptionMessage)
         return foundStoragePoolSetting
 
     def _get_supported_size_range_for_SLO(
@@ -532,10 +379,10 @@ class EMCVMAXProvisionV3(object):
             ElementType=self.utils.get_num(3, '16'),
             Goal=storagePoolSettingInstanceName)
 
-        if rc != 0:
+        if rc != 0L:
             rc, errordesc = self.utils.wait_for_job_complete(
                 conn, supportedSizeDict, extraSpecs)
-            if rc != 0:
+            if rc != 0L:
                 exceptionMessage = (_(
                     "Cannot get supported size range for %(sps)s "
                     "Return code: %(rc)lu. Error: %(error)s.")
@@ -564,18 +411,26 @@ class EMCVMAXProvisionV3(object):
         :param slo: slo string e.g Bronze
         :param workload: workload string e.g DSS
         :param extraSpecs: additional info
-        :returns: supportedSizeDict
+        :returns: maximumVolumeSize - the maximum volume size supported
+        :returns: minimumVolumeSize - the minimum volume size supported
         """
-        supportedSizeDict = {}
-        storagePoolCapabilityInstanceName = self.get_storage_pool_capability(
+        maximumVolumeSize = None
+        minimumVolumeSize = None
+
+        storagePoolCapabilityInstanceName = self._get_storage_pool_capability(
             conn, poolInstanceName)
         if storagePoolCapabilityInstanceName:
-            storagePoolSettingInstanceName = self.get_storage_pool_setting(
+            storagePoolSettingInstanceName = self._get_storage_pool_setting(
                 conn, storagePoolCapabilityInstanceName, slo, workload)
-            supportedSizeDict = self._get_supported_size_range_for_SLO(
-                conn, storageConfigService, poolInstanceName,
-                storagePoolSettingInstanceName, extraSpecs)
-        return supportedSizeDict
+            if storagePoolCapabilityInstanceName:
+                supportedSizeDict = self._get_supported_size_range_for_SLO(
+                    conn, storageConfigService, poolInstanceName,
+                    storagePoolSettingInstanceName, extraSpecs)
+
+                maximumVolumeSize = supportedSizeDict['MaximumVolumeSize']
+                minimumVolumeSize = supportedSizeDict['MinimumVolumeSize']
+
+        return maximumVolumeSize, minimumVolumeSize
 
     def activate_snap_relationship(
             self, conn, repServiceInstanceName, syncInstanceName, extraSpecs):
@@ -590,7 +445,7 @@ class EMCVMAXProvisionV3(object):
         :returns: job object of the replica creation operation
         """
         # Operation 4: activate the snapVx.
-        operation = ACTIVATESNAPVX
+        operation = self.utils.get_num(4, '16')
 
         LOG.debug("Activate snap: %(sv)s  operation: %(operation)s.",
                   {'sv': syncInstanceName, 'operation': operation})
@@ -611,7 +466,7 @@ class EMCVMAXProvisionV3(object):
         :returns: job object of the replica creation operation
         """
         # Operation 4: activate the snapVx.
-        operation = DEACTIVATESNAPVX
+        operation = self.utils.get_num(19, '16')
 
         LOG.debug("Return snap resource back to pool: "
                   "%(sv)s  operation: %(operation)s.",
@@ -644,7 +499,7 @@ class EMCVMAXProvisionV3(object):
 
         rc, job = conn.InvokeMethod(
             'ModifyReplicaSynchronization', repServiceInstanceName,
-            Operation=self.utils.get_num(operation, '16'),
+            Operation=operation,
             Synchronization=syncInstanceName,
             Force=force)
 
@@ -652,10 +507,10 @@ class EMCVMAXProvisionV3(object):
                   "operation: %(operation)s  Return code: %(rc)lu.",
                   {'sv': syncInstanceName, 'operation': operation, 'rc': rc})
 
-        if rc != 0:
+        if rc != 0L:
             rc, errordesc = self.utils.wait_for_job_complete(conn, job,
                                                              extraSpecs)
-            if rc != 0:
+            if rc != 0L:
                 exceptionMessage = (_(
                     "Error modify replica synchronization: %(sv)s "
                     "operation: %(operation)s. "
@@ -699,18 +554,20 @@ class EMCVMAXProvisionV3(object):
              'relationName': relationName,
              'srcGroup': srcGroupInstanceName,
              'tgtGroup': tgtGroupInstanceName})
+        # 7 for snap.
+        syncType = 7
         rc, job = conn.InvokeMethod(
             'CreateGroupReplica',
             replicationService,
             RelationshipName=relationName,
             SourceGroup=srcGroupInstanceName,
             TargetGroup=tgtGroupInstanceName,
-            SyncType=self.utils.get_num(SNAPSYNCTYPE, '16'))
+            SyncType=self.utils.get_num(syncType, '16'))
 
-        if rc != 0:
+        if rc != 0L:
             rc, errordesc = self.utils.wait_for_job_complete(conn, job,
                                                              extraSpecs)
-            if rc != 0:
+            if rc != 0L:
                 exceptionMsg = (_("Error CreateGroupReplica: "
                                   "source: %(source)s target: %(target)s. "
                                   "Return code: %(rc)lu. Error: %(error)s.")
@@ -721,160 +578,3 @@ class EMCVMAXProvisionV3(object):
                 LOG.error(exceptionMsg)
                 raise exception.VolumeBackendAPIException(data=exceptionMsg)
         return rc, job
-
-    def get_srp_pool_stats(self, conn, arrayInfo):
-        """Get the totalManagedSpace, remainingManagedSpace.
-
-        :param conn: the connection to the ecom server
-        :param arrayInfo: the array dict
-        :returns: totalCapacityGb
-        :returns: remainingCapacityGb
-        :returns: subscribedCapacityGb
-        :returns: array_reserve_percent
-        """
-        totalCapacityGb = -1
-        remainingCapacityGb = -1
-        subscribedCapacityGb = -1
-        array_reserve_percent = -1
-        storageSystemInstanceName = self.utils.find_storageSystem(
-            conn, arrayInfo['SerialNumber'])
-
-        srpPoolInstanceNames = conn.AssociatorNames(
-            storageSystemInstanceName,
-            ResultClass='Symm_SRPStoragePool')
-
-        for srpPoolInstanceName in srpPoolInstanceNames:
-            poolnameStr = self.utils.get_pool_name(conn, srpPoolInstanceName)
-
-            if six.text_type(arrayInfo['PoolName']) == (
-                    six.text_type(poolnameStr)):
-                try:
-                    # Check that pool hasn't suddently been deleted.
-                    srpPoolInstance = conn.GetInstance(srpPoolInstanceName)
-                    propertiesList = srpPoolInstance.properties.items()
-                    for properties in propertiesList:
-                        if properties[0] == 'TotalManagedSpace':
-                            cimProperties = properties[1]
-                            totalManagedSpace = cimProperties.value
-                            totalCapacityGb = self.utils.convert_bits_to_gbs(
-                                totalManagedSpace)
-                        elif properties[0] == 'RemainingManagedSpace':
-                            cimProperties = properties[1]
-                            remainingManagedSpace = cimProperties.value
-                            remainingCapacityGb = (
-                                self.utils.convert_bits_to_gbs(
-                                    remainingManagedSpace))
-                        elif properties[0] == 'EMCSubscribedCapacity':
-                            cimProperties = properties[1]
-                            subscribedManagedSpace = cimProperties.value
-                            subscribedCapacityGb = (
-                                self.utils.convert_bits_to_gbs(
-                                    subscribedManagedSpace))
-                        elif properties[0] == 'EMCPercentReservedCapacity':
-                            cimProperties = properties[1]
-                            array_reserve_percent = int(cimProperties.value)
-                except Exception:
-                    pass
-                remainingSLOCapacityGb = (
-                    self._get_remaining_slo_capacity_wlp(
-                        conn, srpPoolInstanceName, arrayInfo,
-                        storageSystemInstanceName['Name']))
-                if remainingSLOCapacityGb != -1:
-                    remainingCapacityGb = remainingSLOCapacityGb
-                else:
-                    LOG.warning(_LW(
-                        "Remaining capacity %(remainingCapacityGb)s "
-                        "GBs is determined from SRP pool capacity "
-                        "and not the SLO capacity. Performance may "
-                        "not be what you expect."),
-                        {'remainingCapacityGb': remainingCapacityGb})
-
-        return (totalCapacityGb, remainingCapacityGb, subscribedCapacityGb,
-                array_reserve_percent)
-
-    def _get_remaining_slo_capacity_wlp(self, conn, srpPoolInstanceName,
-                                        arrayInfo, systemName):
-        """Get the remaining SLO capacity.
-
-        This is derived from the WLP portion of Unisphere. Please
-        see the SMIProvider doc and the readme doc for details.
-
-        :param conn: the connection to the ecom server
-        :param srpPoolInstanceName: SRP instance name
-        :param arrayInfo: the array dict
-        :param systemName: the system name
-        :returns: remainingCapacityGb
-        """
-        remainingCapacityGb = -1
-        if arrayInfo['SLO']:
-            storageConfigService = (
-                self.utils.find_storage_configuration_service(
-                    conn, systemName))
-
-            supportedSizeDict = (
-                self.get_volume_range(
-                    conn, storageConfigService, srpPoolInstanceName,
-                    arrayInfo['SLO'], arrayInfo['Workload'],
-                    None))
-            try:
-                if supportedSizeDict['EMCInformationSource'] == INFO_SRC_V3:
-                    remainingCapacityGb = self.utils.convert_bits_to_gbs(
-                        supportedSizeDict['EMCRemainingSLOCapacity'])
-                    LOG.debug("Received remaining SLO Capacity "
-                              "%(remainingCapacityGb)s GBs for SLO "
-                              "%(SLO)s and workload %(workload)s.",
-                              {'remainingCapacityGb': remainingCapacityGb,
-                               'SLO': arrayInfo['SLO'],
-                               'workload': arrayInfo['Workload']})
-            except KeyError:
-                pass
-        return remainingCapacityGb
-
-    def extend_volume_in_SG(
-            self, conn, storageConfigService, volumeInstanceName,
-            volumeName, volumeSize, extraSpecs):
-        """Extend a volume instance.
-
-        :param conn: connection to the ecom server
-        :param storageConfigservice: the storage configuration service
-        :param volumeInstanceName: the volume instance name
-        :param volumeName: the volume name (String)
-        :param volumeSize: the volume size
-        :param extraSpecs: additional info
-        :returns: volumeDict
-        :returns: int -- return code
-        :raises: VolumeBackendAPIException
-        """
-        startTime = time.time()
-
-        rc, job = conn.InvokeMethod(
-            'CreateOrModifyElementFromStoragePool',
-            storageConfigService, TheElement=volumeInstanceName,
-            Size=self.utils.get_num(volumeSize, '64'))
-
-        LOG.debug("Extend Volume: %(volumename)s. Return code: %(rc)lu.",
-                  {'volumename': volumeName,
-                   'rc': rc})
-
-        if rc != 0:
-            rc, error_desc = self.utils.wait_for_job_complete(conn, job,
-                                                              extraSpecs)
-            if rc != 0:
-                exceptionMessage = (_(
-                    "Error Extend Volume: %(volumeName)s. "
-                    "Return code: %(rc)lu.  Error: %(error)s.")
-                    % {'volumeName': volumeName,
-                       'rc': rc,
-                       'error': error_desc})
-                LOG.error(exceptionMessage)
-                raise exception.VolumeBackendAPIException(
-                    data=exceptionMessage)
-
-        LOG.debug("InvokeMethod CreateOrModifyElementFromStoragePool "
-                  "took: %(delta)s H:MM:SS.",
-                  {'delta': self.utils.get_time_delta(startTime,
-                                                      time.time())})
-
-        # Find the newly created volume.
-        volumeDict = self.get_volume_dict_from_job(conn, job['Job'])
-        return volumeDict, rc
