@@ -1,5 +1,4 @@
 # Copyright (c) 2014 Clinton Knight.  All rights reserved.
-# Copyright (c) 2016 Mike Rooney. All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,22 +15,20 @@
 Volume driver for NetApp Data ONTAP (C-mode) iSCSI storage systems.
 """
 
-from cinder import interface
+from oslo_log import log as logging
+
 from cinder.volume import driver
 from cinder.volume.drivers.netapp.dataontap import block_cmode
 
 
-@interface.volumedriver
-class NetAppCmodeISCSIDriver(driver.BaseVD,
-                             driver.ManageableVD):
+LOG = logging.getLogger(__name__)
+
+
+class NetAppCmodeISCSIDriver(driver.ISCSIDriver):
     """NetApp C-mode iSCSI volume driver."""
 
     DRIVER_NAME = 'NetApp_iSCSI_Cluster_direct'
 
-    # ThirdPartySystems wiki page
-    CI_WIKI_NAME = "NetApp_CI"
-    VERSION = block_cmode.NetAppBlockStorageCmodeLibrary.VERSION
-
     def __init__(self, *args, **kwargs):
         super(NetAppCmodeISCSIDriver, self).__init__(*args, **kwargs)
         self.library = block_cmode.NetAppBlockStorageCmodeLibrary(
@@ -44,13 +41,13 @@ class NetAppCmodeISCSIDriver(driver.BaseVD,
         self.library.check_for_setup_error()
 
     def create_volume(self, volume):
-        return self.library.create_volume(volume)
+        self.library.create_volume(volume)
 
     def create_volume_from_snapshot(self, volume, snapshot):
-        return self.library.create_volume_from_snapshot(volume, snapshot)
+        self.library.create_volume_from_snapshot(volume, snapshot)
 
     def create_cloned_volume(self, volume, src_vref):
-        return self.library.create_cloned_volume(volume, src_vref)
+        self.library.create_cloned_volume(volume, src_vref)
 
     def delete_volume(self, volume):
         self.library.delete_volume(volume)
@@ -62,15 +59,7 @@ class NetAppCmodeISCSIDriver(driver.BaseVD,
         self.library.delete_snapshot(snapshot)
 
     def get_volume_stats(self, refresh=False):
-        return self.library.get_volume_stats(refresh,
-                                             self.get_filter_function(),
-                                             self.get_goodness_function())
-
-    def get_default_filter_function(self):
-        return self.library.get_default_filter_function()
-
-    def get_default_goodness_function(self):
-        return self.library.get_default_goodness_function()
+        return self.library.get_volume_stats(refresh)
 
     def extend_volume(self, volume, new_size):
         self.library.extend_volume(volume, new_size)
@@ -78,7 +67,7 @@ class NetAppCmodeISCSIDriver(driver.BaseVD,
     def ensure_export(self, context, volume):
         return self.library.ensure_export(context, volume)
 
-    def create_export(self, context, volume, connector):
+    def create_export(self, context, volume):
         return self.library.create_export(context, volume)
 
     def remove_export(self, context, volume):
@@ -102,31 +91,3 @@ class NetAppCmodeISCSIDriver(driver.BaseVD,
 
     def get_pool(self, volume):
         return self.library.get_pool(volume)
-
-    def create_group(self, context, group):
-        return self.library.create_group(group)
-
-    def delete_group(self, context, group, volumes):
-        return self.library.delete_group(group, volumes)
-
-    def update_group(self, context, group, add_volumes=None,
-                     remove_volumes=None):
-        return self.library.update_group(group, add_volumes=None,
-                                         remove_volumes=None)
-
-    def create_group_snapshot(self, context, group_snapshot, snapshots):
-        return self.library.create_group_snapshot(group_snapshot, snapshots)
-
-    def delete_group_snapshot(self, context, group_snapshot, snapshots):
-        return self.library.delete_group_snapshot(group_snapshot, snapshots)
-
-    def create_group_from_src(self, context, group, volumes,
-                              group_snapshot=None, snapshots=None,
-                              source_group=None, source_vols=None):
-        return self.library.create_group_from_src(
-            group, volumes, group_snapshot=group_snapshot, snapshots=snapshots,
-            source_group=source_group, source_vols=source_vols)
-
-    def failover_host(self, context, volumes, secondary_id=None, groups=None):
-        return self.library.failover_host(
-            context, volumes, secondary_id=secondary_id)
