@@ -15,25 +15,30 @@
 #
 """HP LeftHand SAN ISCSI REST Proxy."""
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import excutils
+from oslo_utils import importutils
+from oslo_utils import units
 
 from cinder import context
 from cinder import exception
-from cinder.i18n import _
-from cinder.openstack.common import log as logging
-from cinder.openstack.common import units
-from cinder.volume.driver import ISCSIDriver
+from cinder.i18n import _, _LE, _LI, _LW
+from cinder.volume import driver
 from cinder.volume import utils
 from cinder.volume import volume_types
 
+import six
+
+import math
+import re
+
 LOG = logging.getLogger(__name__)
 
-try:
-    import hplefthandclient
-    from hplefthandclient import client
+hplefthandclient = importutils.try_import("hplefthandclient")
+if hplefthandclient:
+    from hplefthandclient import client as hp_lh_client
     from hplefthandclient import exceptions as hpexceptions
-except ImportError:
-    import cinder.tests.fake_hp_lefthand_client as hplefthandclient
 
 hplefthand_opts = [
     cfg.StrOpt('hplefthand_api_url',
@@ -63,6 +68,7 @@ hplefthand_opts = [
 CONF = cfg.CONF
 CONF.register_opts(hplefthand_opts)
 
+MIN_API_VERSION = "1.1"
 
 # map the extra spec key to the REST client option key
 extra_specs_key_map = {
@@ -80,7 +86,7 @@ extra_specs_value_map = {
 }
 
 
-class HPLeftHandRESTProxy(ISCSIDriver):
+class HPLeftHandRESTProxy(driver.ISCSIDriver):
     """Executes REST commands relating to HP/LeftHand SAN ISCSI volumes.
 
     Version history:
@@ -93,9 +99,15 @@ class HPLeftHandRESTProxy(ISCSIDriver):
                 improvement
         1.0.5 - Fixed bug #1311350, Live-migration of an instance when
                 attached to a volume was causing an error.
+        1.0.6 - Removing locks bug #1395953
+        1.0.7 - Fixed bug #1353137, Server was not removed from the HP
+                Lefthand backend after the last volume was detached.
+        1.0.8 - Fixed bug #1418201, A cloned volume fails to attach.
+        1.0.9 - Adding support for manage/unmanage.
+        1.0.10 - Add stats for goodness_function and filter_function
     """
 
-    VERSION = "1.0.5"
+    VERSION = "1.0.10"
 
     device_stats = {}
 
@@ -109,24 +121,36 @@ class HPLeftHandRESTProxy(ISCSIDriver):
         # so we need to use it as a separator
         self.DRIVER_LOCATION = self.__class__.__name__ + ' %(cluster)s %(vip)s'
 
+    def _login(self):
+        client = self.do_setup(None)
+        return client
+
+    def _logout(self, client):
+        client.logout()
+
+    def _create_client(self):
+        return hp_lh_client.HPLeftHandClient(
+            self.configuration.hplefthand_api_url)
+
     def do_setup(self, context):
         """Set up LeftHand client."""
         try:
-            self.client = client.HPLeftHandClient(
-                self.configuration.hplefthand_api_url)
-            self.client.login(
+            client = self._create_client()
+            client.login(
                 self.configuration.hplefthand_username,
                 self.configuration.hplefthand_password)
 
             if self.configuration.hplefthand_debug:
-                self.client.debug_rest(True)
+                client.debug_rest(True)
 
-            cluster_info = self.client.getClusterByName(
+            cluster_info = client.getClusterByName(
                 self.configuration.hplefthand_clustername)
             self.cluster_id = cluster_info['id']
             virtual_ips = cluster_info['virtualIPAddresses']
             self.cluster_vip = virtual_ips[0]['ipV4Address']
-            self._update_backend_status()
+            self._update_backend_status(client)
+
+            return client
         except hpexceptions.HTTPNotFound:
             raise exception.DriverNotInitialized(
                 _('LeftHand cluster not found'))
@@ -134,7 +158,21 @@ class HPLeftHandRESTProxy(ISCSIDriver):
             raise exception.DriverNotInitialized(ex)
 
     def check_for_setup_error(self):
-        pass
+        """Checks for incorrect LeftHand API being used on backend."""
+        client = self._login()
+        try:
+            self.api_version = client.getApiVersion()
+
+            LOG.info(_LI("HPLeftHand API version %s"), self.api_version)
+
+            if self.api_version < MIN_API_VERSION:
+                LOG.warning(_LW("HPLeftHand API is version %(current)s. "
+                                "A minimum version of %(min)s is needed for "
+                                "manage/unmanage support."),
+                            {'current': self.api_version,
+                             'min': MIN_API_VERSION})
+        finally:
+            self._logout(client)
 
     def get_version_string(self):
         return (_('REST %(proxy_ver)s hplefthandclient %(rest_ver)s') % {
@@ -143,6 +181,7 @@ class HPLeftHandRESTProxy(ISCSIDriver):
 
     def create_volume(self, volume):
         """Creates a volume."""
+        client = self._login()
         try:
             # get the extra specs of interest from this volume's volume type
             volume_extra_specs = self._get_volume_extra_specs(volume)
@@ -170,7 +209,7 @@ class HPLeftHandRESTProxy(ISCSIDriver):
             clusterName = self.configuration.hplefthand_clustername
             optional['clusterName'] = clusterName
 
-            volume_info = self.client.createVolume(
+            volume_info = client.createVolume(
                 volume['name'], self.cluster_id,
                 volume['size'] * units.Gi,
                 optional)
@@ -178,47 +217,59 @@ class HPLeftHandRESTProxy(ISCSIDriver):
             return self._update_provider(volume_info)
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def delete_volume(self, volume):
         """Deletes a volume."""
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(volume['name'])
-            self.client.deleteVolume(volume_info['id'])
+            volume_info = client.getVolumeByName(volume['name'])
+            client.deleteVolume(volume_info['id'])
         except hpexceptions.HTTPNotFound:
-            LOG.error(_("Volume did not exist. It will not be deleted"))
+            LOG.error(_LE("Volume did not exist. It will not be deleted"))
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def extend_volume(self, volume, new_size):
         """Extend the size of an existing volume."""
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(volume['name'])
+            volume_info = client.getVolumeByName(volume['name'])
 
             # convert GB to bytes
             options = {'size': int(new_size) * units.Gi}
-            self.client.modifyVolume(volume_info['id'], options)
+            client.modifyVolume(volume_info['id'], options)
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def create_snapshot(self, snapshot):
         """Creates a snapshot."""
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(snapshot['volume_name'])
+            volume_info = client.getVolumeByName(snapshot['volume_name'])
 
             option = {'inheritAccess': True}
-            self.client.createSnapshot(snapshot['name'],
-                                       volume_info['id'],
-                                       option)
+            client.createSnapshot(snapshot['name'],
+                                  volume_info['id'],
+                                  option)
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def delete_snapshot(self, snapshot):
         """Deletes a snapshot."""
+        client = self._login()
         try:
-            snap_info = self.client.getSnapshotByName(snapshot['name'])
-            self.client.deleteSnapshot(snap_info['id'])
+            snap_info = client.getSnapshotByName(snapshot['name'])
+            client.deleteSnapshot(snap_info['id'])
         except hpexceptions.HTTPNotFound:
-            LOG.error(_("Snapshot did not exist. It will not be deleted"))
+            LOG.error(_LE("Snapshot did not exist. It will not be deleted"))
         except hpexceptions.HTTPServerError as ex:
             in_use_msg = 'cannot be deleted because it is a clone point'
             if in_use_msg in ex.get_description():
@@ -228,15 +279,21 @@ class HPLeftHandRESTProxy(ISCSIDriver):
 
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
-    def get_volume_stats(self, refresh):
+    def get_volume_stats(self, refresh=False):
         """Gets volume stats."""
-        if refresh:
-            self._update_backend_status()
+        client = self._login()
+        try:
+            if refresh:
+                self._update_backend_status(client)
 
-        return self.device_stats
+            return self.device_stats
+        finally:
+            self._logout(client)
 
-    def _update_backend_status(self):
+    def _update_backend_status(self, client):
         data = {}
         backend_name = self.configuration.safe_get('volume_backend_name')
         data['volume_backend_name'] = backend_name or self.__class__.__name__
@@ -247,7 +304,7 @@ class HPLeftHandRESTProxy(ISCSIDriver):
             'cluster': self.configuration.hplefthand_clustername,
             'vip': self.cluster_vip})
 
-        cluster_info = self.client.getCluster(self.cluster_id)
+        cluster_info = client.getCluster(self.cluster_id)
 
         total_capacity = cluster_info['spaceTotal']
         free_capacity = cluster_info['spaceAvailable']
@@ -256,6 +313,24 @@ class HPLeftHandRESTProxy(ISCSIDriver):
         data['total_capacity_gb'] = int(total_capacity) / units.Gi
         data['free_capacity_gb'] = int(free_capacity) / units.Gi
 
+        # Collect some stats
+        capacity_utilization = (
+            (float(total_capacity - free_capacity) /
+             float(total_capacity)) * 100)
+        # Don't have a better way to get the total number volumes
+        # so try to limit the size of data for now. Once new lefthand API is
+        # available, replace this call.
+        total_volumes = 0
+        volumes = client.getVolumes(
+            cluster=self.configuration.hplefthand_clustername,
+            fields=['members[id]', 'members[clusterName]'])
+        if volumes:
+            total_volumes = volumes['total']
+        data['capacity_utilization'] = capacity_utilization
+        data['total_volumes'] = total_volumes
+        data['filter_function'] = self.get_filter_function()
+        data['goodness_function'] = self.get_goodness_function()
+
         self.device_stats = data
 
     def initialize_connection(self, volume, connector):
@@ -265,9 +340,10 @@ class HPLeftHandRESTProxy(ISCSIDriver):
         used from that host. HP VSA requires a volume to be assigned
         to a server.
         """
+        client = self._login()
         try:
-            server_info = self._create_server(connector)
-            volume_info = self.client.getVolumeByName(volume['name'])
+            server_info = self._create_server(connector, client)
+            volume_info = client.getVolumeByName(volume['name'])
 
             access_already_enabled = False
             if volume_info['iscsiSessions'] is not None:
@@ -280,14 +356,14 @@ class HPLeftHandRESTProxy(ISCSIDriver):
                         break
 
             if not access_already_enabled:
-                self.client.addServerAccess(
+                client.addServerAccess(
                     volume_info['id'],
                     server_info['id'])
 
             iscsi_properties = self._get_iscsi_properties(volume)
 
-            if ('chapAuthenticationRequired' in server_info
-                    and server_info['chapAuthenticationRequired']):
+            if ('chapAuthenticationRequired' in server_info and
+                    server_info['chapAuthenticationRequired']):
                 iscsi_properties['auth_method'] = 'CHAP'
                 iscsi_properties['auth_username'] = connector['initiator']
                 iscsi_properties['auth_password'] = (
@@ -296,35 +372,58 @@ class HPLeftHandRESTProxy(ISCSIDriver):
             return {'driver_volume_type': 'iscsi', 'data': iscsi_properties}
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def terminate_connection(self, volume, connector, **kwargs):
         """Unassign the volume from the host."""
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(volume['name'])
-            server_info = self.client.getServerByName(connector['host'])
-            self.client.removeServerAccess(
+            volume_info = client.getVolumeByName(volume['name'])
+            server_info = client.getServerByName(connector['host'])
+            volume_list = client.findServerVolumes(server_info['name'])
+
+            removeServer = True
+            for entry in volume_list:
+                if entry['id'] != volume_info['id']:
+                    removeServer = False
+                    break
+
+            client.removeServerAccess(
                 volume_info['id'],
                 server_info['id'])
+
+            if removeServer:
+                client.deleteServer(server_info['id'])
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def create_volume_from_snapshot(self, volume, snapshot):
         """Creates a volume from a snapshot."""
+        client = self._login()
         try:
-            snap_info = self.client.getSnapshotByName(snapshot['name'])
-            volume_info = self.client.cloneSnapshot(
+            snap_info = client.getSnapshotByName(snapshot['name'])
+            volume_info = client.cloneSnapshot(
                 volume['name'],
                 snap_info['id'])
             return self._update_provider(volume_info)
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def create_cloned_volume(self, volume, src_vref):
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(src_vref['name'])
-            self.client.cloneVolume(volume['name'], volume_info['id'])
+            volume_info = client.getVolumeByName(src_vref['name'])
+            clone_info = client.cloneVolume(volume['name'], volume_info['id'])
+            return self._update_provider(clone_info)
         except Exception as ex:
             raise exception.VolumeBackendAPIException(ex)
+        finally:
+            self._logout(client)
 
     def _get_volume_extra_specs(self, volume):
         """Get extra specs from a volume."""
@@ -357,8 +456,8 @@ class HPLeftHandRESTProxy(ISCSIDriver):
                 client_value = value_map[value]
                 client_options[client_key] = client_value
             except KeyError:
-                LOG.error(_("'%(value)s' is an invalid value "
-                            "for extra spec '%(key)s'") %
+                LOG.error(_LE("'%(value)s' is an invalid value "
+                              "for extra spec '%(key)s'") %
                           {'value': value, 'key': key})
         return client_options
 
@@ -370,18 +469,18 @@ class HPLeftHandRESTProxy(ISCSIDriver):
         return {'provider_location': (
             "%s %s %s" % (iscsi_portal, volume_info['iscsiIqn'], 0))}
 
-    def _create_server(self, connector):
+    def _create_server(self, connector, client):
         server_info = None
         chap_enabled = self.configuration.hplefthand_iscsi_chap_enabled
         try:
-            server_info = self.client.getServerByName(connector['host'])
+            server_info = client.getServerByName(connector['host'])
             chap_secret = server_info['chapTargetSecret']
             if not chap_enabled and chap_secret:
-                LOG.warning(_('CHAP secret exists for host %s but CHAP is '
-                              'disabled') % connector['host'])
+                LOG.warning(_LW('CHAP secret exists for host %s but CHAP is '
+                                'disabled') % connector['host'])
             if chap_enabled and chap_secret is None:
-                LOG.warning(_('CHAP is enabled, but server secret not '
-                              'configured on server %s') % connector['host'])
+                LOG.warning(_LW('CHAP is enabled, but server secret not '
+                                'configured on server %s') % connector['host'])
             return server_info
         except hpexceptions.HTTPNotFound:
             # server does not exist, so create one
@@ -394,9 +493,10 @@ class HPLeftHandRESTProxy(ISCSIDriver):
                         'chapTargetSecret': chap_secret,
                         'chapAuthenticationRequired': True
                         }
-        server_info = self.client.createServer(connector['host'],
-                                               connector['initiator'],
-                                               optional)
+
+        server_info = client.createServer(connector['host'],
+                                          connector['initiator'],
+                                          optional)
         return server_info
 
     def create_export(self, context, volume):
@@ -426,12 +526,10 @@ class HPLeftHandRESTProxy(ISCSIDriver):
                                                     'new_type': new_type,
                                                     'diff': diff,
                                                     'host': host})
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(volume['name'])
-        except hpexceptions.HTTPNotFound:
-            raise exception.VolumeNotFound(volume_id=volume['id'])
+            volume_info = client.getVolumeByName(volume['name'])
 
-        try:
             # pick out the LH extra specs
             new_extra_specs = dict(new_type).get('extra_specs')
             lh_extra_specs = self._get_lh_extra_specs(
@@ -450,11 +548,14 @@ class HPLeftHandRESTProxy(ISCSIDriver):
             # map extra specs to LeftHand options
             options = self._map_extra_specs(changed_extra_specs)
             if len(options) > 0:
-                self.client.modifyVolume(volume_info['id'], options)
+                client.modifyVolume(volume_info['id'], options)
             return True
-
+        except hpexceptions.HTTPNotFound:
+            raise exception.VolumeNotFound(volume_id=volume['id'])
         except Exception as ex:
             LOG.warning("%s" % ex)
+        finally:
+            self._logout(client)
 
         return False
 
@@ -491,60 +592,240 @@ class HPLeftHandRESTProxy(ISCSIDriver):
 
         host_location = host['capabilities']['location_info']
         (driver, cluster, vip) = host_location.split(' ')
+        client = self._login()
         try:
             # get the cluster info, if it exists and compare
-            cluster_info = self.client.getClusterByName(cluster)
-            LOG.debug('Clister info: %s' % cluster_info)
+            cluster_info = client.getClusterByName(cluster)
+            LOG.debug('Cluster info: %s' % cluster_info)
             virtual_ips = cluster_info['virtualIPAddresses']
 
             if driver != self.__class__.__name__:
-                LOG.info(_("Cannot provide backend assisted migration for "
-                           "volume: %s because volume is from a different "
-                           "backend.") % volume['name'])
+                LOG.info(_LI("Cannot provide backend assisted migration for "
+                             "volume: %s because volume is from a different "
+                             "backend.") % volume['name'])
                 return false_ret
             if vip != virtual_ips[0]['ipV4Address']:
-                LOG.info(_("Cannot provide backend assisted migration for "
-                           "volume: %s because cluster exists in different "
-                           "management group.") % volume['name'])
+                LOG.info(_LI("Cannot provide backend assisted migration for "
+                             "volume: %s because cluster exists in different "
+                             "management group.") % volume['name'])
                 return false_ret
 
         except hpexceptions.HTTPNotFound:
-            LOG.info(_("Cannot provide backend assisted migration for "
-                       "volume: %s because cluster exists in different "
-                       "management group.") % volume['name'])
+            LOG.info(_LI("Cannot provide backend assisted migration for "
+                         "volume: %s because cluster exists in different "
+                         "management group.") % volume['name'])
             return false_ret
+        finally:
+            self._logout(client)
 
+        client = self._login()
         try:
-            volume_info = self.client.getVolumeByName(volume['name'])
+            volume_info = client.getVolumeByName(volume['name'])
             LOG.debug('Volume info: %s' % volume_info)
 
             # can't migrate if server is attached
             if volume_info['iscsiSessions'] is not None:
-                LOG.info(_("Cannot provide backend assisted migration "
-                           "for volume: %s because the volume has been "
-                           "exported.") % volume['name'])
+                LOG.info(_LI("Cannot provide backend assisted migration "
+                             "for volume: %s because the volume has been "
+                             "exported.") % volume['name'])
                 return false_ret
 
             # can't migrate if volume has snapshots
-            snap_info = self.client.getVolume(
+            snap_info = client.getVolume(
                 volume_info['id'],
                 'fields=snapshots,snapshots[resource[members[name]]]')
             LOG.debug('Snapshot info: %s' % snap_info)
             if snap_info['snapshots']['resource'] is not None:
-                LOG.info(_("Cannot provide backend assisted migration "
-                           "for volume: %s because the volume has "
-                           "snapshots.") % volume['name'])
+                LOG.info(_LI("Cannot provide backend assisted migration "
+                             "for volume: %s because the volume has "
+                             "snapshots.") % volume['name'])
                 return false_ret
 
             options = {'clusterName': cluster}
-            self.client.modifyVolume(volume_info['id'], options)
+            client.modifyVolume(volume_info['id'], options)
         except hpexceptions.HTTPNotFound:
-            LOG.info(_("Cannot provide backend assisted migration for "
-                       "volume: %s because volume does not exist in this "
-                       "management group.") % volume['name'])
+            LOG.info(_LI("Cannot provide backend assisted migration for "
+                         "volume: %s because volume does not exist in this "
+                         "management group.") % volume['name'])
             return false_ret
         except hpexceptions.HTTPServerError as ex:
             LOG.error(ex)
             return false_ret
+        finally:
+            self._logout(client)
 
         return (True, None)
+
+    def manage_existing(self, volume, existing_ref):
+        """Manage an existing LeftHand volume.
+
+        existing_ref is a dictionary of the form:
+        {'source-name': <name of the virtual volume>}
+        """
+        # Check API Version
+        self._check_api_version()
+
+        target_vol_name = self._get_existing_volume_ref_name(existing_ref)
+
+        # Check for the existence of the virtual volume.
+        client = self._login()
+        try:
+            volume_info = client.getVolumeByName(target_vol_name)
+        except hpexceptions.HTTPNotFound:
+            err = (_("Virtual volume '%s' doesn't exist on array.") %
+                   target_vol_name)
+            LOG.error(err)
+            raise exception.InvalidInput(reason=err)
+        finally:
+            self._logout(client)
+
+        # Generate the new volume information based on the new ID.
+        new_vol_name = 'volume-' + volume['id']
+
+        volume_type = None
+        if volume['volume_type_id']:
+            try:
+                volume_type = self._get_volume_type(volume['volume_type_id'])
+            except Exception:
+                reason = (_("Volume type ID '%s' is invalid.") %
+                          volume['volume_type_id'])
+                raise exception.ManageExistingVolumeTypeMismatch(reason=reason)
+
+        new_vals = {"name": new_vol_name}
+
+        client = self._login()
+        try:
+            # Update the existing volume with the new name.
+            client.modifyVolume(volume_info['id'], new_vals)
+        finally:
+            self._logout(client)
+
+        LOG.info(_LI("Virtual volume '%(ref)s' renamed to '%(new)s'."),
+                 {'ref': existing_ref['source-name'], 'new': new_vol_name})
+
+        display_name = None
+        if volume['display_name']:
+            display_name = volume['display_name']
+
+        if volume_type:
+            LOG.info(_LI("Virtual volume %(disp)s '%(new)s' is "
+                         "being retyped."),
+                     {'disp': display_name, 'new': new_vol_name})
+
+            try:
+                self.retype(None,
+                            volume,
+                            volume_type,
+                            volume_type['extra_specs'],
+                            volume['host'])
+                LOG.info(_LI("Virtual volume %(disp)s successfully retyped to "
+                             "%(new_type)s."),
+                         {'disp': display_name,
+                          'new_type': volume_type.get('name')})
+            except Exception:
+                with excutils.save_and_reraise_exception():
+                    LOG.warning(_LW("Failed to manage virtual volume %(disp)s "
+                                    "due to error during retype."),
+                                {'disp': display_name})
+                    # Try to undo the rename and clear the new comment.
+                    client = self._login()
+                    try:
+                        client.modifyVolume(
+                            volume_info['id'],
+                            {'name': target_vol_name})
+                    finally:
+                        self._logout(client)
+
+        updates = {'display_name': display_name}
+
+        LOG.info(_LI("Virtual volume %(disp)s '%(new)s' is "
+                     "now being managed."),
+                 {'disp': display_name, 'new': new_vol_name})
+
+        # Return display name to update the name displayed in the GUI and
+        # any model updates from retype.
+        return updates
+
+    def manage_existing_get_size(self, volume, existing_ref):
+        """Return size of volume to be managed by manage_existing.
+
+        existing_ref is a dictionary of the form:
+        {'source-name': <name of the virtual volume>}
+        """
+        # Check API version.
+        self._check_api_version()
+
+        target_vol_name = self._get_existing_volume_ref_name(existing_ref)
+
+        # Make sure the reference is not in use.
+        if re.match('volume-*|snapshot-*', target_vol_name):
+            reason = _("Reference must be the volume name of an unmanaged "
+                       "virtual volume.")
+            raise exception.ManageExistingInvalidReference(
+                existing_ref=target_vol_name,
+                reason=reason)
+
+        # Check for the existence of the virtual volume.
+        client = self._login()
+        try:
+            volume_info = client.getVolumeByName(target_vol_name)
+        except hpexceptions.HTTPNotFound:
+            err = (_("Virtual volume '%s' doesn't exist on array.") %
+                   target_vol_name)
+            LOG.error(err)
+            raise exception.InvalidInput(reason=err)
+        finally:
+            self._logout(client)
+
+        return int(math.ceil(float(volume_info['size']) / units.Gi))
+
+    def unmanage(self, volume):
+        """Removes the specified volume from Cinder management."""
+        # Check API version.
+        self._check_api_version()
+
+        # Rename the volume's name to unm-* format so that it can be
+        # easily found later.
+        client = self._login()
+        try:
+            volume_info = client.getVolumeByName(volume['name'])
+            new_vol_name = 'unm-' + six.text_type(volume['id'])
+            options = {'name': new_vol_name}
+            client.modifyVolume(volume_info['id'], options)
+        finally:
+            self._logout(client)
+
+        LOG.info(_LI("Virtual volume %(disp)s '%(vol)s' is no longer managed. "
+                     "Volume renamed to '%(new)s'."),
+                 {'disp': volume['display_name'],
+                  'vol': volume['name'],
+                  'new': new_vol_name})
+
+    def _get_existing_volume_ref_name(self, existing_ref):
+        """Returns the volume name of an existing reference.
+
+        Checks if an existing volume reference has a source-name element.
+        If source-name is not present an error will be thrown.
+        """
+        if 'source-name' not in existing_ref:
+            reason = _("Reference must contain source-name.")
+            raise exception.ManageExistingInvalidReference(
+                existing_ref=existing_ref,
+                reason=reason)
+
+        return existing_ref['source-name']
+
+    def _check_api_version(self):
+        """Checks that the API version is correct."""
+        if (self.api_version < MIN_API_VERSION):
+            ex_msg = (_('Invalid HPLeftHand API version found: %(found)s. '
+                        'Version %(minimum)s or greater required for '
+                        'manage/unmanage support.')
+                      % {'found': self.api_version,
+                         'minimum': MIN_API_VERSION})
+            LOG.error(ex_msg)
+            raise exception.InvalidInput(reason=ex_msg)
+
+    def _get_volume_type(self, type_id):
+        ctxt = context.get_admin_context()
+        return volume_types.get_volume_type(ctxt, type_id)
