@@ -32,7 +32,6 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import units
 import six
-from six.moves import range
 
 from cinder.brick.local_dev import lvm
 from cinder import exception
@@ -85,12 +84,12 @@ class retry(object):
             sleep_time = self._sleep_factor
             exc_info = None
 
-            for attempt in range(self._count):
+            for attempt in xrange(self._count):
                 if attempt != 0:
                     LOG.warning(_LW('Retrying failed call to %(func)s, '
-                                    'attempt %(attempt)i.'),
-                                {'func': func_name,
-                                 'attempt': attempt})
+                                    'attempt %(attempt)i.')
+                                % {'func': func_name,
+                                   'attempt': attempt})
                 try:
                     return fun(*args, **kwargs)
                 except self._exceptions:
@@ -128,9 +127,9 @@ class LVM(lvm.LVM):
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
             LOG.exception(_LE('Error activating Volume Group'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.error(_LE('Cmd     :%s') % err.cmd)
+            LOG.error(_LE('StdOut  :%s') % err.stdout)
+            LOG.error(_LE('StdErr  :%s') % err.stderr)
             raise
 
     def deactivate_vg(self):
@@ -148,9 +147,9 @@ class LVM(lvm.LVM):
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
             LOG.exception(_LE('Error deactivating Volume Group'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.error(_LE('Cmd     :%s') % err.cmd)
+            LOG.error(_LE('StdOut  :%s') % err.stdout)
+            LOG.error(_LE('StdErr  :%s') % err.stderr)
             raise
 
     def destroy_vg(self):
@@ -166,9 +165,9 @@ class LVM(lvm.LVM):
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
             LOG.exception(_LE('Error destroying Volume Group'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.error(_LE('Cmd     :%s') % err.cmd)
+            LOG.error(_LE('StdOut  :%s') % err.stdout)
+            LOG.error(_LE('StdErr  :%s') % err.stderr)
             raise
 
     def pv_resize(self, pv_name, new_size_str):
@@ -184,9 +183,9 @@ class LVM(lvm.LVM):
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
             LOG.exception(_LE('Error resizing Physical Volume'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.error(_LE('Cmd     :%s') % err.cmd)
+            LOG.error(_LE('StdOut  :%s') % err.stdout)
+            LOG.error(_LE('StdErr  :%s') % err.stderr)
             raise
 
     def extend_thin_pool(self):
@@ -210,22 +209,22 @@ class LVM(lvm.LVM):
                           run_as_root=True)
         except putils.ProcessExecutionError as err:
             LOG.exception(_LE('Error extending thin provisioning pool'))
-            LOG.error(_LE('Cmd     :%s'), err.cmd)
-            LOG.error(_LE('StdOut  :%s'), err.stdout)
-            LOG.error(_LE('StdErr  :%s'), err.stderr)
+            LOG.error(_LE('Cmd     :%s') % err.cmd)
+            LOG.error(_LE('StdOut  :%s') % err.stdout)
+            LOG.error(_LE('StdErr  :%s') % err.stderr)
             raise
 
 
 @contextlib.contextmanager
 def patched(obj, attr, fun):
-    """Context manager to locally patch a method.
+    '''Context manager to locally patch a method.
 
     Within the managed context, the `attr` method of `obj` will be replaced by
     a method which calls `fun` passing in the original `attr` attribute of
     `obj` as well as any positional and keyword arguments.
 
     At the end of the context, the original method is restored.
-    """
+    '''
 
     orig = getattr(obj, attr)
 
@@ -242,7 +241,7 @@ def patched(obj, attr, fun):
 
 @contextlib.contextmanager
 def handle_process_execution_error(message, info_message, reraise=True):
-    """Consistently handle `putils.ProcessExecutionError` exceptions
+    '''Consistently handle `putils.ProcessExecutionError` exceptions
 
     This context-manager will catch any `putils.ProcessExecutionError`
     exceptions raised in the managed block, and generate logging output
@@ -259,7 +258,7 @@ def handle_process_execution_error(message, info_message, reraise=True):
     object will be raised instead (so you most likely want it to be some
     `Exception`). Any `False` value will result in the exception to be
     swallowed.
-    """
+    '''
 
     try:
         yield
@@ -460,10 +459,8 @@ class SRBDriver(driver.VolumeDriver):
 
     @staticmethod
     def _activate_lv(orig, *args, **kwargs):
-        """Activate lv.
-
-        Use with `patched` to patch `lvm.LVM.activate_lv` to ignore `EEXIST`
-        """
+        '''Use with `patched` to patch `lvm.LVM.activate_lv` to ignore `EEXIST`
+        '''
         try:
             orig(*args, **kwargs)
         except putils.ProcessExecutionError as exc:
@@ -580,8 +577,7 @@ class SRBDriver(driver.VolumeDriver):
     def _attach_file(self, volume):
         name = self._get_volname(volume)
         devname = self._device_name(volume)
-        LOG.debug('Attaching volume %(name)s as %(devname)s',
-                  {'name': name, 'devname': devname})
+        LOG.debug('Attaching volume %s as %s', name, devname)
 
         count = self._get_attached_count(volume)
         if count == 0:
@@ -625,8 +621,8 @@ class SRBDriver(driver.VolumeDriver):
                     self._do_deactivate(volume, vg)
                 except putils.ProcessExecutionError:
                     LOG.warning(_LW('All attempts to recover failed detach '
-                                    'of %(volume)s failed.'),
-                                {'volume': volname})
+                                    'of %(volume)s failed.')
+                                % {'volume': volname})
 
     @lockutils.synchronized('devices', 'cinder-srb-')
     def _detach_file(self, volume):
@@ -638,8 +634,9 @@ class SRBDriver(driver.VolumeDriver):
         count = self._get_attached_count(volume)
         if count > 1:
             LOG.info(_LI('Reference count of %(volume)s is %(count)d, '
-                         'not detaching.'),
-                     {'volume': volume['name'], 'count': count})
+                         'not detaching.')
+                     % {'volume': volume['name'],
+                        'count': count})
             return
 
         message = (_('Could not detach volume %(vol)s from device %(dev)s.')
@@ -652,15 +649,18 @@ class SRBDriver(driver.VolumeDriver):
                 if vg is not None:
                     self._do_deactivate(volume, vg)
             except putils.ProcessExecutionError:
-                LOG.error(_LE('Could not deactivate volume group %s'),
-                          self._get_volname(volume))
+                msg = _LE('Could not deactivate volume groupe %s')\
+                    % (self._get_volname(volume))
+                LOG.error(msg)
                 raise
 
             try:
                 self._do_detach(volume, vg=vg)
             except putils.ProcessExecutionError:
-                LOG.error(_LE('Could not detach volume %(vol)s from device '
-                              '%(dev)s.'), {'vol': name, 'dev': devname})
+                msg = _LE('Could not detach volume '
+                          '%(vol)s from device %(dev)s.') \
+                    % {'vol': name, 'dev': devname}
+                LOG.error(msg)
                 raise
 
             self._decrement_attached_count(volume)
@@ -748,8 +748,8 @@ class SRBDriver(driver.VolumeDriver):
                 self._destroy_lvm(volume)
             self._detach_file(volume)
 
-        LOG.debug('Deleting volume %(volume_name)s, attached=%(attached)s',
-                  {'volume_name': volume['name'], 'attached': attached})
+        LOG.debug('Deleting volume %s, attached=%s',
+                  volume['name'], attached)
 
         self._destroy_file(volume)
 
@@ -836,6 +836,11 @@ class SRBISCSIDriver(SRBDriver, driver.ISCSIDriver):
             self.configuration.safe_get('volume_backend_name') or 'SRB_iSCSI'
         self.protocol = 'iSCSI'
 
+    def set_execute(self, execute):
+        super(SRBISCSIDriver, self).set_execute(execute)
+        if self.target_driver is not None:
+            self.target_driver.set_execute(execute)
+
     def ensure_export(self, context, volume):
         device_path = self._mapper_path(volume)
 
@@ -845,7 +850,7 @@ class SRBISCSIDriver(SRBDriver, driver.ISCSIDriver):
         if model_update:
             self.db.volume_update(context, volume['id'], model_update)
 
-    def create_export(self, context, volume, connector):
+    def create_export(self, context, volume):
         """Creates an export for a logical volume."""
         self._attach_file(volume)
         vg = self._get_lvm_vg(volume)
