@@ -33,12 +33,11 @@ import time
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_service import loopingcall
 from oslo_utils import importutils
-import six
 
 from cinder import exception
 from cinder.i18n import _, _LE, _LW, _LI
+from cinder.openstack.common import loopingcall
 from cinder import utils
 
 LOG = logging.getLogger(__name__)
@@ -48,6 +47,7 @@ if vmemclient:
     LOG.info(_LI("Running with vmemclient version: %s."),
              vmemclient.__version__)
 
+# version vmos versions V6.3.0.4 or newer
 VMOS_SUPPORTED_VERSION_PATTERNS = ['V6.3.0.[4-9]', 'V6.3.[1-9].?[0-9]?']
 
 violin_opts = [
@@ -74,10 +74,9 @@ class V6000Common(object):
 
     Version history:
         1.0 - Initial driver
-        1.0.1 - Fixes polling for export completion
     """
 
-    VERSION = '1.0.1'
+    VERSION = '1.0'
 
     def __init__(self, config):
         self.vip = None
@@ -169,7 +168,7 @@ class V6000Common(object):
         """
         lun_type = '0'
 
-        LOG.debug("Creating LUN %(name)s, %(size)s GB.",
+        LOG.debug("Creating LUN %(name)s, %(size)s GB." %
                   {'name': volume['name'], 'size': volume['size']})
 
         if self.config.san_thin_provision:
@@ -189,7 +188,7 @@ class V6000Common(object):
             LOG.debug("Lun %s already exists, continuing.", volume['id'])
 
         except Exception:
-            LOG.warning(_LW("Lun create for %s failed!"), volume['id'])
+            LOG.warn(_LW("Lun create for %s failed!"), volume['id'])
             raise
 
     @utils.synchronized('vmem-lun')
@@ -214,8 +213,8 @@ class V6000Common(object):
             LOG.debug("Lun %s already deleted, continuing.", volume['id'])
 
         except exception.ViolinBackendErrExists:
-            LOG.warning(_LW("Lun %s has dependent snapshots, skipping."),
-                        volume['id'])
+            LOG.warn(_LW("Lun %s has dependent snapshots, skipping."),
+                     volume['id'])
             raise exception.VolumeIsBusy(volume_name=volume['id'])
 
         except Exception:
@@ -233,7 +232,7 @@ class V6000Common(object):
             volume   -- volume object provided by the Manager
             new_size -- new (increased) size in GB to be applied
         """
-        LOG.debug("Extending lun %(id)s, from %(size)s to %(new_size)s GB.",
+        LOG.debug("Extending lun %(id)s, from %(size)s to %(new_size)s GB." %
                   {'id': volume['id'], 'size': volume['size'],
                    'new_size': new_size})
 
@@ -376,7 +375,7 @@ class V6000Common(object):
         start = time.time()
         done = False
 
-        if isinstance(success_msgs, six.string_types):
+        if isinstance(success_msgs, basestring):
             success_msgs = [success_msgs]
 
         while not done:
@@ -436,7 +435,7 @@ class V6000Common(object):
         request_needed = True
         verify_needed = True
 
-        if isinstance(request_success_msgs, six.string_types):
+        if isinstance(request_success_msgs, basestring):
             request_success_msgs = [request_success_msgs]
 
         rargs = rargs if rargs else []
@@ -500,9 +499,9 @@ class V6000Common(object):
 
         return igroup_name
 
-    def _wait_for_export_state(self, volume_name, snapshot_name=None,
-                               state=False):
-        """Polls backend to verify volume's export state.
+    def _wait_for_export_config(self, volume_name, snapshot_name=None,
+                                state=False):
+        """Polls backend to verify volume's export configuration.
 
         XG sets/queries following a request to create or delete a lun
         export may fail on the backend if vshared is still processing
@@ -524,41 +523,27 @@ class V6000Common(object):
             (depending on 'state' param)
         """
         if not snapshot_name:
-            bn = "/vshare/state/local/container/%s/lun/%s/usn_id" \
+            bn = "/vshare/config/export/container/%s/lun/%s" \
                 % (self.container, volume_name)
         else:
-            bn = "/vshare/state/snapshot/container/%s/lun/%s/snap/%s/usn_id" \
+            bn = "/vshare/config/export/snapshot/container/%s/lun/%s/snap/%s" \
                 % (self.container, volume_name, snapshot_name)
 
         def _loop_func(state):
             status = [False, False]
             mg_conns = [self.mga, self.mgb]
 
-            LOG.debug("Entering _wait_for_export_state loop: state=%s.",
+            LOG.debug("Entering _wait_for_export_config loop: state=%s.",
                       state)
 
-            # TODO(rlucio): May need to handle situations where export
-            # fails, i.e., HBAs go offline and the array is in
-            # degraded mode.
-            #
-            for node_id in range(2):
+            for node_id in xrange(2):
                 resp = mg_conns[node_id].basic.get_node_values(bn)
-
-                if state:
-                    # Verify export was added.  Validates when the usn_id is
-                    # altered to a non-default binding string.
-                    #
-                    if resp[bn] != "(not exported)":
-                        status[node_id] = True
-                else:
-                    # Verify export was removed.  Validates when the usn_id is
-                    # reset to the default binding string.
-                    #
-                    if resp[bn] == "(not exported)":
-                        status[node_id] = True
+                if state and len(resp.keys()):
+                    status[node_id] = True
+                elif (not state) and (not len(resp.keys())):
+                    status[node_id] = True
 
             if status[0] and status[1]:
-                LOG.debug("_wait_for_export_state loopingcall complete.")
                 raise loopingcall.LoopingCallDone(retvalue=True)
 
         timer = loopingcall.FixedIntervalLoopingCall(_loop_func, state)
