@@ -21,8 +21,9 @@ import StringIO
 import time
 import urllib2
 
-from cinder.i18n import _
-from cinder.openstack.common import log
+from oslo_log import log
+
+from cinder.i18n import _LE, _LI
 
 LOG = log.getLogger(__name__)
 
@@ -176,7 +177,7 @@ class RestClientURL(object):
                 self.headers['x-auth-session'] = \
                     result.get_header('x-auth-session')
                 self.do_logout = True
-                LOG.info(_('ZFSSA version: %s') %
+                LOG.info(_LI('ZFSSA version: %s') %
                          result.get_header('x-zfssa-version'))
 
             elif result.status == httplib.NOT_FOUND:
@@ -242,7 +243,7 @@ class RestClientURL(object):
     def request(self, path, request, body=None, **kwargs):
         """Make an HTTP request and return the results
 
-        :param path: Path used with the initiazed URL to make a request
+        :param path: Path used with the initialized URL to make a request
         :param request: HTTP request type (GET, POST, PUT, DELETE)
         :param body: HTTP body of request
         :key accept: Set HTTP 'Accept' header with this value
@@ -279,21 +280,21 @@ class RestClientURL(object):
                 if err.code == httplib.NOT_FOUND:
                     LOG.debug('REST Not Found: %s' % err.code)
                 else:
-                    LOG.error(_('REST Not Available: %s') % err.code)
+                    LOG.error(_LE('REST Not Available: %s') % err.code)
 
                 if err.code == httplib.SERVICE_UNAVAILABLE and \
                    retry < maxreqretries:
                     retry += 1
                     time.sleep(1)
-                    LOG.error(_('Server Busy retry request: %s') % retry)
+                    LOG.error(_LE('Server Busy retry request: %s') % retry)
                     continue
                 if (err.code == httplib.UNAUTHORIZED or
                     err.code == httplib.INTERNAL_SERVER_ERROR) and \
                    '/access/v1' not in zfssaurl:
                     try:
-                        LOG.error(_('Authorizing request: '
-                                    '%(zfssaurl)s'
-                                    'retry: %(retry)d .')
+                        LOG.error(_LE('Authorizing request: '
+                                      '%(zfssaurl)s'
+                                      'retry: %(retry)d .')
                                   % {'zfssaurl': zfssaurl,
                                      'retry': retry})
                         self._authorize()
@@ -308,7 +309,7 @@ class RestClientURL(object):
                 return RestResult(err=err)
 
             except urllib2.URLError as err:
-                LOG.error(_('URLError: %s') % err.reason)
+                LOG.error(_LE('URLError: %s') % err.reason)
                 raise RestClientError(-1, name="ERR_URLError",
                                       message=err.reason)
 
