@@ -14,10 +14,10 @@ from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 
 from cinder import exception
-from cinder.i18n import _
-import cinder.privsep.targets.scst
+from cinder import utils
+from cinder.i18n import _, _LE
 from cinder.volume.targets import iscsi
-from cinder.volume import volume_utils
+from cinder.volume import utils as vutils
 
 LOG = logging.getLogger(__name__)
 
@@ -28,7 +28,7 @@ class SCSTAdm(iscsi.ISCSITarget):
         super(SCSTAdm, self).__init__(*args, **kwargs)
         self.volumes_dir = self.configuration.safe_get('volumes_dir')
         self.iscsi_target_prefix = self.configuration.safe_get(
-            'target_prefix')
+            'iscsi_target_prefix')
         self.target_name = self.configuration.safe_get('scst_target_iqn_name')
         self.target_driver = self.configuration.safe_get('scst_target_driver')
         self.chap_username = self.configuration.safe_get('chap_username')
@@ -37,7 +37,7 @@ class SCSTAdm(iscsi.ISCSITarget):
         self.remove_initiator_iqn = None
 
     def scst_execute(self, *args):
-        return cinder.privsep.targets.scst.run_scstadmin(*args)
+        return utils.execute('scstadmin', *args, run_as_root=True)
 
     def validate_connector(self, connector):
         # iSCSI drivers require the initiator information
@@ -118,7 +118,8 @@ class SCSTAdm(iscsi.ISCSITarget):
                                             'enabled=1')
             LOG.debug('StdOut from set driver attribute: %s', out)
         except putils.ProcessExecutionError as e:
-            LOG.error("Failed to set attribute for enable target driver %s", e)
+            LOG.error(_LE("Failed to set attribute for enable target driver "
+                          "%s"), e)
             raise exception.ISCSITargetHelperCommandFailed(
                 error_message="Failed to enable SCST Target driver.")
 
@@ -128,18 +129,18 @@ class SCSTAdm(iscsi.ISCSITarget):
                                                 '-driver', self.target_driver)
                 LOG.debug("StdOut from scstadmin create target: %s", out)
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to create iscsi target for volume "
-                          "id:%(vol_id)s: %(e)s", {'vol_id': name, 'e': e})
+                LOG.error(_LE("Failed to create iscsi target for volume "
+                          "id:%(vol_id)s: %(e)s"), {'vol_id': name, 'e': e})
                 raise exception.ISCSITargetCreateFailed(volume_id=vol_name)
             try:
                 (out, _err) = self.scst_execute('-enable_target', name,
                                                 '-driver', self.target_driver)
                 LOG.debug("StdOut from scstadmin enable target: %s", out)
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to set 'enable' attribute for "
-                          "SCST target %s", e)
+                LOG.error(_LE("Failed to set 'enable' attribute for "
+                              "SCST target %s"), e)
                 raise exception.ISCSITargetHelperCommandFailed(
-                    error_message="Failed to enable SCST Target.")
+                    error_mesage="Failed to enable SCST Target.")
             if chap_auth and self.target_name:
                 try:
                     chap_string = self._iscsi_authentication('IncomingUser=',
@@ -157,7 +158,7 @@ class SCSTAdm(iscsi.ISCSITarget):
                             "SCST target.")
                     LOG.exception(msg)
                     raise exception.ISCSITargetHelperCommandFailed(
-                        error_message=msg)
+                        error_mesage=msg)
 
         if self.target_name:
             if self._get_group() is None:
@@ -168,7 +169,8 @@ class SCSTAdm(iscsi.ISCSITarget):
                                                     '-target', name)
                     LOG.debug("StdOut from scstadmin create group: %s", out)
                 except putils.ProcessExecutionError as e:
-                    LOG.error("Failed to create group to SCST target %s", e)
+                    LOG.error(_LE("Failed to create group to SCST target "
+                                  "%s"), e)
                     raise exception.ISCSITargetHelperCommandFailed(
                         error_message="Failed to create group to SCST target.")
             try:
@@ -179,8 +181,8 @@ class SCSTAdm(iscsi.ISCSITarget):
                                                 '-group', scst_group)
                 LOG.debug("StdOut from scstadmin add initiator: %s", out)
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to add initiator to group "
-                          " for SCST target %s", e)
+                LOG.error(_LE("Failed to add initiator to group "
+                          " for SCST target %s"), e)
                 raise exception.ISCSITargetHelperCommandFailed(
                     error_message="Failed to add Initiator to group for "
                                   "SCST target.")
@@ -196,7 +198,7 @@ class SCSTAdm(iscsi.ISCSITarget):
                               '-handler', 'vdisk_fileio',
                               '-attributes', 'filename=%s' % path)
         except putils.ProcessExecutionError as e:
-            LOG.error("Failed to add device to handler %s", e)
+            LOG.error(_LE("Failed to add device to handler %s"), e)
             raise exception.ISCSITargetHelperCommandFailed(
                 error_message="Failed to add device to SCST handler.")
 
@@ -213,8 +215,8 @@ class SCSTAdm(iscsi.ISCSITarget):
                                   '-target', name,
                                   '-device', disk_id)
         except putils.ProcessExecutionError as e:
-            LOG.error("Failed to add lun to SCST target "
-                      "id:%(vol_id)s: %(e)s", {'vol_id': name, 'e': e})
+            LOG.error(_LE("Failed to add lun to SCST target "
+                      "id:%(vol_id)s: %(e)s"), {'vol_id': name, 'e': e})
             raise exception.ISCSITargetHelperCommandFailed(
                 error_message="Failed to add LUN to SCST Target for "
                               "volume " + vol_name)
@@ -224,33 +226,23 @@ class SCSTAdm(iscsi.ISCSITarget):
         try:
             self.scst_execute('-write_config', '/etc/scst.conf')
         except putils.ProcessExecutionError as e:
-            LOG.error("Failed to write in /etc/scst.conf.")
+            LOG.error(_LE("Failed to write in /etc/scst.conf."))
             raise exception.ISCSITargetHelperCommandFailed(
                 error_message="Failed to write in /etc/scst.conf.")
 
         return tid
 
     def _iscsi_location(self, ip, target, iqn, lun=None):
-        return "%s:%s,%s %s %s" % (volume_utils.sanitize_host(ip),
-                                   self.configuration.target_port,
+        return "%s:%s,%s %s %s" % (ip, self.configuration.iscsi_port,
                                    target, iqn, lun)
 
-    def _get_iscsi_name(self, volume):
-        if self.target_name is None:
-            return "%s%s" % (self.configuration.target_prefix,
-                             volume['name'])
-        else:
-            return self.target_name
-
     def _get_iscsi_target(self, context, vol_id):
         # FIXME(jdg): Need to implement abc method
         pass
 
-    def _get_target_chap_auth(self, context, volume):
+    def _get_target_chap_auth(self, context, iscsi_name):
         # FIXME(jdg): Need to implement abc method
 
-        iscsi_name = self._get_iscsi_name(volume)
-
         if self._get_target(iscsi_name) is None:
             return None
         (out, _err) = self.scst_execute('-list_tgt_attr', iscsi_name,
@@ -263,19 +255,23 @@ class SCSTAdm(iscsi.ISCSITarget):
         out = out.split("\n")[2]
         if "IncomingUser" in out:
             out = out.split(" ")
-            out = [a for a in out if a != ""]
+            out = filter(lambda a: a != "", out)
             return (out[1], out[2])
         else:
             return None
 
     def ensure_export(self, context, volume, volume_path):
         iscsi_target, lun = self._get_target_and_lun(context, volume)
-        iscsi_name = self._get_iscsi_name(volume)
+        if self.target_name is None:
+            iscsi_name = "%s%s" % (self.configuration.iscsi_target_prefix,
+                                   volume['name'])
+        else:
+            iscsi_name = self.target_name
 
         if self.chap_username and self.chap_password:
             chap_auth = (self.chap_username, self.chap_password)
         else:
-            chap_auth = self._get_target_chap_auth(context, volume)
+            chap_auth = self._get_target_chap_auth(context, iscsi_name)
 
         self.create_iscsi_target(iscsi_name, volume['id'], iscsi_target,
                                  lun, volume_path, chap_auth)
@@ -283,21 +279,25 @@ class SCSTAdm(iscsi.ISCSITarget):
     def create_export(self, context, volume, volume_path):
         """Creates an export for a logical volume."""
         iscsi_target, lun = self._get_target_and_lun(context, volume)
-        iscsi_name = self._get_iscsi_name(volume)
+        if self.target_name is None:
+            iscsi_name = "%s%s" % (self.configuration.iscsi_target_prefix,
+                                   volume['name'])
+        else:
+            iscsi_name = self.target_name
 
         if self.chap_username and self.chap_password:
             chap_auth = (self.chap_username, self.chap_password)
         else:
-            chap_auth = self._get_target_chap_auth(context, volume)
+            chap_auth = self._get_target_chap_auth(context, iscsi_name)
             if not chap_auth:
-                chap_auth = (volume_utils.generate_username(),
-                             volume_utils.generate_password())
+                chap_auth = (vutils.generate_username(),
+                             vutils.generate_password())
         tid = self.create_iscsi_target(iscsi_name, volume['id'], iscsi_target,
                                        lun, volume_path, chap_auth)
 
         data = {}
         data['location'] = self._iscsi_location(
-            self.configuration.target_ip_address, tid, iscsi_name, lun)
+            self.configuration.iscsi_ip_address, tid, iscsi_name, lun)
         LOG.debug('Set provider_location to: %s', data['location'])
         data['auth'] = self._iscsi_authentication(
             'CHAP', *chap_auth)
@@ -311,8 +311,8 @@ class SCSTAdm(iscsi.ISCSITarget):
             self.show_target(iscsi_target, iqn)
 
         except Exception:
-            LOG.error("Skipping remove_export. No iscsi_target is "
-                      "presently exported for volume: %s", volume['id'])
+            LOG.error(_LE("Skipping remove_export. No iscsi_target is"
+                          "presently exported for volume: %s"), volume['id'])
             return
         vol = self.db.volume_get(context, volume['id'])
         lun = "".join(vol['provider_location'].split(" ")[-1:])
@@ -334,15 +334,15 @@ class SCSTAdm(iscsi.ISCSITarget):
                                   '-rem_target', iqn,
                                   '-driver', 'iscsi')
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to remove iscsi target for volume "
-                          "id:%(vol_id)s: %(e)s", {'vol_id': vol_id, 'e': e})
+                LOG.error(_LE("Failed to remove iscsi target for volume "
+                          "id:%(vol_id)s: %(e)s"), {'vol_id': vol_id, 'e': e})
                 raise exception.ISCSITargetRemoveFailed(volume_id=vol_id)
             try:
                 self.scst_execute('-noprompt',
                                   '-close_dev', "disk%s" % tid,
                                   '-handler', 'vdisk_fileio')
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to close disk device %s", e)
+                LOG.error(_LE("Failed to close disk device %s"), e)
                 raise exception.ISCSITargetHelperCommandFailed(
                     error_message="Failed to close disk device for "
                                   "SCST handler.")
@@ -353,8 +353,8 @@ class SCSTAdm(iscsi.ISCSITarget):
                                       '-rem_target', iqn,
                                       '-driver', self.target_driver)
                 except putils.ProcessExecutionError as e:
-                    LOG.error("Failed to remove iscsi target for "
-                              "volume id:%(vol_id)s: %(e)s",
+                    LOG.error(_LE("Failed to remove iscsi target for "
+                                  "volume id:%(vol_id)s: %(e)s"),
                               {'vol_id': vol_id, 'e': e})
                     raise exception.ISCSITargetRemoveFailed(volume_id=vol_id)
         else:
@@ -368,7 +368,7 @@ class SCSTAdm(iscsi.ISCSITarget):
                                   '-target', iqn, '-group',
                                   scst_group)
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to remove LUN %s", e)
+                LOG.error(_LE("Failed to remove LUN %s"), e)
                 raise exception.ISCSITargetHelperCommandFailed(
                     error_message="Failed to remove LUN for SCST Target.")
 
@@ -377,7 +377,7 @@ class SCSTAdm(iscsi.ISCSITarget):
                                   '-close_dev', disk_id,
                                   '-handler', 'vdisk_fileio')
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to close disk device %s", e)
+                LOG.error(_LE("Failed to close disk device %s"), e)
                 raise exception.ISCSITargetHelperCommandFailed(
                     error_message="Failed to close disk device for "
                                   "SCST handler.")
@@ -393,3 +393,10 @@ class SCSTAdm(iscsi.ISCSITarget):
         if tid is None:
             raise exception.ISCSITargetHelperCommandFailed(
                 error_message="Target not found")
+
+    def initialize_connection(self, volume, connector):
+        iscsi_properties = self._get_iscsi_properties(volume)
+        return {
+            'driver_volume_type': 'iscsi',
+            'data': iscsi_properties
+        }
