@@ -11,15 +11,15 @@
 #    under the License.
 
 import os
-import textwrap
+import re
 import time
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
-from oslo_utils import fileutils
 
 from cinder import exception
-import cinder.privsep.targets.tgt
+from cinder.openstack.common import fileutils
+from cinder.i18n import _LI, _LW, _LE
 from cinder import utils
 from cinder.volume.targets import iscsi
 
@@ -35,18 +35,27 @@ class TgtAdm(iscsi.ISCSITarget):
     etc.
     """
 
-    VOLUME_CONF = textwrap.dedent("""
-                <target %(name)s>
-                    backing-store %(path)s
-                    driver %(driver)s
-                    %(chap_auth)s
-                    %(target_flags)s
-                    write-cache %(write_cache)s
+    VOLUME_CONF = """
+                <target %s>
+                    backing-store %s
+                    driver %s
+                    write-cache %s
                 </target>
-                  """)
+                  """
+    VOLUME_CONF_WITH_CHAP_AUTH = """
+                                <target %s>
+                                    backing-store %s
+                                    driver %s
+                                    %s
+                                    write-cache %s
+                                </target>
+                                 """
+
+    def __init__(self, *args, **kwargs):
+        super(TgtAdm, self).__init__(*args, **kwargs)
 
     def _get_target(self, iqn):
-        (out, err) = cinder.privsep.targets.tgt.tgtadmin_show()
+        (out, err) = utils.execute('tgt-admin', '--show', run_as_root=True)
         lines = out.split('\n')
         for line in lines:
             if iqn in line:
@@ -61,7 +70,7 @@ class TgtAdm(iscsi.ISCSITarget):
         capture = False
         target_info = []
 
-        (out, err) = cinder.privsep.targets.tgt.tgtadmin_show()
+        (out, err) = utils.execute('tgt-admin', '--show', run_as_root=True)
         lines = out.split('\n')
 
         for line in lines:
@@ -78,7 +87,7 @@ class TgtAdm(iscsi.ISCSITarget):
         return backing_lun
 
     def _recreate_backing_lun(self, iqn, tid, name, path):
-        LOG.warning('Attempting recreate of backing lun...')
+        LOG.warning(_LW('Attempting recreate of backing lun...'))
 
         # Since we think the most common case of this is a dev busy
         # (create vol from snapshot) we're going to add a sleep here
@@ -89,11 +98,15 @@ class TgtAdm(iscsi.ISCSITarget):
 
         (out, err) = (None, None)
         try:
-            (out, err) = cinder.privsep.targets.tgt.tgtadm_create(tid, path)
+            (out, err) = utils.execute('tgtadm', '--lld', 'iscsi',
+                                       '--op', 'new', '--mode',
+                                       'logicalunit', '--tid',
+                                       tid, '--lun', '1', '-b',
+                                       path, run_as_root=True)
         except putils.ProcessExecutionError as e:
-            LOG.error("Failed recovery attempt to create "
-                      "iscsi backing lun for Volume "
-                      "ID:%(vol_id)s: %(e)s",
+            LOG.error(_LE("Failed recovery attempt to create "
+                          "iscsi backing lun for Volume "
+                          "ID:%(vol_id)s: %(e)s"),
                       {'vol_id': name, 'e': e})
         finally:
             LOG.debug('StdOut from recreate backing lun: %s', out)
@@ -107,13 +120,41 @@ class TgtAdm(iscsi.ISCSITarget):
         iscsi_target = 0  # NOTE(jdg): Not used by tgtadm
         return iscsi_target, lun
 
+    def _get_target_chap_auth(self, context, iscsi_name):
+        """Get the current chap auth username and password."""
+        volumes_dir = self.volumes_dir
+        vol_id = iscsi_name.split(':')[1]
+        volume_path = os.path.join(volumes_dir, vol_id)
+
+        try:
+            with open(volume_path, 'r') as f:
+                volume_conf = f.read()
+        except IOError as e_fnf:
+            LOG.debug('Failed to open config for Volume %(vol_id)s: %(e)s',
+                      {'vol_id': vol_id, 'e': e_fnf})
+            # tgt is linux specific
+            if e_fnf.errno == 2:
+                return None
+            else:
+                raise
+        except Exception as e_vol:
+            LOG.error(_LE('Failed to open config for %(vol_id)s: %(e)s'),
+                      {'vol_id': vol_id, 'e': e_vol})
+            raise
+
+        m = re.search('incominguser (\w+) (\w+)', volume_conf)
+        if m:
+            return (m.group(1), m.group(2))
+        LOG.debug('Failed to find CHAP auth from config for %s', vol_id)
+        return None
+
     @utils.retry(putils.ProcessExecutionError)
-    def _do_tgt_update(self, name, force=False):
-        (out, err) = cinder.privsep.targets.tgt.tgtadmin_update(name, force)
-        LOG.debug("StdOut from tgt-admin --update: %s", out)
-        LOG.debug("StdErr from tgt-admin --update: %s", err)
+    def _do_tgt_update(self, name):
+            (out, err) = utils.execute('tgt-admin', '--update', name,
+                                       run_as_root=True)
+            LOG.debug("StdOut from tgt-admin --update: %s", out)
+            LOG.debug("StdErr from tgt-admin --update: %s", err)
 
-    @utils.retry(exception.NotFound)
     def create_iscsi_target(self, name, tid, lun, path,
                             chap_auth=None, **kwargs):
 
@@ -123,35 +164,38 @@ class TgtAdm(iscsi.ISCSITarget):
         # NOTE(jdg): Remove this when we get to the bottom of bug: #1398078
         # for now, since we intermittently hit target already exists we're
         # adding some debug info to try and pinpoint what's going on
-        (out, err) = cinder.privsep.targets.tgt.tgtadm_show()
+        (out, err) = utils.execute('tgtadm',
+                                   '--lld',
+                                   'iscsi',
+                                   '--op',
+                                   'show',
+                                   '--mode',
+                                   'target',
+                                   run_as_root=True)
         LOG.debug("Targets prior to update: %s", out)
         fileutils.ensure_tree(self.volumes_dir)
 
         vol_id = name.split(':')[1]
         write_cache = self.configuration.get('iscsi_write_cache', 'on')
         driver = self.iscsi_protocol
-        chap_str = ''
 
-        if chap_auth is not None:
+        if chap_auth is None:
+            volume_conf = self.VOLUME_CONF % (name, path, driver, write_cache)
+        else:
             chap_str = 'incominguser %s %s' % chap_auth
-
-        target_flags = self.configuration.get('iscsi_target_flags', '')
-        if target_flags:
-            target_flags = 'bsoflags ' + target_flags
-
-        volume_conf = self.VOLUME_CONF % {
-            'name': name, 'path': path, 'driver': driver,
-            'chap_auth': chap_str, 'target_flags': target_flags,
-            'write_cache': write_cache}
-
+            volume_conf = self.VOLUME_CONF_WITH_CHAP_AUTH % (name, path,
+                                                             driver, chap_str,
+                                                             write_cache)
         LOG.debug('Creating iscsi_target for Volume ID: %s', vol_id)
         volumes_dir = self.volumes_dir
         volume_path = os.path.join(volumes_dir, vol_id)
 
         if os.path.exists(volume_path):
-            LOG.debug(('Persistence file already exists for volume, '
-                       'found file at: %s'), volume_path)
-        utils.robust_file_write(volumes_dir, vol_id, volume_conf)
+            LOG.warning(_LW('Persistence file already exists for volume, '
+                            'found file at: %s'), volume_path)
+        f = open(volume_path, 'w+')
+        f.write(volume_conf)
+        f.close()
         LOG.debug(('Created volume path %(vp)s,\n'
                    'content: %(vc)s'),
                   {'vp': volume_path, 'vc': volume_conf})
@@ -160,7 +204,7 @@ class TgtAdm(iscsi.ISCSITarget):
         old_name = kwargs.get('old_name', None)
         if old_name is not None:
             LOG.debug('Detected old persistence file for volume '
-                      '%(vol)s at %(old_name)s',
+                      '%{vol}s at %{old_name}s',
                       {'vol': vol_id, 'old_name': old_name})
             old_persist_file = os.path.join(volumes_dir, old_name)
 
@@ -175,13 +219,13 @@ class TgtAdm(iscsi.ISCSITarget):
             if "target already exists" in e.stderr:
                 # Adding the additional Warning message below for a clear
                 # ER marker (Ref bug: #1398078).
-                LOG.warning('Could not create target because '
-                            'it already exists for volume: %s', vol_id)
+                LOG.warning(_LW('Could not create target because '
+                                'it already exists for volume: %s'), vol_id)
                 LOG.debug('Exception was: %s', e)
 
             else:
-                LOG.error("Failed to create iscsi target for Volume "
-                          "ID: %(vol_id)s: %(e)s",
+                LOG.error(_LE("Failed to create iscsi target for Volume "
+                              "ID: %(vol_id)s: %(e)s"),
                           {'vol_id': vol_id, 'e': e})
 
             # Don't forget to remove the persistent file we created
@@ -191,19 +235,24 @@ class TgtAdm(iscsi.ISCSITarget):
         # Grab targets list for debug
         # Consider adding a check for lun 0 and 1 for tgtadm
         # before considering this as valid
-        cinder.privsep.targets.tgt.tgtadm_show()
+        (out, err) = utils.execute('tgtadm',
+                                   '--lld',
+                                   'iscsi',
+                                   '--op',
+                                   'show',
+                                   '--mode',
+                                   'target',
+                                   run_as_root=True)
         LOG.debug("Targets after update: %s", out)
 
         iqn = '%s%s' % (self.iscsi_target_prefix, vol_id)
         tid = self._get_target(iqn)
         if tid is None:
-            LOG.warning("Failed to create iscsi target for Volume "
-                        "ID: %(vol_id)s. It could be caused by problem "
-                        "with concurrency. "
-                        "Also please ensure your tgtd config "
-                        "file contains 'include %(volumes_dir)s/*'",
-                        {'vol_id': vol_id,
-                         'volumes_dir': volumes_dir, })
+            LOG.error(_LE("Failed to create iscsi target for Volume "
+                          "ID: %(vol_id)s. Please ensure your tgtd config "
+                          "file contains 'include %(volumes_dir)s/*'"), {
+                      'vol_id': vol_id,
+                      'volumes_dir': volumes_dir, })
             raise exception.NotFound()
 
         # NOTE(jdg): Sometimes we have some issues with the backing lun
@@ -223,18 +272,27 @@ class TgtAdm(iscsi.ISCSITarget):
                 os.unlink(volume_path)
                 raise exception.ISCSITargetCreateFailed(volume_id=vol_id)
 
-        if old_persist_file is not None:
-            fileutils.delete_if_exists(old_persist_file)
+        if old_persist_file is not None and os.path.exists(old_persist_file):
+            os.unlink(old_persist_file)
 
         return tid
 
+    def initialize_connection(self, volume, connector):
+        iscsi_properties = self._get_iscsi_properties(volume,
+                                                      connector.get(
+                                                          'multipath'))
+        return {
+            'driver_volume_type': self.iscsi_protocol,
+            'data': iscsi_properties
+        }
+
     def remove_iscsi_target(self, tid, lun, vol_id, vol_name, **kwargs):
-        LOG.info('Removing iscsi_target for Volume ID: %s', vol_id)
+        LOG.info(_LI('Removing iscsi_target for Volume ID: %s'), vol_id)
         vol_uuid_file = vol_name
         volume_path = os.path.join(self.volumes_dir, vol_uuid_file)
         if not os.path.exists(volume_path):
-            LOG.warning('Volume path %s does not exist, '
-                        'nothing to remove.', volume_path)
+            LOG.warning(_LW('Volume path %s does not exist, '
+                            'nothing to remove.'), volume_path)
             return
 
         if os.path.isfile(volume_path):
@@ -245,17 +303,21 @@ class TgtAdm(iscsi.ISCSITarget):
         try:
             # NOTE(vish): --force is a workaround for bug:
             #             https://bugs.launchpad.net/cinder/+bug/1159948
-            cinder.privsep.targets.tgt.tgtadmin_delete(iqn, force=True)
+            utils.execute('tgt-admin',
+                          '--force',
+                          '--delete',
+                          iqn,
+                          run_as_root=True)
         except putils.ProcessExecutionError as e:
             non_fatal_errors = ("can't find the target",
                                 "access control rule does not exist")
 
             if any(error in e.stderr for error in non_fatal_errors):
-                LOG.warning("Failed target removal because target or "
-                            "ACL's couldn't be found for iqn: %s.", iqn)
+                LOG.warning(_LW("Failed target removal because target or "
+                                "ACL's couldn't be found for iqn: %s."), iqn)
             else:
-                LOG.error("Failed to remove iscsi target for Volume "
-                          "ID: %(vol_id)s: %(e)s",
+                LOG.error(_LE("Failed to remove iscsi target for Volume "
+                              "ID: %(vol_id)s: %(e)s"),
                           {'vol_id': vol_id, 'e': e})
                 raise exception.ISCSITargetRemoveFailed(volume_id=vol_id)
         # NOTE(jdg): There's a bug in some versions of tgt that
@@ -270,12 +332,15 @@ class TgtAdm(iscsi.ISCSITarget):
         #    https://bugs.launchpad.net/cinder/+bug/1304122
         if self._get_target(iqn):
             try:
-                LOG.warning('Silent failure of target removal '
-                            'detected, retry....')
-                cinder.privsep.targets.tgt.tgtadmin_delete(iqn)
+                LOG.warning(_LW('Silent failure of target removal '
+                                'detected, retry....'))
+                utils.execute('tgt-admin',
+                              '--delete',
+                              iqn,
+                              run_as_root=True)
             except putils.ProcessExecutionError as e:
-                LOG.error("Failed to remove iscsi target for Volume "
-                          "ID: %(vol_id)s: %(e)s",
+                LOG.error(_LE("Failed to remove iscsi target for Volume "
+                              "ID: %(vol_id)s: %(e)s"),
                           {'vol_id': vol_id, 'e': e})
                 raise exception.ISCSITargetRemoveFailed(volume_id=vol_id)
 
