@@ -17,35 +17,33 @@
 Utility functions related to the Zone Manager.
 
 """
+import logging
+
 from oslo_log import log
 
+from cinder.i18n import _LI, _LW
 from cinder.volume import configuration
 from cinder.volume import manager
 from cinder.zonemanager import fc_san_lookup_service
 from cinder.zonemanager import fc_zone_manager
 
 LOG = log.getLogger(__name__)
+LOG.logger.setLevel(logging.DEBUG)
 
 
 def create_zone_manager():
     """If zoning is enabled, build the Zone Manager."""
     config = configuration.Configuration(manager.volume_manager_opts)
-    LOG.debug("Zoning mode: %s.", config.safe_get('zoning_mode'))
+    LOG.debug("Zoning mode: %s", config.safe_get('zoning_mode'))
     if config.safe_get('zoning_mode') == 'fabric':
         LOG.debug("FC Zone Manager enabled.")
         zm = fc_zone_manager.ZoneManager()
-        if zm.initialized:
-            LOG.info("Using FC Zone Manager %(zm_version)s,"
-                     " Driver %(drv_name)s %(drv_version)s.",
-                     {'zm_version': zm.get_version(),
-                      'drv_name': zm.driver.__class__.__name__,
-                      'drv_version': zm.driver.get_version()})
-            return zm
-        else:
-            LOG.debug("FC Zone Manager %(zm_version)s disabled",
-                      {"zm_version": zm.get_version()})
-            return None
-
+        LOG.info(_LI("Using FC Zone Manager %(zm_version)s,"
+                     " Driver %(drv_name)s %(drv_version)s."),
+                 {'zm_version': zm.get_version(),
+                  'drv_name': zm.driver.__class__.__name__,
+                  'drv_version': zm.driver.get_version()})
+        return zm
     else:
         LOG.debug("FC Zone Manager not enabled in cinder.conf.")
         return None
@@ -53,11 +51,11 @@ def create_zone_manager():
 
 def create_lookup_service():
     config = configuration.Configuration(manager.volume_manager_opts)
-    LOG.debug("Zoning mode: %s.", config.safe_get('zoning_mode'))
+    LOG.debug("Zoning mode: %s", config.safe_get('zoning_mode'))
     if config.safe_get('zoning_mode') == 'fabric':
         LOG.debug("FC Lookup Service enabled.")
         lookup = fc_san_lookup_service.FCSanLookupService()
-        LOG.info("Using FC lookup service %s.", lookup.lookup_service)
+        LOG.info(_LI("Using FC lookup service %s"), lookup.lookup_service)
         return lookup
     else:
         LOG.debug("FC Lookup Service not enabled in cinder.conf.")
@@ -73,27 +71,51 @@ def get_formatted_wwn(wwn_str):
                          for i in range(0, len(wwn_str), 2)])).lower()
 
 
-def add_fc_zone(connection_info):
-    """Utility function to add a FC Zone."""
-    if connection_info:
-        vol_type = connection_info.get('driver_volume_type', None)
+def AddFCZone(initialize_connection):
+    """Decorator to add a FC Zone."""
+    def decorator(self, *args, **kwargs):
+        conn_info = initialize_connection(self, *args, **kwargs)
+        if not conn_info:
+            LOG.warn(_LW("Driver didn't return connection info, "
+                         "can't add zone."))
+            return None
+
+        vol_type = conn_info.get('driver_volume_type', None)
         if vol_type == 'fibre_channel':
-            if 'initiator_target_map' in connection_info['data']:
+
+            if 'initiator_target_map' in conn_info['data']:
+                init_target_map = conn_info['data']['initiator_target_map']
                 zm = create_zone_manager()
                 if zm:
-                    LOG.debug("add_fc_zone connection info: %(conninfo)s.",
-                              {'conninfo': connection_info})
-                    zm.add_connection(connection_info)
+                    LOG.debug("Add FC Zone for mapping '%s'.",
+                              init_target_map)
+                    zm.add_connection(init_target_map)
+
+        return conn_info
+
+    return decorator
 
 
-def remove_fc_zone(connection_info):
-    """Utility function for FC drivers to remove zone."""
-    if connection_info:
-        vol_type = connection_info.get('driver_volume_type', None)
+def RemoveFCZone(terminate_connection):
+    """Decorator for FC drivers to remove zone."""
+    def decorator(self, *args, **kwargs):
+        conn_info = terminate_connection(self, *args, **kwargs)
+        if not conn_info:
+            LOG.warn(_LW("Driver didn't return connection info from "
+                         "terminate_connection call."))
+            return None
+
+        vol_type = conn_info.get('driver_volume_type', None)
         if vol_type == 'fibre_channel':
-            if 'initiator_target_map' in connection_info['data']:
+
+            if 'initiator_target_map' in conn_info['data']:
+                init_target_map = conn_info['data']['initiator_target_map']
                 zm = create_zone_manager()
                 if zm:
-                    LOG.debug("remove_fc_zone connection info: %(conninfo)s.",
-                              {'conninfo': connection_info})
-                    zm.delete_connection(connection_info)
+                    LOG.debug("Remove FC Zone for mapping '%s'.",
+                              init_target_map)
+                    zm.delete_connection(init_target_map)
+
+        return conn_info
+
+    return decorator
